VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA34401xDMM"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 1998-2003 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for Agilent 34401x DMM

Private mModel As String
Private mAutoRange As Boolean
Private mRange As String
Private mResolution As String

Private mGPIB As cwGPIB

Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
        .Clear
        
    End With
    
    mModel = QueryData("*IDN?")
    
    Initialize = True
    
ErrHandler:
    If RaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

'range and resolution
Public Property Let AutoRange(ByVal vData As Boolean)
    mAutoRange = vData
End Property

Public Property Get AutoRange() As Boolean
    AutoRange = mAutoRange
End Property

Public Property Let Range(ByVal vData As Single)
    mRange = CStr(vData)
End Property

Public Property Get Range() As Single
    If IsNumeric(mRange) Then
        Range = CSng(mRange)
    Else
        Range = -1
    End If
End Property

Public Property Let Resolution(ByVal vData As Single)
    mResolution = CStr(vData)
End Property

Public Property Get Resolution() As Single
    If IsNumeric(mResolution) Then
        Resolution = CSng(mResolution)
    Else
        Resolution = -1
    End If
End Property

'display, error, msg
Public Property Let DisplayEnabled(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "DISP " & IIf(vData, "ON", "OFF")
    mGPIB.Write sCmd
End Property

Public Property Get DisplayEnabled() As Boolean
    Dim sCmd$
    
    sCmd = "DISP?"
    sCmd = QueryData(sCmd)
    DisplayEnabled = (sCmd = "1")
End Property

Public Property Let DisplayText(ByVal sText As String)
    If sText <> "" Then
        mGPIB.Write "DISP:TEXT " + "'" + sText + "'"
    Else
        mGPIB.Write "DISP:TEXT:CLE"
    End If
End Property

Public Property Get DisplayText() As String
    DisplayText = QueryData("DISP:TEXT?")
End Property

Public Property Get LastError() As String
    LastError = QueryData("SYST:ERR?")
End Property

Public Property Get ModelName() As String
    ModelName = mModel
End Property

'measurement
Public Property Get DCV() As Single
    DCV = VoltageDC
End Property

Public Property Get DCI() As Single
    DCI = CurrentDC
End Property

Public Property Get VoltageDC() As Single
    Dim sCmd$
    
    sCmd = "MEAS:VOLT:DC?" + GetRangeString
    VoltageDC = QueryDataNumeric(sCmd)
End Property

Public Property Get CurrentDC() As Single
    Dim sCmd$
    
    sCmd = "MEAS:CURR:DC?" + GetRangeString
    CurrentDC = QueryDataNumeric(sCmd)
End Property

'private stuff
Private Function GetRangeString() As String
    Dim s$
    
    If mAutoRange Then
        s = " AUTO"
    ElseIf mRange <> "" Then
        s = " " + mRange
        If mResolution <> "" Then
            s = s + "," + mResolution
        End If
    End If
    
    GetRangeString = s
End Function

Private Function QueryDataNumeric(ByVal sCmd As String) As Single
    Dim s$
    
    s = QueryData(sCmd)
    If IsNumeric(s) Then
        QueryDataNumeric = CSng(s)
    Else
        QueryDataNumeric = -9999
    End If
End Function

Private Function QueryData(ByVal sCmd As String) As String
    Dim s$
    
    mGPIB.Write sCmd
    s = mGPIB.Read
    s = Left(s, Len(s) - 1)
    
    QueryData = s
End Function

Private Sub Class_Initialize()
    mRange = ""
    mResolution = ""
    mAutoRange = True
End Sub
