VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA34970"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2001 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for Agilent 34970A DAQ/Switch Unit


Private Const msIOCard$ = "34907A"
Private Const MyName$ = "Agilent 34970A"

Private mGPIB As cwGPIB

'system, check IO card type
Public Property Get IsIOCard(ByVal Slot As Integer) As Boolean
    Dim s$
    
    s = Me.ModuleType(Slot)
    IsIOCard = (InStr(s, msIOCard) <> 0)
End Property

'system, read card information
Public Property Get ModuleType(ByVal Slot As Integer) As String
    Dim sCmd$
    
    sCmd = "SYST:CTYP? " & Slot & "00"
    mGPIB.Write sCmd
    ModuleType = GetGPIBRead
    
End Property

'system, read unit error
Public Property Get Error() As String
    Dim sCmd$
    
    sCmd = "SYST:ERR?"
    mGPIB.Write sCmd
    Error = mGPIB.Read
End Property

'system, reset
Public Sub Reset()
    mGPIB.Write "*RST"
End Sub

'open or close, exclusively, one channel
Public Property Let SWClose(ByVal Slot As Integer, ByVal Channel As Integer, vData As Boolean)
    Dim sCmd$
    
    'clear scan list first
    Me.ConfigClear
    
    'do switch
    If vData Then
        sCmd = "ROUT:CLOS:EXCL"
    Else
        sCmd = "ROUT:OPEN"
    End If
    sCmd = sCmd + GetChString(Slot, Channel)
    
    mGPIB.Write sCmd
End Property

Public Property Get SWClose(ByVal Slot As Integer, ByVal Channel As Integer) As Boolean
    Dim s$
    
    s = "ROUT:CLOS?" + GetChString(Slot, Channel)
    mGPIB.Write s
    s = GetGPIBRead
    
    SWClose = (s = "1")
End Property

'output/read IO card DC output
Private Function GetAOChannel(ByVal Channel As Integer) As Integer
    Const chOffset% = 3
    
    If Channel <= chOffset Then Channel = Channel + chOffset
    If Channel > chOffset + 2 Then Err.Raise -102, MyName, "Invalid AO channel number"
    GetAOChannel = Channel
End Function

Public Property Let DCOut(ByVal Slot As Integer, ByVal Channel As Integer, ByVal vData As Single)
    Dim s$
    
    Channel = GetAOChannel(Channel)
    s = "SOUR:VOLT " & vData & "," & GetChString(Slot, Channel)
    mGPIB.Write s
    
End Property

Public Property Get DCOut(ByVal Slot As Integer, ByVal Channel As Integer) As Single
    Dim s$
    
    Channel = GetAOChannel(Channel)
    s = "SOUR:VOLT? " & GetChString(Slot, Channel)
    mGPIB.Write s
    DCOut = mGPIB.Read

End Property

'initialize this class
Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    On Error GoTo ErrHandler
    
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
        .Clear
        Initialize = .IsOnline
    End With
    
ErrHandler:
    If RaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

'------------------------------------------------------------------------------------'
'-------------------------------configure scan list----------------------------------'
'------------------------------------------------------------------------------------'
'clear scan list
Public Sub ConfigClear()
    mGPIB.Write "ROUT:SCAN (@)"
End Sub

'config for DCV measurement
Public Sub ConfigDCV(ByVal Slot As Integer, ByVal Channel As Integer)
    Dim sCmd$
    
    sCmd = "CONF:VOLT:DC AUTO," & GetChString(Slot, Channel)
    mGPIB.Write sCmd
End Sub

'do DMM reading
Public Function Read() As Single
    Dim sCmd$
    Dim sOut$
    
    sCmd = "READ?"
    mGPIB.Write sCmd
    sOut = mGPIB.Read
    If IsNumeric(sOut) Then
        Read = sOut
    Else
        Read = 1E-45
    End If
End Function

'open all relays
Public Function OpenAll(ByVal Slot As Integer) As Boolean
    Dim s$, sType$
    Dim MaxChannel%
    
    'clear scan list
    Me.ConfigClear
    
    'get max channel
    sType = Me.ModuleType(Slot)
    Select Case 17
        Case InStr(1, sType, "34901A"):  MaxChannel = 20
        Case InStr(1, sType, "34902A"):  MaxChannel = 16
        Case InStr(1, sType, "34903A"):  MaxChannel = 20
        Case InStr(1, sType, "349044"):  MaxChannel = 48
        Case InStr(1, sType, "34905A"):  MaxChannel = 24
        Case InStr(1, sType, "34906A"):  MaxChannel = 24
        Case InStr(1, sType, msIOCard):  MaxChannel = -1
        Case InStr(1, sType, "34908A"):  MaxChannel = 40
    End Select
        
    'open all
    If MaxChannel <> -1 Then
        s = "ROUT:OPEN " & GetChString(Slot, 1, MaxChannel)
        mGPIB.Write s
        OpenAll = True
    End If

End Function

'disconnect MUX from DMM
Public Sub DisconnectFromDMM(ByVal Slot As Integer)
    Me.ConfigClear
    Me.SWClose(Slot, 97) = True
    Me.SWClose(Slot, 98) = True
End Sub

'local private functions
Private Function GetChString(ByVal Slot As Integer, ByVal FirstChannel As Integer, _
                             Optional ByVal LastChannel As Integer = -1) As String
    Dim s$, s1$
    
    If Slot < 0 Or Slot > 3 Then
        Err.Raise -101, MyName, "Invaild Slot Number: " & Slot
    Else
        s = Slot & Format(FirstChannel, "00")
        If LastChannel <> -1 Then
            s1 = Slot & Format(LastChannel, "00")
            s = s + ":" + s1
        End If
        s = " (@" + s + ")"
        GetChString = s
    End If
End Function

Private Function GetGPIBRead() As String
    Dim s$
    s = mGPIB.Read
    GetGPIBRead = Left(s, Len(s) - 1)
End Function
