VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA8156Attenuator"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2003-2004 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a class for Agilent 8156A optic attenuators
'It shares the same interface with another class for JDSU optic attenuators

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const MyName$ = "Agilent 8156A Optic Attenuator"

'Private Const bitSettled% = &H4

Private mModelName As String

Private mGPIB As cwGPIB

'initialize this class
Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal bRaiseError As Boolean) As Boolean
    Dim s$, sData$()
    
    'first ini
    On Error GoTo ErrHandler
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
        .Clear
        
        'get instrument type
       'get instrument type
        .Write "*IDN?"
        s = .Read
        sData = Split(s, ",")
        mModelName = sData(1)
        
        Initialize = True
    End With
    
ErrHandler:
    If bRaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

Public Property Get ModelName() As String
    ModelName = mModelName
End Property

Public Property Let Attenuation_dB(ByVal vData As Single)
    Dim s$
    s = "INP:ATT " & vData
    mGPIB.Write s
    'WaitComplete
End Property

Public Property Get Attenuation_dB() As Single
    Attenuation_dB = QueryData_Numeric("INP:ATT?")
End Property

Public Property Let Attenuation_Offset_dB(ByVal vData As Single)
    Dim s$
    s = "INP:OFFS " & vData
    mGPIB.Write s
End Property

Public Property Get Attenuation_Offset_dB() As Single
    Attenuation_Offset_dB = QueryData_Numeric("INP:OFFS?")
End Property

Public Property Let BeamBlocked(ByVal vData As Boolean)
    Dim s$
    s = "OUTP " & IIf(vData, 0, 1)
    mGPIB.Write s
End Property

Public Property Get BeamBlocked() As Boolean
    Dim s$
    s = QueryData("OUTP?")
    BeamBlocked = (s = "0")
End Property

Public Property Let PowerModeEnabled(ByVal vData As Boolean)
    Dim s$
    
    s = "OUTP:APM " + IIf(vData, "1", "0")
    mGPIB.Write s
End Property

Public Property Let Power_dBm(ByVal vData As Single)
    Dim s$
    
    s = "OUTP:POW " & vData
    PowerModeEnabled = True
    mGPIB.Write s
End Property

Public Property Get Power_dBm() As Single
    PowerModeEnabled = True
    Power_dBm = QueryData_Numeric("OUTP:POW?")
End Property

'Public Property Let Power_Offset_dBm(ByVal vData As Single)
'    Dim s$
'    s = "PCAL " & vData
'    mGPIB.Write s
'End Property
'
'Public Property Get Power_Offset_dBm() As Single
'    Power_Offset_dBm = QueryData_Numeric("PCAL?")
'End Property

Public Property Let Wavemength_nm(ByVal vData As Integer)
    Dim s$
        
    'Agilent 8156A keeps the attenuation constant
    'when reseting the wavelength, do there is no
    'need to do the attenuation factor reset like
    'that for JDSU HA9 attenuator
        
    s = "INP:WAV " & vData & " nm"
    mGPIB.Write s
End Property

Public Property Get Wavemength_nm() As Integer
    Wavemength_nm = 1000# * QueryData_Numeric("INP:WAV?")
End Property

Public Sub Reset()
    mGPIB.Write "*RST"
End Sub

'Public Sub DisplayPower()
'    mGPIB.Write "DISP 1"
'End Sub
'
'Public Sub DisplayAttenuation()
'    mGPIB.Write "DISP 0"
'End Sub

Private Function QueryData(ByVal sCmd As String) As String
    mGPIB.Write sCmd
    sCmd = mGPIB.Read
    QueryData = Left(sCmd, Len(sCmd) - 2)
End Function

Private Function QueryData_Numeric(ByVal sCmd As String) As Single
    sCmd = QueryData(sCmd)
    If IsNumeric(sCmd) Then QueryData_Numeric = CSng(sCmd)
End Function

Private Sub WaitComplete()
'    Dim bStatus&
'    Dim MyStart!, Mytime!
'
'    Const TimeOutInSec = 10!
'
'    On Error GoTo ErrHandler
'
'    'do serial pool to see if the command has completed
'    MyStart = Timer
'    While ((bStatus And bitSettled) <> bitSettled) And Mytime < TimeOutInSec
'        bStatus = mGPIB.SerialPoll
'        'Debug.Print bStatus
'        Sleep 10   'do not lock up system
'        Mytime = (Timer - MyStart)
'    Wend
    
'ErrHandler:
'    If Err.Number = 30336 Then Err.Number = 0
End Sub


