VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA8163xPowerMeter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2001-2004 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a class for Agilent 8163X Serial Modular Power Meters

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const MyName$ = "Agilent 8163X Power Meter Class"

Public Enum iA8163XPowerUnitEnum
    iapU_mW = 1
    iapU_dB = 2
    iapU_dBm = 3
'   they are the same as that for Newport powermeter
'    npUnit_mW = 1
'    npUnit_dB = 2
'    npUnit_dBm = 3
'    npunit_REL = 4
End Enum

'Public Enum npPowerMeterUnitEnum
'    npUnit_mW = 1
'    npUnit_dB = 2
'    npUnit_dBm = 3
'    npunit_REL = 4
'End Enum

Private mGPIB As cwGPIB
Private mSlot As Integer
Private mChannel As Integer
Private mModelName As String

'initialize this class
Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal bRaiseError As Boolean) As Boolean
    Dim s$, sData$()
    
    'first ini
    On Error GoTo ErrHandler
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT3s
        .Configure
        .Clear
        
        'get instrument type
        s = QueryData("*IDN?")
        sData = Split(s, ",")
        mModelName = sData(1)
    
        'reset
        'mGPIB.Write "*RST;*CLS"
        'Sleep 1000
        
        Initialize = True
    End With
    
ErrHandler:
    If bRaiseError And Err.Number <> 0 Then
        'Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
        RaiseError -1
    End If
End Function

'model name
Public Property Get ModelName() As String
    ModelName = mModelName
End Property

Public Property Get SlotInfo() As String
    SlotInfo = QueryData(":SLOT" & mSlot & ":IDN?")
End Property

Public Property Get ChannelInfo() As String
    ChannelInfo = QueryData(":SLOT" & mSlot & ":HEAD" & mChannel & ":IDN?")
End Property


'other information
Public Property Let MeterSlotNumber(ByVal vData As Integer)
    mSlot = vData
    mGPIB.Write "INIT" & mSlot & ":CONT 1"
'    Dim valid$
'    If vData < 1 Then RaiseError 1001, vData
'    valid = QueryData("SLOT" & vData & ":EMPT?")
'    If valid = "0" Then
'        mSlot = vData
'        'set meter to continus mode
'        mGPIB.Write "INIT" & mSlot & ":CONT 1"
'    Else
'        RaiseError 1002, vData
'    End If
End Property

Public Property Get MeterSlotNumber() As Integer
    MeterSlotNumber = mSlot
End Property

Public Property Let MeterChannelNumber(ByVal vData As Integer)
    mChannel = vData
'    Dim valid$
'
'    If vData < 1 Then RaiseError 1001, vData
'    valid = QueryData("SLOT" & mSlot & ":HEAD" & vData & ":EMPT?")
'    If valid = "0" Then
'        mChannel = vData
'    Else
'        RaiseError 1002, vData
'    End If
End Property

Public Property Get MeterChannelNumber() As Integer
    MeterChannelNumber = mChannel
End Property

'unit
Public Property Let Unit(ByVal vData As iA8163XPowerUnitEnum)
    Dim sCmd$
    
    Select Case vData
        Case iapU_mW:   sCmd = ":POW:UNIT 1"
        Case iapU_dB:   sCmd = ":POW:UNIT 0;REF:STAT 1"
        Case iapU_dBm:  sCmd = ":POW:UNIT 0;REF:STAT 0"
    End Select
    
    sCmd = ":SENS" & mSlot & ":CHAN" & mChannel & sCmd
    mGPIB.Write sCmd
    
End Property

Public Property Get Unit() As iA8163XPowerUnitEnum
    Dim s$
    Dim Value%

    s = QueryData("SENS" & mSlot & ":CHAN" & mChannel & ":POW:UNIT?")
    If s = "1" Then
        Value = iapU_mW
    Else
        s = QueryData("SENS" & mSlot & ":CHAN" & mChannel & ":POW:REF:STAT?")
        Value = IIf(s = "1", iapU_dB, iapU_dBm)
    End If
    
    Unit = Value
End Property

'wave length
Public Property Let Wavelength_nm(ByVal vData As Single)
    mGPIB.Write ":SENS" & mSlot & ":CHAN" & mChannel & ":POW:WAV " & vData & " nm"
End Property

Public Property Get Wavelength_nm() As Single
    Wavelength_nm = QueryData_Numeric(":SENS" & mSlot & ":CHAN" & mChannel & ":POW:WAV?")
    Wavelength_nm = Wavelength_nm * 1000000000#
End Property

'auto range
Public Property Let AutoRange(ByVal vData As Boolean)
    mGPIB.Write "SENS" & mSlot & ":CHAN" & mChannel & ":POW:RANG:AUTO " & IIf(vData, 1, 0)
End Property

Public Property Get AutoRange() As Boolean
    Dim auto$
    auto = QueryData("SENS" & mSlot & ":CHAN" & mChannel & ":POW:RANG:AUTO?")
    AutoRange = (auto = "1")
End Property

'average time
Public Property Let AverageTime_ms(ByVal vData As Single)
    mGPIB.Write "SENS" & mSlot & ":CHAN1" & ":POW:ATIM " & vData & "MS"
End Property

Public Property Get AverageTime_ms() As Single
    AverageTime_ms = 1000! * QueryData_Numeric("SENS" & mSlot & ":CHAN1" & ":POW:ATIM?")
End Property

Public Sub StoreReference()
    Dim Pow!
    
    'get current reading
    Me.Unit = iapU_dBm
    Pow = Me.MeasPower(5)
    'store it as reference
    mGPIB.Write "SENS" & mSlot & ":CHAN" & mChannel & ":POW:REF TOREF," & Pow & "DBM"
    Me.Unit = iapU_dB
End Sub

Public Sub StoreReferenceCustom(ByVal ReferenceValue As Single)
    Dim Pow!
    mGPIB.Write "SENS" & mSlot & ":CHAN" & mChannel & ":POW:REF TOREF," & ReferenceValue & "W"
End Sub


'measure power
Public Function MeasPower(Optional ByVal Samples As Integer = 1) As Single
    Dim i%
    Dim Value!

    For i = 1 To Samples
        Value = Value + QueryData_Numeric(":FETC" & mSlot & ":CHAN" & mChannel & ":POW?")
    Next i
    MeasPower = Value / Samples
End Function

Public Property Get Power_mW() As Single
    Power_mW = Me.MeasPower * 1000!
End Property

Private Function QueryData_Numeric(ByVal sCmd As String) As Single
    sCmd = QueryData(sCmd)
    If IsNumeric(sCmd) Then
        QueryData_Numeric = CSng(sCmd)
    Else
        QueryData_Numeric = -9999
    End If
End Function

Private Function QueryData(ByVal sCmd As String) As String
    On Error Resume Next
    mGPIB.Write sCmd & vbCrLf
    'WaitComplete
    sCmd = mGPIB.Read
    If Err.Number <> 0 Then sCmd = mGPIB.Read
    QueryData = Left(sCmd, Len(sCmd) - 1)
End Function

'error message
Private Sub RaiseError(ByVal ErrorNumber As Integer, Optional ByVal ErrorData As String)
    Select Case ErrorNumber
        'native error
        Case -1:    Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
        'error for this module
        Case 1001:  Err.Raise ErrorNumber, MyName, "Invalid slot/channel number: " & ErrorData
        Case 1002:  Err.Raise ErrorNumber, MyName, "Slot/channel is empty: " & ErrorData
    End Select
End Sub

'Private Sub WaitComplete()
'    Dim bStatus&
'    Dim MyStart!, Mytime!
'
'    Const TimeOutInSec = 10!
'
'    On Error GoTo ErrHandler
'
'    'do serial pool to see if the command has completed
'    MyStart = Timer
'    While (bStatus = 1) And Mytime < TimeOutInSec
'        bStatus = QueryData_Numeric("*OPC?")
'        Mytime = (Timer - MyStart)
'    Wend
'
'ErrHandler:
'    If Err.Number = 30336 Then Err.Number = 0
'End Sub

Private Sub Class_Initialize()
    mSlot = 1
    mChannel = 1
End Sub
