VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA81640xTLS"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2003 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for Agilent 8164X Tunable Laser

'Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const MyName$ = "Agilent 8164X Tunable Laser"
Private Const OneK& = 1000#
Private Const Cvacuum# = 299792458
Private Const Nair# = 1 'use as vacuum
'Private Const Nair# = 1.00026407


Private mGPIB As cwGPIB
Private mSlot As Integer
Private mUnit_mW As VbTriState

Public Function Initialize(ByRef cwGPIB As cwGPIB, _
                           ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$, sData$()
    
    On Error GoTo ErrHandler
    
    Set mGPIB = cwGPIB
                          
    With mGPIB
        'configure GPIB link
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT10s
        .Configure
                          
        'do common settings
        mUnit_mW = vbUseDefault
        With Me
            .OutputEnabled = False
            .PowerUnit_mW = True
            .PowerMode = True
            .ModulationEnabled = False
        End With
    End With
    
    Initialize = True
    
ErrHandler:
    If RaiseError Then Err.Raise Err.Number, Err.Source, Err.Description
            
End Function

'module instrument setting
'Public Property Let SlotNumber(ByVal vData As Integer)
'    Dim valid%
'
'    If vData < 0 Then RaiseError 1001, vData
'    mGPIB.Write "SLOT" & vData & ":EMPT?"
'    valid = mGPIB.Read
'    If valid = 0 Then
'        mSlot = vData
'        set meter to continus mode
'        mGPIB.Write "INIT" & mSlot & ":CONT 1"
'    Else
'        RaiseError 1002, vData
'    End If
'End Property

'model name
Public Property Get ModelName() As String
    mGPIB.Write "SLOT" & mSlot & ":IDN?"
    ModelName = mGPIB.Read
End Property

'enable/disable
Public Property Let OutputEnabled(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "1", "0")
    sCmd = "SOUR" & mSlot & ":POW:STAT " & sCmd
    mGPIB.Write sCmd
End Property

Public Property Get OutputEnabled() As Boolean
    Dim sCmd$
    
    sCmd = "SOUR" & mSlot & ":POW:STAT?"
    OutputEnabled = (QueryDataNumeric(sCmd) = 1)
End Property

'DC out
Public Property Let ModulationEnabled(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "1", "0")
    sCmd = "SOUR" & mSlot & ":AM:STAT " & sCmd
    mGPIB.Write sCmd
    
End Property

Public Property Get ModulationEnabled() As Boolean
    Dim sCmd$
    
    sCmd = "SOUR" & mSlot & ":AM:STAT?"
    ModulationEnabled = (QueryDataNumeric(sCmd) = 1)
    
End Property

'wavelength
Public Property Let Wavelength_nm(ByVal vData As Double)
    Dim sCmd$
    
    sCmd = "SOUR" & mSlot & ":WAV " & vData / 1000000000#
    mGPIB.Write sCmd
   
End Property

Public Property Get Wavelength_nm() As Double
    Dim sCmd$
    
    sCmd = "SOUR" & mSlot & ":WAV?"
    Wavelength_nm = QueryDataNumeric(sCmd)
    Wavelength_nm = Wavelength_nm * 1000000000#
    
End Property

Public Property Let Frequency_GHz(ByVal vData As Double)
    Me.Wavelength_nm = Cvacuum / Nair / vData
End Property

Public Property Get Frequency_GHz() As Double
    Frequency_GHz = Cvacuum / Nair / Me.Wavelength_nm
End Property

'power
Public Property Let PowerUnit_mW(ByVal vData As Boolean)
    Dim sCmd$
    
    mUnit_mW = vData
    
    sCmd = IIf(vData, "1", "0")
    sCmd = "SOUR" & mSlot & ":POW:UNIT " & sCmd
    mGPIB.Write sCmd
    
End Property

Public Property Get PowerUnit_mW() As Boolean
    Dim sCmd$
    
    If mUnit_mW = vbUseDefault Then
        sCmd = "SOUR" & mSlot & ":POW:UNIT?"
        mUnit_mW = (QueryDataNumeric(sCmd) = 1)
    End If
    
    PowerUnit_mW = mUnit_mW
End Property

Public Property Let PowerMode(ByVal vData As Boolean)
    Dim sCmd$
    
    mUnit_mW = vData
    
    sCmd = IIf(vData, "1", "0")
    sCmd = "SOUR" & mSlot & ":POW:ATT:AUTO " & sCmd
    mGPIB.Write sCmd
End Property

Public Property Let Power(ByVal vData As Single)
    Dim sCmd$
    
    If Me.PowerUnit_mW Then vData = vData / OneK
    
    sCmd = "SOUR" & mSlot & ":POW " & vData
    mGPIB.Write sCmd
End Property

Public Property Get Power() As Single
    Dim sCmd$
    
    sCmd = "SOUR" & mSlot & ":POW?"
    Power = QueryDataNumeric(sCmd)
    
    If Me.PowerUnit_mW Then Power = Power * OneK
    
End Property

'-------------------------------------------------------------------------
Private Function QueryDataNumeric(ByVal sCmd As String) As Single
    Dim s$
    
    s = QueryData(sCmd)
    If IsNumeric(s) Then
        QueryDataNumeric = CSng(s)
    Else
        QueryDataNumeric = -9999
    End If
End Function

Private Function QueryData(ByVal sCmd As String) As String
    Dim s$
    
    mGPIB.Write sCmd
    s = mGPIB.Read
    s = Left(s, Len(s) - 1)
    
    QueryData = s
End Function

'error message
Private Sub RaiseError(ByVal ErrorNumber As Integer, Optional ByVal ErrorData As String)
    Select Case ErrorNumber
        'native error
        Case -1:    Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
        'error for this module
        Case 1001:  Err.Raise ErrorNumber, MyName, "Invalid slot number: " & ErrorData
        Case 1002:  Err.Raise ErrorNumber, MyName, "Slot is empty: " & ErrorData
    End Select
End Sub

Private Sub Class_Initialize()
    mSlot = 0
    mUnit_mW = vbUseDefault
End Sub
