VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA8169A"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit


'----------------------------------------------------------------------------------
'------- (C) 2002-2005 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a class for Agilent/HP 8169A Polarization Controller

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const MyName$ = "Agilent/HP 8169A Polarization Controller"
Private Const cModelName$ = "HP8169A"
Private Const cAngleResolution = 0.05!

Private Const bitOPC& = &H1     'operation compelte

Private Enum iA8169AErrorEnum
    A8169A_E_Angle = 1001
    
End Enum


Private mGPIB As cwGPIB
Private mModelName As String

Private mChFirst As Integer
Private mChLast As Integer
Private mScanOn As Boolean

'initialize this class
Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal bRaiseError As Boolean) As Boolean
    Dim s$, sData$()
    
    'first ini
    On Error GoTo ErrHandler
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
        .Clear
        
        'get instrument type
        .Write "*IDN?"
        s = .Read
        sData = Split(s, ",")
        mModelName = sData(1)
        
        'reset all the angles
        .Write "*RST;*CLS"
        
        Initialize = True
    End With
    
ErrHandler:
    If bRaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

Public Property Get AngleResolution() As Single
    AngleResolution = cAngleResolution
End Property

Public Property Let AnglePolarizer(ByVal vData As Single)
    If vData >= -360! And vData <= 360! Then
        SendCmd ":POS:POL " & vData
        mScanOn = False
    Else
        'RaiseError A8169A_E_Angle
    End If
    
End Property

Public Property Get AnglePolarizer() As Single
    AnglePolarizer = QueryData_Numeric(":POS:POL?")
End Property

Public Property Let AngleHalfWavePlate(ByVal vData As Single)
    If vData >= -360! And vData <= 360! Then
        mGPIB.Write ":POS:HALF " & vData
    Else
        RaiseError A8169A_E_Angle
    End If
End Property

Public Property Get AngleHalfWavePlate() As Single
    AngleHalfWavePlate = QueryData_Numeric(":POS:HALF?")
End Property

Public Property Let AngleQuarterWavePlate(ByVal vData As Single)
    If vData >= -360! And vData <= 360! Then
        mGPIB.Write ":POS:QUAR " & vData
    Else
        RaiseError A8169A_E_Angle
    End If
End Property

Public Property Get AngleQuarterWavePlate() As Single
    AngleQuarterWavePlate = QueryData_Numeric(":POS:QUAR?")
End Property

Public Property Let SphereEpsilon(ByVal vData As Single)
    If vData >= -720! And vData <= 720! Then
        mGPIB.Write ":CIRC:EPS " & vData
    Else
        RaiseError A8169A_E_Angle
    End If

End Property

Public Property Get SphereEpsilon() As Single
    SphereEpsilon = QueryData_Numeric(":CIRC:EPS?")
End Property

Public Property Let SphereTheta(ByVal vData As Single)
    If vData >= -2160! And vData <= 2160! Then
        mGPIB.Write ":CIRC:THET " & vData
    Else
        'RaiseError A8169A_E_Angle
    End If

End Property

Public Property Get SphereTheta() As Single
    SphereTheta = QueryData_Numeric(":CIRC:THET?")
End Property

Public Property Get ScanOn() As Boolean
    ScanOn = mScanOn
End Property

Public Sub ScanStart(ByVal FastSpeed As Boolean)
    Dim sCmd$
    
    sCmd = IIf(FastSpeed, "1", "0")
    sCmd = ":PSPH:RATE " + sCmd
    sCmd = sCmd + ";:INIT"
    
    mGPIB.Write sCmd
    
    mScanOn = True
End Sub

Public Sub ScanStop()
    mGPIB.Write ":ABOR"
    
    mScanOn = False
End Sub

Private Function QueryData_Numeric(ByVal sCmd As String) As Single
    sCmd = QueryData(sCmd)
    If IsNumeric(sCmd) Then
        QueryData_Numeric = CSng(sCmd)
    Else
        QueryData_Numeric = -9999
    End If
End Function

Private Function QueryData(ByVal sCmd As String) As String
    mGPIB.Write sCmd
    sCmd = mGPIB.Read
    QueryData = Left(sCmd, Len(sCmd) - 1)
End Function

Private Sub SendCmd(ByVal sCmd As String)
    mGPIB.Write sCmd
    WaitComplete
End Sub

Private Sub WaitComplete()
    Dim bStatus&
    Dim MyStart!, Mytime!
    
    Const TimeOutInSec = 10!
    
    On Error GoTo ErrHandler
    
    'do serial pool to see if the command has completed
    MyStart = Timer
    While ((bStatus And bitOPC) <> bitOPC) And Mytime < TimeOutInSec
        bStatus = QueryData_Numeric("*OPC?")
        Sleep 10   'do not lock up system
        Mytime = (Timer - MyStart)
    Wend
    
ErrHandler:
    If Err.Number = 30336 Then Err.Number = 0
End Sub

Private Sub RaiseError(ByVal ErrorNum As iA8169AErrorEnum)
    Dim sMsg$
    
    
    Select Case ErrorNum
        Case A8169A_E_Angle:    sMsg = "Request angle out of range."
    
    End Select
    
    Err.Raise ErrorNum, MyName, sMsg
End Sub
