VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA86060C"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2002-2004 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a class for Agilent 86060C Optical Switch
'It has the same software interface as JDSU SB Serial Optic Switches


Private Const MyName$ = "Agilent 86060C Optical Switch"

Private mGPIB As CWGPIB
Private mModelName As String

Private mChFirst As Integer
Private mChLast As Integer

'initialize this class
Public Function Initialize(ByRef CWGPIB As CWGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal bRaiseError As Boolean) As Boolean
    Dim s$, sData$()
    
    'first ini
    On Error GoTo ErrHandler
    Set mGPIB = CWGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
        .Clear
        
        'get instrument type
        .Write "*IDN?"
        s = .Read
        sData = Split(s, ",")
        mModelName = Right(sData(0), 6)
        
        'get system config
        s = Me.SysConfigInfo
        
        Initialize = True
    End With
    
ErrHandler:
    If bRaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

Public Property Let ChOut(ByVal vData As Integer)
    Dim sCmd$
    
    If vData < mChFirst Or vData > mChLast Then
        sCmd = "Channel number " & vData & " out of range." & vbCrLf
        sCmd = sCmd & "Valid channel number is between " & mChFirst & " - " & mChLast
        Err.Raise 1001, MyName, sCmd
    End If
    
    sCmd = ":ROUT:LAY1:CHAN A1,B" & vData
    mGPIB.Write sCmd
End Property

Public Function Off()
    Me.ChOut = 0
End Function

Public Property Get ChOut() As Integer
    Dim s$
    
    s = QueryData(":ROUT:LAY1:CHAN?")
    ChOut = Mid(s, 5)
End Property

Public Property Get SysConfigInfo() As String
    Dim s$
    
    s = QueryData(":SYST:CONF?")
    
    mChFirst = Mid(s, 8, 1)
    mChLast = Mid(s, 10, 1)
    
    SysConfigInfo = s
End Property

Private Function QueryData(ByVal sCmd As String) As String
    mGPIB.Write sCmd
    sCmd = mGPIB.Read
    QueryData = Left(sCmd, Len(sCmd) - 1)
End Function

