VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA86120CWLMeter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit


'----------------------------------------------------------------------------------
'------- (C) 2002 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for Agilent 96120x Wavelength Meter

Private Const bitMAV = &H10

Private mGPIB As cwGPIB
Private mHighResolution As Boolean

Public Function Initialize(ByRef cwGPIB As cwGPIB, _
                           ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
      
    On Error GoTo ErrHandler
    
    Dim s$, sData$()
    
    Set mGPIB = cwGPIB
        
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT3s
        .Configure
        .Clear
    End With
    
    'initial settings
    HighResolution = True
    LinearPowerUnit = True
    SingleTriggerMode = True
    MediumSTDAir = True
    Elevation_m = 0
    DeviceNarrowBand = True
    Attenuation_dB = 0
    
    'return
    Initialize = True
    Exit Function

ErrHandler:
    If RaiseError Then
        Err.Raise Err.Number, Err.Source, Err.Description
    End If
End Function

'power unit
Public Property Let LinearPowerUnit(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "W", "DBM")
    sCmd = "UNIT:POW " + sCmd
    mGPIB.Write sCmd
End Property

Public Property Get LinearPowerUnit() As Boolean
    Dim sCmd$
    
    sCmd = "UNIT:POW?"
    sCmd = ReadData(sCmd)
    LinearPowerUnit = (sCmd = "W")
End Property

'trigger mode
Public Property Let SingleTriggerMode(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "0", "1")
    sCmd = "INIT:CONT " + sCmd
    mGPIB.Write sCmd
End Property

Public Property Get SingleTriggerMode() As Boolean
    Dim sCmd$
    
    sCmd = "INIT:CONT?"
    sCmd = ReadData(sCmd)
    SingleTriggerMode = (sCmd = "0")
End Property

Public Property Let DeviceNarrowBand(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "NARR", "BRO")
    sCmd = "SENS:CORR:DEV " + sCmd
    mGPIB.Write sCmd
End Property

Public Property Get DeviceNarrowBand() As Boolean
    Dim sCmd$
    
    sCmd = "SENS:CORR:DEV?"
    sCmd = ReadData(sCmd)
    DeviceNarrowBand = (sCmd = "NARR")
End Property

'Freq/wavelength conversion
Public Property Let MediumSTDAir(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "AIR", "VAC")
    sCmd = "SENS:CORR:MED " + sCmd
    mGPIB.Write sCmd
End Property

Public Property Get MediumSTDAir() As Boolean
    Dim sCmd$
    
    sCmd = "SENS:CORR:MED?"
    sCmd = ReadData(sCmd)
    MediumSTDAir = (sCmd = "AIR")
End Property

Public Property Let Elevation_m(ByVal vData As Integer)
    Dim sCmd$

    sCmd = "SENS:CORR:ELEV " & vData
    mGPIB.Write sCmd
    'wait until the pending command is finished
    sCmd = "*WAI"
    mGPIB.Write sCmd
End Property

Public Property Get Elevation_m() As Integer
    Dim sCmd$
    
    sCmd = "SENS:CORR:ELEV?"
    sCmd = ReadData(sCmd)
    Elevation_m = CInt(sCmd)
End Property

'resolution
Public Property Let HighResolution(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "MIN", "MAX")
    
    sCmd = "CONF:SCAL:POW:FREQ DEF," + sCmd
    mGPIB.Write sCmd
    
    'read back to record setting, this is the only sure way
    sCmd = "CONF?"
    sCmd = ReadData(sCmd)
    sCmd = Right(sCmd, 6)
    sCmd = Left(sCmd, 5)
    mHighResolution = (sCmd = "0.001")
End Property

Public Property Get HighResolution() As Boolean
    HighResolution = mHighResolution
End Property

'power offset
Public Property Let Attenuation_dB(ByVal vData As Single)
    Dim sCmd$

    sCmd = "SENS:CORR:OFFS " & vData
    mGPIB.Write sCmd
End Property

Public Property Get Attenuation_dB() As Single
    Dim sCmd$

    sCmd = "SENS:CORR:OFFS?"
    sCmd = ReadData(sCmd)
    Attenuation_dB = CInt(sCmd)
End Property

'measurements
Public Function GetFrequency_Thz(Optional DoNewScan As Boolean = True) As Double
    Dim sCmd$
    Dim Freq#
    
    'meas or fetch
    sCmd = IIf(DoNewScan, "MEAS", "FETC")
    'get data
    sCmd = sCmd + ":SCAL:POW:FREQ?"
    sCmd = ReadData(sCmd)
    Freq = CDbl(sCmd)
    'return
    GetFrequency_Thz = Freq / 1000000000000#
End Function

'measurements
Public Function GetWavelength_nm(Optional DoNewScan As Boolean = True) As Double
    Dim sCmd$
    Dim Freq#
    
    'meas or fetch
    sCmd = IIf(DoNewScan, "MEAS", "FETC")
    'get data
    sCmd = sCmd + ":SCAL:POW:WAV?"
    sCmd = ReadData(sCmd)
    Freq = CDbl(sCmd)
    'return
    GetWavelength_nm = Freq * 1000000000#
End Function

Public Function GetPower_mW(Optional DoNewScan As Boolean = True) As Double
    Dim sCmd$
    Dim Freq#
    
    'meas or fetch
    sCmd = IIf(DoNewScan, "MEAS", "FETC")
    'get data
    sCmd = sCmd + ":SCAL:POW? MAX"
    sCmd = ReadData(sCmd)
    Freq = CDbl(sCmd)
    'return
    GetPower_mW = Freq * 1000#
End Function

'private stuff
Private Function ReadData(ByVal sQuery As String) As String
    Dim s$
    Dim StatusByte As Byte
    
    'write
    mGPIB.Write sQuery
    'Wait   it seems that this is not needed as HP is doing a good job
    '       of holding return until new message is available.
    'While (StatusByte And bitMAV) <> bitMAV
    '    StatusByte = mGPIB.SerialPoll
    'Wend
    'read
    s = mGPIB.Read
    ReadData = Left(s, Len(s) - 1)
End Function
