VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iA8614xOSA"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2001-2005 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a class for Agilent 8614x Serial OSA

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
'Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)

Private Const MyName$ = "Agilent 8614xB OSA"
Private Const DataError! = 9.91E+37
Private Const bitOPC& = &H1
Private Const CVacuum# = 299792458#

Private mGPIB As cwGPIB
Private mModelName As String

Private mChFirst As Integer
Private mChLast As Integer

'initialize this class
Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal bRaiseError As Boolean) As Boolean
    Dim s$, sData$()
    
    'first ini
    On Error GoTo ErrHandler
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT10s
        .Configure
        .Clear
        
        'get instrument type
        .Write "*IDN?"
        s = .Read
        sData = Split(s, ",")
        mModelName = sData(1)
        
        Me.PowerLogScale = True
        Me.TraceDataBinaryFormat = False
        
        Initialize = True
    End With
    
ErrHandler:
    If bRaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

Public Sub InitialSetting(ByVal sIni As String)
    Dim cmdBuff$()
    Dim i%
    
    cmdBuff = Split(sIni, ";")
    For i = 0 To UBound(cmdBuff)
        mGPIB.Write cmdBuff(i)
    Next i
    
End Sub

Public Property Get ModelName() As String
    ModelName = mModelName
End Property

'---------------------------- Y/Power related----------------------------------
Public Property Let PowerLogScale(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "DISP:WIND:TRAC:Y:SPAC "
    sCmd = sCmd + IIf(vData, "LOG", "LIN")
    mGPIB.Write sCmd
    
End Property

Public Property Get PowerLogScale() As Boolean
    Dim sCmd$
    
    sCmd = "DISP:WIND:TRAC:Y:SPAC?"
    sCmd = QueryData(sCmd)
    PowerLogScale = (sCmd = "LOG")
    
End Property

Public Property Let PowerDivision_dB(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "DISP:WIND:TRAC:Y:SCAL:PDIV " & vData
    mGPIB.Write sCmd
End Property

Public Property Get PowerDivision_dB() As Single
    Dim sCmd$
    
    sCmd = "DISP:WIND:TRAC:Y:SCAL:PDIV?"
    PowerDivision_dB = QueryData_Numeric(sCmd)
End Property

Public Property Let AutoRange(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "SENS:POW:RANG:AUTO "
    sCmd = sCmd + IIf(vData, "1", "0")
    mGPIB.Write sCmd
End Property

Public Property Get AutoRange() As Boolean
    Dim sCmd$
    
    sCmd = QueryData("SENS:POW:RANG:AUTO?")
    AutoRange = (sCmd = "1")
End Property

Public Property Let AutoSensitivity(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "SENS:POW:RANG:LOW:AUTO "
    sCmd = sCmd + IIf(vData, "1", "0")
    mGPIB.Write sCmd
End Property

Public Property Get AutoSensitivity() As Boolean
    Dim sCmd$
    
    sCmd = QueryData("SENS:POW:RANG:LOW:AUTO?")
    AutoSensitivity = (sCmd = "1")
End Property

Public Property Let Sensivility_dBm(ByVal vData As Single)
     Dim sCmd$
    
    sCmd = "SENS:POW:RANG:LOW " & vData & " DBM"
    mGPIB.Write sCmd
End Property

Public Property Get Sensivility_dBm() As Single
     Dim sCmd$
    
    sCmd = "SENS:POW:RANG:LOW?"
    Sensivility_dBm = QueryData_Numeric(sCmd)
End Property

'---------------------------- X/Wavelength related----------------------------------
Public Sub CenterPeak()
    mGPIB.Write "CALC:MARK:MAX;SCEN"
End Sub

Public Property Let ScanStart_nm(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "SENS:WAV:STAR "
    sCmd = sCmd + CStr(vData) + "NM"
    mGPIB.Write sCmd
End Property

Public Property Get ScanStart_nm() As Single
    Dim Value$
    
    Value = QueryData_Numeric("SENS:WAV:STAR?")
    ScanStart_nm = Value * 1E+09!
End Property

Public Property Let ScanStop_nm(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "SENS:WAV:STOP "
    sCmd = sCmd + CStr(vData) + "NM"
    mGPIB.Write sCmd
End Property

Public Property Get ScanStop_nm() As Single
    Dim Value$
    
    Value = QueryData_Numeric("SENS:WAV:STOP?")
    ScanStop_nm = Value * 1E+09!
End Property

Public Property Let ScanCenter_nm(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "SENS:WAV:CENT "
    sCmd = sCmd + CStr(vData) + "NM"
    mGPIB.Write sCmd
End Property

Public Property Get ScanCenter_nm() As Single
    Dim Value$
    
    Value = QueryData_Numeric("SENS:WAV:CENT?")
    ScanCenter_nm = Value * 1E+09!
End Property

Public Property Let ScanSpan_nm(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "SENS:WAV:SPAN "
    sCmd = sCmd + CStr(vData) + "NM"
    mGPIB.Write sCmd
End Property

Public Property Get ScanSpan_nm() As Single
    Dim Value$
    
    Value = QueryData_Numeric("SENS:WAV:SPAN?")
    ScanSpan_nm = Value * 1E+09!
End Property

Public Property Let ScanStep_nm(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "SENS:WAV:CENT:STEP "
    sCmd = sCmd + CStr(vData) + "NM"
    mGPIB.Write sCmd
End Property

Public Property Get ScanStep_nm() As Single
    Dim Value$
    
    Value = QueryData_Numeric("SENS:WAV:CENT:STEP?")
    ScanStep_nm = Value * 1E+09!
End Property

Public Property Let ScanStepAuto(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "SENS:WAV:CENT:STEP:AUTO "
    sCmd = sCmd + IIf(vData, "1", "0")
    mGPIB.Write sCmd
End Property

Public Property Get ScanStepAuto() As Boolean
    Dim sCmd$
    
    sCmd = QueryData_Numeric("SENS:WAV:CENT:STEP:AUTO?")
    ScanStepAuto = (sCmd = "1")
End Property

'scan
Public Property Let Continus(ByVal vData As Boolean)
    mGPIB.Write "INIT:CONT " + IIf(vData, "1", "0")
End Property

Public Property Get Continus() As Boolean
    Dim s$
    s = QueryData("INIT:CONT?")
    Continus = (s = "1")
End Property

'measurement functions
Public Sub SetOSAForBWMeasurement(ByVal BWResolution_nm As Single)
    Me.ScanStepAuto = True
    mGPIB.Write "SENS:BWID:RES " & BWResolution_nm & " NM"              'set resolution bandwidth to min
    mGPIB.Write "SENS:BWID:VID:AUTO 1"
    mGPIB.Write "CALC1:MARK1:FUNC:BWID:INT ON"      'use interpolation
    mGPIB.Write "CALC1:MARK1:FUNC:BWID:READ FREQ"   'use freq unit
    mGPIB.Write "CALC1:MARK1:FUNC:BWID:STAT ON"     'enable marker
End Sub

Public Sub DoScanForBWMeasurement(Optional ByVal Center_nm As Single, _
                                  Optional ByVal Span_nm As Single, _
                                  Optional ByVal RecenterOSA As Boolean = True)
    Dim tWait&
    Dim OldT&
    
    'set scan center
    If Center_nm <> 0 Then mGPIB.Write "SENS:WAV:CENT " + CStr(Center_nm) + " NM"
    'set scan span
    If Span_nm <> 0 Then mGPIB.Write "SENS:WAV:SPAN " + CStr(Span_nm) + "  NM"
    'calculate wait time
    tWait = 200& * ScanSpan_nm
    OldT = mGPIB.TimeoutValue
    mGPIB.TimeoutValue = cwGPIBTimeoutT100s
    'do one scan
    mGPIB.Write "INIT:IMM"
    Sleep tWait
    WaitComplete
    
    If RecenterOSA Then
        'center peak to have best resolution
        mGPIB.Write "CALC:MARK:MAX;SCEN"
        'scan again
        mGPIB.Write "INIT:IMM"
        Sleep tWait
        WaitComplete
    End If
    
    mGPIB.TimeoutValue = OldT
End Sub

Public Function GetMarkerPower() As Single
    GetMarkerPower = QueryData_Numeric("CALC:MARK1:Y?")
End Function

Public Function GetMarkerFrequency_GHz() As Single
    mGPIB.Write "CALC:MARK1:X:READ FREQ"
    GetMarkerFrequency_GHz = QueryData_Numeric("CALC:MARK1:X?") / 1E+09!
End Function

Public Function GetMarkerWavelength_nm() As Single
    mGPIB.Write "CALC:MARK1:X:READ WAV"
    GetMarkerWavelength_nm = QueryData_Numeric("CALC:MARK1:X?") * 1E+09!
End Function

Public Function GetPower(ByVal Frequency_GHz As Single) As Single
    'set marker
    mGPIB.Write "CALC:MARK1:X:FREQ " & Frequency_GHz * 1E+09!
    'get power
    GetPower = GetMarkerPower
End Function

Public Sub GetBandwidth(ByVal YRel As Single, ByRef YPeak As Single, ByRef BW As Single, _
                        ByRef XLeft As Single, ByRef XRight As Single, ByRef XCenter As Single)
    Const OneGHz! = 1E+09!
    
    mGPIB.Write "CALC1:MARK1:MAX"
    mGPIB.Write "CALC1:MARK1:FUNC:BWID:NDB " + CStr(-YRel) + " db"
    
    BW = QueryData("CALC1:MARK1:FUNC:BWID:RES?") / OneGHz
    XLeft = QueryData("CALC1:MARK1:FUNC:BWID:X:LEFT?") / OneGHz
    XRight = QueryData("CALC1:MARK1:FUNC:BWID:X:RIGH?") / OneGHz
    XCenter = QueryData("CALC1:MARK1:FUNC:BWID:X:CENT?") / OneGHz
    YPeak = QueryData("CALC1:MARK1:Y?")
End Sub

'trace data
Public Property Let TraceDataBinaryFormat(ByVal vData As Boolean)
    mGPIB.Write "FORM " + IIf(vData, "REAL", "ASC")
End Property

Public Property Get TraceDataBinaryFormat() As Boolean
    Dim s$
    
    s = QueryData("FORM?")
    TraceDataBinaryFormat = (s = "REAL,+64")
End Property

Public Property Let TracePoints(ByVal vData As Integer)
    If vData < 3 Then vData = 3
    If vData > 10001 Then vData = 10001
    mGPIB.Write "SENS:SWE:POIN " & vData
End Property

Public Property Get TracePoints() As Integer
    TracePoints = QueryData_Numeric("SENS:SWE:POIN?")
End Property

Public Sub GetTraceData(ByRef Freq() As Single, ByRef Pow() As Single)
    Dim Buff&
    Dim s$
    Dim sData$()
    Dim i%
    Dim Temp#
    Dim wStart#, wStop#, wStep#
    
    Buff = Me.TracePoints
    Buff = Buff * 17    'each data point has 16 digit plus a comma, or Cr for the last one

    mGPIB.Write "TRAC:Y? TRA"
    s = mGPIB.Read(Buff)
    sData = Split(s, ",")
    
    Buff = UBound(sData)
    ReDim Freq(Buff)
    ReDim Pow(Buff)
    
    wStart = Me.ScanStart_nm
    wStop = Me.ScanStop_nm
    wStep = (wStop - wStart) / CDbl(Buff)
    
    For i = 0 To Buff
        Temp = wStart + CDbl(i) * wStep
        Freq(i) = CVacuum / Temp
        Pow(i) = sData(i)
    Next i
  
End Sub


'---------- private matter
Private Function QueryData(ByVal sCmd As String) As String
    mGPIB.Write sCmd
    sCmd = mGPIB.Read
    QueryData = Left(sCmd, Len(sCmd) - 1)
End Function

Private Function QueryData_Numeric(ByVal sCmd As String) As Single
    Dim Value!

    sCmd = QueryData(sCmd)
    If IsNumeric(sCmd) Then
        Value = CSng(sCmd)
        If Value < DataError Then
            QueryData_Numeric = Value
        End If
    End If
End Function

Private Sub WaitComplete()
    Dim bStatus&
    Dim MyStart!, Mytime!
    
    Const TimeOutInSec = 10!
    
    'On Error GoTo ErrHandler
    
    'do serial pool to see if the command has completed
    MyStart = Timer
    While ((bStatus And bitOPC) <> bitOPC) And Mytime < TimeOutInSec
        bStatus = QueryData_Numeric("*OPC?")
        'Debug.Print bStatus
        Sleep 10   'do not lock up system
        Mytime = (Timer - MyStart)
    Wend
    
    
ErrHandler:
    If Err.Number = 30336 Then Err.Number = 0
End Sub
