VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iAgilentPowerSupply"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 1998-2005 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a generic class for Agilent E364x, E363x, E361x, and 661x serials power supplies.

Public Enum apsMeasEnum
    aps_MeasDCV = 0
    aps_MeasDCI = 1
End Enum

Public Enum AgilentPowerSupplyChannel
    aps_Ch_Low = 1
    aps_Ch_High = 2
    aps_Ch_Negative = 3
    
    aps_Ch_P6V = aps_Ch_Low
    aps_Ch_P8V = aps_Ch_Low
    aps_Ch_P35V = aps_Ch_Low
    
    aps_Ch_P25V = aps_Ch_High
    aps_Ch_P20V = aps_Ch_High
    aps_Ch_P60V = aps_Ch_High
    
    aps_Ch_N25V = aps_Ch_Negative
    
End Enum

Private Const MyName = "Agilent Power Supply Class"

'list of programmed instruments
Private Const sE364xA = "E364?A"
Private Const sE363xA = "E363?A"
Private Const sE3631A = "E3631A"
Private Const s661xC = "661?C"

Private Const idE364xA% = 0
Private Const idE3631A% = 1
Private Const idE363xA% = 2
Private Const id661xC% = 3

Private mIDString As String
Private mIDValue As Integer

Private mGPIB As cwGPIB
Private mEnabled As Boolean
Private mChannel As Integer

'initialize this class
Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    On Error GoTo ErrHandler
    
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
        .Clear
        
        'get instrument type
        .Write "*IDN?"
        mIDString = GetGPIBRead
        mIDValue = GetIDValue(mIDString)
        
        'get output status
        mEnabled = Me.EnableOutput
        
        'get current range
        mChannel = GetChannelOrRange
        
        'return
        Initialize = (mIDValue <> -1)
    End With
    
ErrHandler:
    If RaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

Public Property Get InstrumentType() As String
    InstrumentType = mIDString
End Property

'enable/siable
Public Property Let EnableOutput(ByVal vData As Boolean)
    Dim sCmd$
    
    If mEnabled <> vData Then
        sCmd = IIf(vData, "ON", "OFF")
        sCmd = "OUTP " + sCmd
        mGPIB.Write sCmd
        'set internal value
        mEnabled = vData
    End If
End Property

Public Property Get EnableOutput() As Boolean
    Dim sCmd$
    
    sCmd = "OUTP?"
    mGPIB.Write sCmd
    
    sCmd = GetGPIBRead
    mEnabled = (sCmd = "1")
    EnableOutput = mEnabled
    
End Property

'set channel or range
Public Function GetChannelOrRange() As AgilentPowerSupplyChannel
    Dim sCmd$
    Dim i%
    
    Select Case mIDValue
        Case idE364xA, idE363xA
            sCmd = "VOLT:RANG?"
        Case idE3631A
            sCmd = "INST?"
    End Select
    
    If sCmd <> "" Then
        mGPIB.Write sCmd
    
        sCmd = GetGPIBRead
    
        Select Case sCmd
            Case "P6V", "P8V", "P35V":    i = 1
            Case "P25V", "P20V", "P60V":  i = 2
            Case "N25V":                  i = 3
        End Select
    
    Else
        i = 1
    End If
    
    GetChannelOrRange = i
    
End Function

Public Function SetChannelOrRange(ByVal ChannelRange As AgilentPowerSupplyChannel) As Boolean
    Dim sCmd$
    
    'set channel is needed
    If mChannel <> ChannelRange Then
        Select Case mIDValue
            Case idE364xA, idE363xA
                Select Case ChannelRange
                    Case aps_Ch_Low:    sCmd = "LOW"
                    Case aps_Ch_High:   sCmd = "HIGH"
                    Case Else:          GoTo ChannelError
                End Select
                sCmd = "VOLT:RANG " + sCmd
            
            Case idE3631A
                Select Case ChannelRange
                    Case 1, 2, 3:       sCmd = "INST:NSEL " & ChannelRange
                    Case Else:          GoTo ChannelError
                End Select
        End Select
        
        'set cmd
        mGPIB.Write sCmd
    
        'set internal value
        mChannel = ChannelRange
    End If
    
    SetChannelOrRange = True
    Exit Function

ChannelError:
    MsgBox ChannelRange & " is an invalid channel/range for this module: " & mIDString, , MyName
End Function
    
'set DCI, current limit
Public Function SetDCI(ByVal DCI_mA As Single, Optional ByVal ChannelRange As AgilentPowerSupplyChannel) As Boolean
    Dim sCmd$
    Dim success As Boolean
    
    success = True
    If ChannelRange > 0 Then success = SetChannelOrRange(ChannelRange)
    
    If success Then
        DCI_mA = 0.001 * DCI_mA
        sCmd = "CURR " & DCI_mA
        mGPIB.Write sCmd
        SetDCI = True
    End If
    
End Function

    
'set DCV output
Public Function SetDCV(ByVal DCV As Single, Optional ByVal ChannelRange As AgilentPowerSupplyChannel) As Boolean
    Dim sCmd$
    Dim success As Boolean
    
    success = True
    If ChannelRange > 0 Then success = SetChannelOrRange(ChannelRange)
    
    If success Then
        sCmd = "VOLT " & DCV
        mGPIB.Write sCmd
        SetDCV = True
    End If
    
End Function

'meas output
Public Function Measure(ByVal MeasType As apsMeasEnum, Optional ByVal ChannelRange As AgilentPowerSupplyChannel) As Single
    Dim sCmd$
    Dim success As Boolean
    
    success = True
    If ChannelRange > 0 Then success = SetChannelOrRange(ChannelRange)
    
    If success Then
        Select Case MeasType
            Case aps_MeasDCV:   sCmd = "MEAS:VOLT:DC?"
            Case aps_MeasDCI:   sCmd = "MEAS:CURR:DC?"
        End Select
        mGPIB.Write sCmd
        Measure = CSng(mGPIB.Read)
        
    'wrong channel
    Else
        Measure = 1E-45
    End If
    
End Function


'strip out the EOF character
Private Function GetGPIBRead() As String
    Dim s$
    s = mGPIB.Read
    GetGPIBRead = Left(s, Len(s) - 1)
End Function

'private function for ID recognition
Private Function GetIDValue(ByVal sID) As Integer
    Dim sData$()
    Dim i%
    
    sData = Split(sID, ",")
    sID = sData(1)
    
    Select Case True
        Case sID Like sE364xA:      i = idE364xA
        Case sID = sE3631A:         i = idE3631A
        Case sID Like sE363xA:      i = idE363xA    'this has to be below last statement to get E3631A out first
        Case sID Like s661xC:       i = id661xC
        'others are not supported
        Case Else
            i = -1
            MsgBox sID + " is not supported", , MyName
    End Select
    
    GetIDValue = i
    
End Function

