VERSION 5.00
Object = "{648A5603-2C6E-101B-82B6-000000000014}#1.1#0"; "MSCOMM32.OCX"
Begin VB.UserControl iCV1000 
   ClientHeight    =   1920
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   2550
   InvisibleAtRuntime=   -1  'True
   Picture         =   "iCV1000.ctx":0000
   ScaleHeight     =   1920
   ScaleWidth      =   2550
   ToolboxBitmap   =   "iCV1000.ctx":030A
   Begin MSCommLib.MSComm MSComm1 
      Left            =   900
      Top             =   60
      _ExtentX        =   1005
      _ExtentY        =   1005
      _Version        =   393216
      DTREnable       =   -1  'True
   End
End
Attribute VB_Name = "iCV1000"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public Enum iCV1000UnitEnum
    cvUnit_Inch = 0
    cvUnit_mm = 1
    cvUnit_Deg = 2
    cvUnit_DSM = 3
End Enum

'API
Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

'name
Private Const MyName = "iCV1000"

'Default Property Values:
Private Const m_def_BuadRate = 600
Private Const m_def_Address = 1

'Property Variables:
Private m_Unit As iCV1000UnitEnum
Private m_DisplayOn As Boolean
Private m_AxisID As String
Private m_BuadRate As Integer
Private m_Address As Integer

Public Property Get BuadRate() As Integer
    BuadRate = m_BuadRate
End Property

Public Property Let BuadRate(ByVal New_BuadRate As Integer)
    Select Case New_BuadRate
        Case 150, 300, 600, 1200, 2400, 4800, 9600
            m_BuadRate = New_BuadRate
            PropertyChanged "BuadRate"
        Case Else
            Err.Raise -102, MyName, "The specified buad rate is not available for the hardware." _
                                    & " Available buad rates are: 150, 300, 600, 1200, 2400, 4800, or 9600"
    End Select
End Property

Public Property Get Address() As Integer
    Address = m_Address
End Property

Public Property Let Address(ByVal New_Address As Integer)
    If New_Address < 1 Then Err.Raise -101, MyName, "Comm port address should be larger than 0"
    m_Address = New_Address
    PropertyChanged "Address"
End Property

'run time property
Public Property Get AxisID() As String
    'If Ambient.UserMode = False Then Err.Raise 387
    AxisID = m_AxisID
End Property

Public Property Get DisplayOn() As Boolean
    'If Ambient.UserMode = False Then Err.Raise 387
    DisplayOn = m_DisplayOn
End Property

Public Property Let DisplayOn(ByVal New_DisplayOn As Boolean)
    
    If Ambient.UserMode = False Then Err.Raise 387
    m_DisplayOn = New_DisplayOn
    
    SendCmd IIf(m_DisplayOn, "N", "O")
End Property

Public Property Get Unit() As iCV1000UnitEnum
    'If Ambient.UserMode = False Then Err.Raise 387
    Unit = m_Unit
End Property

Public Property Let Unit(ByVal New_Unit As iCV1000UnitEnum)
    
    If Ambient.UserMode = False Then Err.Raise 387
    
    If New_Unit = m_Unit Then
        'already there, do nothing
    ElseIf Abs(m_Unit \ 2 - New_Unit \ 2) = 1 Then
        'cannot exchange from translational to rotational by software
        Err.Raise -105, MyName, "Cannot change between translational and rotational units by software. Please do a hardware change according to the menu."
    Else
        'do change
        m_Unit = New_Unit
        SendCmd IIf(m_Unit < 2, "I", "D")
    End If
End Property

'methods
Public Sub Initialize(Optional ByVal Port As Integer, _
                           Optional ByVal BuadRate As Integer)
    Dim s$
    
    'get value
    If Port <> 0 Then Me.Address = Port
    If BuadRate <> 0 Then Me.BuadRate = BuadRate
      
    'ini
    With MSComm1
        'close port in case it is open
        Me.ClosePort
        .CommPort = m_Address
        'CV1000 hardware settings are: no parity, 8 data bits, and 2 stop bits.
        .Settings = m_BuadRate & ",N,8,2"
        'read whole buffer at once
        .InputLen = 0
        'CV1000 uses hardware handshake
        .Handshaking = comRTS
        'Request To Sead from CV1000
        .RTSEnable = True
        'Open the port
        .PortOpen = True
        'set display on
        Me.DisplayOn = True
        'get unit setting from instrument
        s = DoReading("P")
        Select Case Right(s, 1)
            Case "h":   m_Unit = cvUnit_Inch
            Case "m":   m_Unit = cvUnit_mm
            Case "g":   m_Unit = cvUnit_Deg
            Case "s":   m_Unit = cvUnit_DSM
        End Select
        'get axis ID
        m_AxisID = Left(s, 1)
    End With
End Sub

Public Sub ClosePort()
    If MSComm1.PortOpen Then MSComm1.PortOpen = False
End Sub

Public Sub ZeroReading()
    SendCmd "ZI"
End Sub

Public Function Meas() As String
    Dim s$
    s = DoReading("P")
    'check manual for the consitions
    Select Case Right(s, 1)
        'in inch or mm, translational
        Case "h", "m"
            s = Trim(Mid(s, 2, 10))
        'in deg, rotational
        Case "g"
            s = Trim(Mid(s, 2, 9))
        'in degree-minutes-seconds, rotational
        Case "s"
            s = Trim(Mid(s, 2, 11))
    End Select
    Meas = s
End Function

Public Function GetSettings() As String
    Dim s$
    s = DoReading("i")
    s = Replace(s, m_EOFChar, ",")
    s = Replace(s, vbCr, ",")
    s = Replace(s, Chr(20), "")
    GetSettings = s
End Function

Private Function DoReading(ByVal sCmd As String) As String
    Dim s$
    Dim lCheck%
    Dim sCheck$
    Dim haveLF As Boolean
    
    'clear buffer (echo from other commands)
    s = MSComm1.Input
    'send cmd
    SendCmd sCmd
    'read
    s = MSComm1.Input
    'check EOF character
    If Right(s, 1) = vbLf Then
        lCheck = 3
        sCheck = "a" + vbCrLf
    Else
        lCheck = 2
        sCheck = "a" + vbCr
    End If
    'get rid off echo
    While Left(s, lCheck) = sCheck
        s = Mid(s, lCheck + 1)
    Wend
    'get eof
    DoReading = Left(s, Len(s) - 2 * (lCheck - 1))
End Function

Private Sub SendCmd(ByVal sCmd As String)
    MSComm1.Output = sCmd & vbCr
    'wait - CV1000 does not like to receive cmd too fast
    'this is not a very efficient way if we only send one cmd
    'but this can avoid to put delay in every place where two cmds are sent
    Sleep 100
End Sub

'Initialize Properties for User Control
Private Sub UserControl_InitProperties()
    m_BuadRate = m_def_BuadRate
    m_Address = m_def_Address
End Sub

'Load property values from storage
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    m_BuadRate = PropBag.ReadProperty("BuadRate", m_def_BuadRate)
    m_Address = PropBag.ReadProperty("Address", m_def_Address)
End Sub

Private Sub UserControl_Resize()
    UserControl.Width = 480
    UserControl.Height = 480
End Sub

Private Sub UserControl_Terminate()
    Me.ClosePort
End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    Call PropBag.WriteProperty("BuadRate", m_BuadRate, m_def_BuadRate)
    Call PropBag.WriteProperty("Address", m_Address, m_def_Address)
End Sub


