VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iChorMaster"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2006 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'insturment class for Coherent FieldMaser GS or LabMaster serial power meter
'it can be used for either GPIB (LabMaster only) or Serial communication

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const mFieldMaser$ = "FieldMaster GS"
Private Const mLabMaster$ = "LabMaster Ultima"

Private mDualChannel As Boolean
Private mChStr As String
Private mPort As Object

Public Function Initialize(ByRef cwPort As Object, ByVal PortNum As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mPort = cwPort
    
    If TypeOf mPort Is cwSerial Then
        With mPort
            .ComPort = PortNum
            .BaudRate = 9600
            .DataBits = cwDataBitsEight
            .StopBits = cwStopBitsOne
            .Parity = cwParityNone
            .FlowControl = cwFlowControlNone
            'others
            .DataAsString = True
            .TimeoutValue = 1000
            .EOSChar = vbCr
            .Configure
        End With
    
    ElseIf TypeOf mPort Is CWGPIB Then
        With mPort
            .BoardNumber = 0
            .PrimaryAddress = PortNum
            .TimeoutValue = cwGPIBTimeoutT1s
            .Configure
        End With
    
    Else
        s = "Unexpected controller."
        MsgBox s, vbExclamation + vbOKOnly
        Exit Function
    End If

    s = ModelName
    Select Case s
        Case mFieldMaser
            mDualChannel = False
        Case mLabMaster
            mDualChannel = True
        Case Else
            s = "Unexpected instrument model: " + s
            s = s + ". Class is intended for Coherent's "
            s = s + mFieldMaser + " or " + mLabMaster
            MsgBox s, vbExclamation + vbOKOnly
    End Select
    
    'default to channel A
    Me.Channel = 1

    Initialize = True
    
ErrHandler:
    If RaiseError Then Err.Raise Err.Number, Err.Source, Err.Description

End Function

Public Property Get ModelName()
    Dim s$
    Dim p%
    
    If TypeOf mPort Is CWGPIB Then
        s = QueryData("*IDN?")
    Else
        On Error GoTo LocalError
        s = QueryData("*ind")
    End If
    
    p = InStr(1, s, ",")
    If p > 0 Then
        s = Mid(s, p + 1)
        p = InStr(1, s, ",")
        If p > 1 Then s = Left(s, p - 1)
    End If

LocalError:
    'lab master does not have IDN command in serial interface
    'so it will time out. This is ok
    If Err.Number = 30301 And TypeOf mPort Is cwSerial Then s = mLabMaster
    
    ModelName = Trim(s)
End Property

Public Property Get DetectorType()
    Dim sCmd$
    
    sCmd = "dt? " + mChStr
    DetectorType = QueryData(sCmd)
End Property

Public Property Let Attenuation(ByVal vData As Integer)
    Dim sCmd$
    
    If vData < 1 Then Exit Property
    sCmd = "at " & mChStr & vData
    SendCmd sCmd
End Property

Public Property Get Attenuation() As Integer
    Dim sCmd$
    
    sCmd = "at? " + mChStr
    Attenuation = QueryData_Numeric(sCmd)
End Property

Public Property Let AutoRange(ByVal vData As Boolean)
    Range_W = IIf(vData, 0, 100)
End Property

Public Property Get AutoRange() As Boolean
    AutoRange = (Range_W = 0)
End Property

Public Property Let Channel(ByVal vData As Integer)
    If mDualChannel Then
        mChStr = IIf(vData = 1, "a, ", "b, ")
    Else
        mChStr = ""
    End If
End Property

Public Property Let NumberForAverage(ByVal vData As Integer)
    Dim sCmd$
    
    If vData < 1 Then Exit Property
    sCmd = "pa " & mChStr & vData
    SendCmd sCmd
End Property

Public Property Get NumberForAverage() As Integer
     Dim sCmd$
    
    sCmd = "pa? " + mChStr
    NumberForAverage = QueryData_Numeric(sCmd)
End Property

Public Property Let Range_W(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "rn " & mChStr & vData
    SendCmd sCmd
End Property

Public Property Get Range_W() As Single
    Dim sCmd$
    
    sCmd = "rn? " + mChStr
    Range_W = QueryData_Numeric(sCmd)
End Property


Public Property Get Power_W() As Single
    Dim sCmd$
    
    sCmd = "pw? " + mChStr
    Power_W = QueryData_Numeric(sCmd)
End Property

Public Property Get Power_mW() As Single
    Power_mW = 1000! * Power_W
End Property

Public Property Let Wavelength_nm(ByVal vData As Double)
    Dim sCmd$
    
    sCmd = "wv " & mChStr & vData & "E-9"
    SendCmd sCmd
End Property

Public Property Get Wavelength_nm() As Double
    Dim sCmd$
    Dim v#
    
    sCmd = "wv? " + mChStr
    v = QueryData_Numeric(sCmd)
    Wavelength_nm = 1000000000# * v
End Property

'zero
Public Property Let ZeroMeter(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "of " + mChStr
    sCmd = sCmd + IIf(vData, "on", "off")
    SendCmd sCmd
End Property

Public Property Get ZeroMeter() As Boolean
    ZeroMeter = (ZeroOffset <> 0)
End Property

Public Property Get ZeroOffset() As Single
    Dim sCmd$
    
    sCmd = "of? " + mChStr
    ZeroOffset = QueryData_Numeric(sCmd)
End Property

'local property
Private Sub SendCmd(ByVal sCmd As String)
    Dim s$
    
    s = Right(sCmd, 1)
    If s = " " Or s = "," Then sCmd = Left(sCmd, Len(sCmd) - 1)
    If s <> vbCr Then sCmd = sCmd + vbCr    'append cartridge return to all command
    
    mPort.Write sCmd
End Sub

Private Function QueryData(ByVal sCmd As String) As String
    'input buffer need to be cleared before query
    'instrument echos the numeric data sent
    If TypeOf mPort Is cwSerial Then mPort.ClearInputBuffer
    'start query
    SendCmd sCmd
    sCmd = mPort.Read
    Sleep 5
    QueryData = Left(sCmd, Len(sCmd) - 1)
End Function

Private Function QueryData_Numeric(ByVal sCmd As String) As Double
    sCmd = QueryData(sCmd)
    If IsNumeric(sCmd) Then QueryData_Numeric = CDbl(sCmd)
End Function


