VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iEIBisynch"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2006 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

' This is a generic class for Eurothermal controllers with EI Bisynch ASCII protocol
' It works with Eurotherm controllers implemented with EI Bisynch
'               Marlow SE5000 serial TEC controllers (w/ Eurotherm 847)

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const Timeout& = 1000   '1 sec
Private Const Adrs$ = "0000"    'controller address

Private mComm As MSComm
Private mError As Integer

Public Function Initialize(ByRef hMSComm As MSComm, ByVal PortNum As Integer, _
                           ByVal BaudRate As Integer, Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mComm = hMSComm
    With mComm
        .CommPort = PortNum
        .Settings = BaudRate & ",E,7,1"
        .Handshaking = comNone
        .InputLen = 0
        .InputMode = comInputModeText
        .PortOpen = True
    End With

    s = Temperature_Measured

    Initialize = (LastError = "")
    
ErrHandler:
    If RaiseError Then Err.Raise Err.Number, Err.Source, Err.Description

End Function

Public Property Get Temperature_Measured() As Single
    Temperature_Measured = QueryData("PV")
End Property

'-----------------------------------------------------------
Public Property Let Temperature_Setpoint(ByVal vData As Single)
    SendCmd "SL" + CStr(vData)
End Property

Public Property Get Temperature_Setpoint() As Single
   Temperature_Setpoint = QueryData("SL")
End Property

'-----------------------------------------------------------
Public Property Let Temperature_Setpoint_Max(ByVal vData As Single)
    SendCmd "HS" + CStr(vData)
End Property

Public Property Get Temperature_Setpoint_Max() As Single
   Temperature_Setpoint_Max = QueryData("HS")
End Property

'-----------------------------------------------------------
Public Property Let Temperature_Setpoint_Min(ByVal vData As Single)
    SendCmd "LS" + CStr(vData)
End Property

Public Property Get Temperature_Setpoint_Min() As Single
   Temperature_Setpoint_Min = QueryData("LS")
End Property

'-----------------------------------------------------------
'Power is in %, 100% = max heat
'                50% = minimum power
'                 0% = max cool
Public Property Get OutputPower_Measured() As Single
    OutputPower_Measured = QueryData("OP")
End Property

'-----------------------------------------------------------
Public Property Let OutputPower_Limit(ByVal vData As Single)
    SendCmd "HO" + CStr(vData)
End Property

Public Property Get OutputPower_Limit() As Single
    OutputPower_Limit = QueryData("HO")
End Property

'-----------------------------------------------------------
Public Property Let RampRate(ByVal vData As Single)
    SendCmd "SR" + CStr(vData)
End Property

Public Property Get RampRate() As Single
    RampRate = QueryData("SR")
End Property

'-----------------------------------------------------------
Public Property Let PID_Proportional(ByVal vData As Single)
    SendCmd "XP" + CStr(vData)
End Property

Public Property Get PID_Proportional() As Single
    PID_Proportional = QueryData("XP")
End Property

'-----------------------------------------------------------
Public Property Let PID_Integral(ByVal vData As Single)
    SendCmd "TI" + CStr(vData)
End Property

Public Property Get PID_Integral() As Single
    PID_Integral = QueryData("TI")
End Property

'-----------------------------------------------------------
Public Property Let PID_Derivative(ByVal vData As Single)
    SendCmd "TD" + CStr(vData)
End Property

Public Property Get PID_Derivative() As Single
    PID_Derivative = QueryData("TD")
End Property

'-----------------------------------------------------------
Public Property Get LastError() As String
    Select Case mError
        Case 0, 6:  LastError = ""
        Case 11:    LastError = "Incomplete read"
        Case 12:    LastError = "Read BCC error"
        Case 15:    LastError = "Failed to write"
    End Select
End Property

'private commands ----------------
Private Function CalculateBCC(ByVal sCmd$) As Integer
    Dim s$
    Dim BCC%
    Dim i%, ii%
    
    BCC = &H3   'ETX character is included in the calculation
    
    ii = Len(sCmd)
    For i = 1 To ii
        s = Mid(sCmd, i, 1)
        BCC = BCC Xor Asc(s)
    Next i
    
    CalculateBCC = BCC
End Function

Private Sub SendCmd(ByVal sCmd As String)
    Dim s$
    Dim BCC%
    Dim i%, ii%
        
    'build command
    BCC = CalculateBCC(sCmd)
    s = Chr(&H4) + Adrs + Chr(&H2) + sCmd + Chr(&H3) + Chr(BCC)
    
    With mComm
        'clear buffer
        sCmd = .Input
        'send
        .Output = s
        'wait
        Sleep 50
        'Read
        s = .Input
        'check
        mError = Asc(s)
    End With
End Sub

Private Function QueryData(ByVal sCmd As String) As Single
    Dim s$
    Dim BCC%
    Dim p1%
    Dim p2%
  
    With mComm
        'clear buffer
        s = .Input
        'build command
        s = Chr(&H4) + Adrs + sCmd + Chr(&H5)
        'send
        .Output = s
        'wait
        Sleep 50
        'Read
        s = .Input
    End With
    
    'parse
    
    p1 = InStr(1, s, sCmd)      'find sent cmd
    p2 = InStr(1, s, Chr(&H3))  'find end of text character
    If p1 > 0 Then
        p1 = p1 + Len(sCmd)
        If p2 > p1 Then
            BCC = Asc(Right(s, 1))
            s = Mid(s, p1, p2 - p1)
            'retrun
            If IsNumeric(s) Then
                QueryData = CSng(s)
                mError = 0  'no error
            End If
            'check BCC
            If BCC <> CalculateBCC(sCmd + s) Then mError = 12
        Else
            mError = 11
        End If
    Else
        mError = 11
    End If
End Function






