VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iESP300"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit


'----------------------------------------------------------------------------------
'------- (C) 2002-2003 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a calss for Newport ESP3000 motion controllers

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const MyName$ = "ESP300"
Private Const cstIDN$ = MyName
Private Const cstError! = -9999

Private Const DefMaxAxis% = 6
Private Const DefAxis% = 1

Private Const bitOPC& = &H40

Private mMaxAxis As Integer
Private mAxis As Integer


Private mGPIB As cwGPIB

Public Function Initialize(ByRef cwGPIB As cwGPIB, _
                           ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mGPIB = cwGPIB
                    
    With mGPIB
        'configure GPIB link
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
                          
        'get model num
        s = QueryData("*IDN?", False)
        If Left(s, Len(cstIDN)) <> cstIDN Then
            s = "Instrument <" & s & "> is not supported."
            Err.Raise 9000, "NetTest Tunable Light Source", s
        End If
        
        'do common settings
        With Me
            .EncoderDisplayResolution = 4
        End With
    End With
                          
    Initialize = True
    Exit Function
    
ErrHandler:
    If RaiseError Then Err.Raise Err.Number, Err.Source, Err.Description
End Function


Public Property Let Axis(ByVal vData As Integer)
    Dim s$

    If vData < 1 Or vData > mMaxAxis Then
        s = "Invalid axis number: " & vData
        s = s + vbCrLf + "Valid axis number: 1 - " & mMaxAxis
        Err.Raise 9001, MyName, s
    Else
        mAxis = vData
    End If
    
End Property

Public Property Get Axis() As Integer
    Axis = mAxis
End Property

Public Property Get CurrentPosition() As Single
    CurrentPosition = QueryData_Numeric("TP?")
End Property

Public Property Let Enabled(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "MO", "MF")
    SendCmd sCmd
End Property

Public Property Get Enabled() As Boolean
    Enabled = (QueryData("MO?") = "1")
End Property

Public Property Let EncoderDisplayResolution(ByVal vData As Integer)
    'only 0-7 are valid
    Select Case vData
        Case Is < 0:    vData = 0
        Case Is > 7:    vData = 7
    End Select
    SendCmd "FP" & vData
End Property

Public Property Get EncoderDisplayResolution() As Integer
    EncoderDisplayResolution = QueryData_Numeric("FP?")
End Property

Public Property Get ErrMsg() As String
    ErrMsg = QueryData("TB?", False)
End Property

Public Property Get ID() As String
    ID = QueryData("ID?")
End Property

Public Property Let MaxAxisNumber(ByVal vData As Integer)
    mMaxAxis = vData
End Property

Public Property Get MaxAxisNumber() As Integer
    MaxAxisNumber = mMaxAxis
End Property

Public Property Get MotionCompleted() As Boolean
    MotionCompleted = (QueryData("MD?") = "1")
End Property

Public Sub DefineHome(ByVal HomeValue As Single)
    SendCmd "DH" & HomeValue
End Sub

'seppd
Public Property Let VelocityMax(ByVal vData As Single)
    SendCmd "VU" & vData
End Property

Public Property Get VelocityMax() As Single
    Velocity = QueryData_Numeric("VU?")
End Property

Public Property Let Velocity(ByVal vData As Single)
    SendCmd "VA" & vData
End Property

Public Property Get Velocity() As Single
    Velocity = QueryData_Numeric("VA?")
End Property

'movement function, move absolute, return absolution position
Public Function MoveAbsolute(ByVal Target As Single) As Single
    MoveAbsolute = Move(Target, True)
End Function

'movement function, move relative, BUT return absolution position
Public Function MoveRelative(ByVal Amount As Single) As Single
    MoveRelative = Move(Amount, False)
End Function

Private Function Move(ByVal Amount As Single, ByVal MoveAbsolute As Boolean)
    Dim sCmd$
        
    'set command
    sCmd = IIf(MoveAbsolute, "PA", "PR")
    sCmd = mAxis & sCmd & Amount        'move absolute
    sCmd = sCmd & ";" & mAxis & "WS10"  'wait motion complete plus an addition 10 ms
    sCmd = sCmd & ";RQ0"                'generate an interupt after completion
   
    'move
    mGPIB.Write sCmd
    
    'check for finish
    WaitComplete
    
    'return the current position
    Move = Me.CurrentPosition
End Function

'private stuff
Private Sub SendCmd(ByVal sCmd As String, Optional ByVal AppendAxisInfo As Boolean = True)
    'append axis number
    If AppendAxisInfo Then sCmd = mAxis & sCmd
    'write
    mGPIB.Write sCmd
End Sub

Private Function QueryData_Numeric(ByVal sCmd As String, Optional ByVal AppendAxisInfo As Boolean = True) As Single
    Dim s$
    s = QueryData(sCmd, AppendAxisInfo)
    If IsNumeric(s) Then
        QueryData_Numeric = CSng(s)
    Else
        QueryData_Numeric = cstError
    End If
End Function

Private Function QueryData(ByVal sCmd As String, Optional ByVal AppendAxisInfo As Boolean = True) As String
    SendCmd sCmd, AppendAxisInfo
    Sleep 20
    sCmd = mGPIB.Read
    QueryData = Left(sCmd, Len(sCmd) - 2)
End Function

Private Sub WaitBit(ByVal BitMask As Long, Optional ByVal TimeOutInSec As Single = 60)
    Dim bStatus&
    Dim MyStart!, Mytime!
    
    On Error GoTo ErrHandler
    
    'do serial pool to see if the command has completed
    MyStart = Timer
    While ((bStatus And BitMask) <> BitMask) And Mytime < TimeOutInSec
        Sleep 10
        bStatus = mGPIB.SerialPoll
        Mytime = (Timer - MyStart)
    Wend
    
ErrHandler:
    If Err.Number = 30336 Then Err.Number = 0
End Sub

Private Sub WaitComplete(Optional ByVal TimeOutInSec As Single = 60)
    WaitBit bitOPC, TimeOutInSec
End Sub

Private Sub Class_Initialize()
    mMaxAxis = DefMaxAxis
    mAxis = DefAxis
End Sub
