VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iHCA200"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2006 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

' This is a galss for Coherent Tutcore HCA2000 power supply with RS232 control

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Public Enum HCAOutputModeEnum
    HCA_CW_DC = 0
    HCA_Pulse = 1
    HCA_CW_Gated = 2
    HCA_Pulse_Gated = 3
End Enum

Private Const mModelName = "Test"

Private mSerial As cwSerial


Public Function Initialize(ByRef cwSerial As cwSerial, ByVal PortNum As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mSerial = cwSerial
    With mSerial
        .ComPort = PortNum
        .BaudRate = 9600
        .DataBits = cwDataBitsEight
        .StopBits = cwStopBitsOne
        .Parity = cwParityNone
        .FlowControl = cwFlowControlNone
        'others
        .DataAsString = True
        .TimeoutValue = 1000
        .EOSChar = vbCr
        .Configure
    End With
    
    'test action
    s = OutputEnabled

    Initialize = True
    
ErrHandler:
    If RaiseError Then Err.Raise Err.Number, Err.Source, Err.Description

End Function

Public Property Let Current_A(ByVal vData As Single)
    Dim s$
    
    s = "amps " & vData & "A"
    SendCmd s
    
    'extra wait
    Sleep 100
End Property

Public Property Get Current_A() As Single
    Current_A = QueryData_Numeric("amps")
End Property

Public Property Let CurrentLimit_A(ByVal vData As Single)
    Dim s$
    
    s = "maxamps " & vData & "A"
    SendCmd s
End Property

Public Property Get CurrentLimit_A() As Single
    CurrentLimit_A = QueryData_Numeric("maxamps")
End Property

Public Property Let OutputEnabled(ByVal vData As Boolean)
    Dim s$
    
    s = "output " + IIf(vData, "1", "0")
    SendCmd s
    If vData Then Sleep 100     'extra time for enable
End Property

Public Property Get OutputEnabled() As Boolean
    Dim i%
    
    i = QueryData_Numeric("output")
    OutputEnabled = (i = 1)
End Property

Public Property Let OutputMode(ByVal vData As HCAOutputModeEnum)
    Dim s$
    Dim OldEnabled As Boolean
    
    'exit if it is already there
    If OutputMode = vData Then Exit Property
    
    'disable output first
    OldEnabled = OutputEnabled
    If OldEnabled Then OutputEnabled = False
    
    'change mode
    Select Case vData
        Case HCA_Pulse:         s = "PULSED"
        Case HCA_CW_DC:         s = "CW"
        Case HCA_CW_Gated:      s = "CW_G"
        Case HCA_Pulse_Gated:   s = "PULSED_G"
    End Select
    s = "opmode " + s
    SendCmd s
    
    'set state back
    If OldEnabled Then OutputEnabled = True
    
End Property

Public Property Get OutputMode() As HCAOutputModeEnum
    OutputMode = QueryData_Numeric("opmode")
End Property

Public Property Let PulsePeriod_ms(ByVal vData As Single)
    Dim s$
    
    s = "period " & vData & "ms"
    SendCmd s
End Property

Public Property Get PulsePeriod_ms() As Single
    PulsePeriod_ms = QueryData_Numeric("period")
End Property

Public Property Let PulseWidth_us(ByVal vData As Single)
    Dim s$
    
    s = "pw " & vData & "us"
    SendCmd s
End Property

Public Property Get PulseWidth_us() As Single
    PulseWidth_us = QueryData_Numeric("pw")
End Property

Public Property Let Voltage_V(ByVal vData As Single)
    Dim s$
    
    s = "volts " & vData & "V"
    SendCmd s
End Property

Public Property Get Voltage_V() As Single
    Voltage_V = QueryData_Numeric("ldvolts")
End Property

Public Property Let VoltageMinimum_V(ByVal vData As Single)
    Dim s$
    
    s = "minvolts " & vData & "V"
    SendCmd s
End Property

'private commands ----------------

Private Sub SendCmd(ByVal sCmd As String)
    'append cartridge return to all command
    mSerial.Write sCmd + vbCr
    Sleep 10
End Sub

Private Function QueryData(ByVal sCmd As String) As String
    mSerial.ClearInputBuffer                    'clear echo
    SendCmd sCmd                                'send cmd
    sCmd = mSerial.Read                         'read echo
    sCmd = mSerial.Read                         'read data
    If Len(sCmd) > 1 Then sCmd = Mid(sCmd, 2)   'remove the leading Cr
    QueryData = Left(sCmd, Len(sCmd) - 1)       'remove the ending Cr
    Sleep 30                                    'this is necessary as HCA200 is slow
End Function

Private Function QueryData_Numeric(ByVal sCmd As String) As Double
    sCmd = QueryData(sCmd)
    QueryData_Numeric = Val(sCmd)
End Function


