VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iJDSU_HA9"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2004-5 W2Laboratories, www.w2lab.com                      ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const MyName$ = "JDSU HA9 Serial Optic Attenuator"

Private Const bitSettled% = &H4

Private mModelName As String

Private mGPIB As cwGPIB

'initialize this class
Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal bRaiseError As Boolean) As Boolean
    Dim s$, sData$()
    
    'first ini
    On Error GoTo ErrHandler
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT10s
        .Configure
        .Clear
        
        'get instrument type
       'get instrument type
        .Write "IDN?"
        s = .Read
        sData = Split(s, ",")
        mModelName = sData(1)
        
        Initialize = True
    End With
    
ErrHandler:
    If bRaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

Public Property Get ModelName() As String
    ModelName = mModelName
End Property

Public Property Let Attenuation_dB(ByVal vData As Single)
    Dim s$
    s = "ATT " & vData
    mGPIB.Write s
    WaitComplete
End Property

Public Property Get Attenuation_dB() As Single
    Attenuation_dB = QueryData_Numeric("ATT?")
End Property

Public Property Let Attenuation_Offset_dB(ByVal vData As Single)
    Dim s$
    s = "CAL " & vData
    mGPIB.Write s
End Property

Public Property Get Attenuation_Offset_dB() As Single
    Attenuation_Offset_dB = QueryData_Numeric("CAL?")
End Property

Public Property Let BeamBlocked(ByVal vData As Boolean)
    Dim s$
    s = "D " & IIf(vData, 1, 0)
    mGPIB.Write s
End Property

Public Property Get BeamBlocked() As Boolean
    Dim s$
    s = QueryData("D?")
    BeamBlocked = (s = "1")
End Property

Public Property Let Power_dBm(ByVal vData As Single)
    Dim s$
    s = "PWR " & vData
    mGPIB.Write s
End Property

Public Property Get Power_dBm() As Single
    Power_dBm = QueryData_Numeric("PWR?")
End Property

Public Property Let Power_Offset_dBm(ByVal vData As Single)
    Dim s$
    s = "PCAL " & vData
    mGPIB.Write s
End Property

Public Property Get Power_Offset_dBm() As Single
    Power_Offset_dBm = QueryData_Numeric("PCAL?")
End Property

Public Property Let Wavemength_nm(ByVal vData As Integer)
    Dim s$
    Dim OldAtten!
    
    'save current attenuation
    OldAtten = Attenuation_dB
    'set cprrect wavelength
    s = "WVL " & (CSng(vData) / 1E+09!)
    mGPIB.Write s
    'keep the same attenuation
    Attenuation_dB = OldAtten
End Property

Public Property Get Wavemength_nm() As Integer
    Dim s$
    s = QueryData("WVL?")
    If IsNumeric(s) Then
        Wavemength_nm = CSng(s) * 1E+09!
    End If
End Property

Public Sub Reset()
    mGPIB.Write "Reset"
End Sub

Public Sub DisplayPower()
    mGPIB.Write "DISP 1"
End Sub

Public Sub DisplayAttenuation()
    mGPIB.Write "DISP 0"
End Sub

Private Function QueryData(ByVal sCmd As String) As String
    mGPIB.Write sCmd
    sCmd = mGPIB.Read
    QueryData = Left(sCmd, Len(sCmd) - 2)
End Function

Private Function QueryData_Numeric(ByVal sCmd As String) As Single
    sCmd = QueryData(sCmd)
    If IsNumeric(sCmd) Then QueryData_Numeric = CSng(sCmd)
End Function

Private Sub WaitComplete()
    Dim bStatus&
    Dim MyStart!, Mytime!
    
    Const TimeOutInSec = 10!
    
    On Error GoTo ErrHandler
    
    'do serial pool to see if the command has completed
    MyStart = Timer
    While ((bStatus And bitSettled) <> bitSettled) And Mytime < TimeOutInSec
        bStatus = mGPIB.SerialPoll
        'Debug.Print bStatus
        Sleep 10   'do not lock up system
        Mytime = (Timer - MyStart)
    Wend
    
ErrHandler:
    If Err.Number = 30336 Then Err.Number = 0
End Sub

