VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iKeithley590"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2004 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

' This is a class for Keithley 590 CV meter

Public Enum K590RangeEnum
    k590_R_2pf = 1
    k590_R_20pf = 2
    k590_R_200pf = 3
    k590_R_2000pf = 4
    
    k590_R_Min = k590_R_2pf
    k590_R_Max = k590_R_2000pf
End Enum

Public Enum K590SampleRateEnum
    k590_S_1000Hz = 0
    k590_S_75Hz = 1
    k590_S_18Hz = 2
    k590_S_10Hz = 3
    k590_S_1Hz = 4
End Enum

Public Enum K590FrequencyEnum
    k590_F_100kHz = 0
    k590_F_1MHz = 1
    k590_F_NoTest = 2
End Enum

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const MyName$ = "Keithley 590"
Private Const DataError$ = "E+29"
Private Const DataError2$ = "99.9"

Private mGPIB As cwGPIB
Private mModelName As String


'initialize this class
Public Function Initialize(ByRef cwGPIB As cwGPIB, ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal bRaiseError As Boolean) As Boolean
    Dim s$
    
    'first ini
    On Error GoTo ErrHandler
    Set mGPIB = cwGPIB
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
        .Clear
        
        'get instrument type
        s = QueryData("U0")
        mModelName = s
       
        'set default settings
        WriteCmd "G1O0,0"     'output format and data being reported
        '           ^ this is latter O not numeric 0
        
        'returm
        Initialize = True
    End With
    
ErrHandler:
    If bRaiseError And Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function

Public Property Let FilterEnabled(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "P" & IIf(vData, "1", "0")
    WriteCmd sCmd
End Property

'range
Public Property Let AutoRange(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "R" & IIf(vData, "0", "9")
    WriteCmd sCmd
End Property

Public Property Get AutoRange() As Boolean
    Dim s$
    
    s = QueryData("U2")
    s = Mid(s, 7, 1)
    AutoRange = (s = "1")
End Property

Public Property Let Range(ByVal vData As K590RangeEnum)
    WriteCmd "R" & vData
End Property

Public Property Get Range() As K590RangeEnum
    Dim s$
    
    s = QueryData("U2")
    s = Mid(s, 5, 1)
    Range = CInt(s) + 1
End Property

Public Property Get Overflow() As Boolean
    Dim s$
    
    EnableMeasurement
    Sleep 500
    s = ReadData
    'Debug.Print s
    Overflow = (InStr(1, s, DataError) > 0) Or (InStr(1, s, DataError2) > 0)
End Property

Public Property Get Underflow() As Boolean
    Dim s$
    Dim iRange!
    Dim Value!
    
    iRange = Range
    Select Case iRange
        Case k590_R_2pf:      iRange = 2
        Case k590_R_20pf:    iRange = 20
        Case k590_R_200pf:   iRange = 200
        Case k590_R_2000pf:  iRange = 2000
    End Select
    
    EnableMeasurement
    Sleep 500
    s = ReadData
    Value = Val(s)
    Value = Value * 1E+12!
    
    Underflow = (Value < 0.3 * iRange)
    
End Property

Public Property Let TestSignalFreq(ByVal vData As K590FrequencyEnum)
    WriteCmd "F" & vData
End Property

'Public Property Get TestSignalFreq() As K590FrequencyEnum
'     Dim s$
'
'    s = QueryData("U2")
'    s = Mid(s, 5, 1)
'    Range = CInt(s) + 1
'End Property

Public Sub FindBestRange()
    Dim iRange As K590RangeEnum
    
    'iRange = k590_R_Min
    'Range = iRange
    
    iRange = Range
    
    While Overflow And iRange <= k590_R_Max
        iRange = iRange + 1
        Range = iRange
    Wend
    
    While Underflow And iRange >= k590_R_Min
        iRange = iRange - 1
        Range = iRange
    Wend
    
End Sub

'bias
Public Sub EnableExternalBias()
    WriteCmd "W4N1"
End Sub
    
'measure
Public Sub EnableMeasurement()
    WriteCmd "T0,0"
End Sub

Public Property Let SampleRate(ByVal vData As K590SampleRateEnum)
    WriteCmd "S" & vData
End Property

Public Property Get SampleRate() As K590SampleRateEnum
    Dim s$
    
    s = QueryData("U2")
    s = Mid(s, 11, 1)
    SampleRate = CInt(s)
End Property


'measure, return false for overflow
Public Function Measure(ByRef Cap_pf As Single, ByRef G As Single, ByRef Bias_V As Single, _
                        ByVal Samples As Integer) As Boolean
    Dim s$
    Dim sData$()
    Dim iRange%
    Dim i%
    
    Dim bStatus As Byte
    Const bReady = &H8
     
    s = ReadData
    sData = Split(s, ",")
    While InStr(1, sData(0), DataError) > 0
        iRange = Range
        If iRange <= k590_R_Max Then
           Range = iRange + 1
           EnableMeasurement
           Sleep 100
           s = ReadData
        Else
            Exit Function
        End If
    Wend
    
    Cap_pf = 0
    G = 0
    Bias_V = 0
    
    For i = 1 To Samples
        s = ReadData
        sData = Split(s, ",")
        
        Cap_pf = Cap_pf + CSng(sData(0)) * 1E+12!
        G = G + CSng(sData(1))
        Bias_V = Bias_V + CSng(sData(2))
    Next i
    
    Cap_pf = Cap_pf / Samples
    G = G / Samples
    Bias_V = Bias_V / Samples
    

    Measure = True
End Function

Public Sub TuneOnManualTrigger()
    WriteCmd "H23"
End Sub

'private
Private Function QueryData(ByVal sCmd As String) As String
    WriteCmd sCmd
    sCmd = mGPIB.Read
    QueryData = Left(sCmd, Len(sCmd) - 2)
End Function

Private Function ReadData() As String
    Dim bStatus As Byte
    Const bReady = &H8
    
    ReadData = mGPIB.Read
    While (bStatus And bReady) = 0
        bStatus = mGPIB.SerialPoll
    Wend
    
End Function

Private Sub WriteCmd(ByVal sCmd As String)
    Dim bStatus As Byte
    Const bReady = &H10
    
    mGPIB.Write sCmd + "X" + vbCrLf
    
    While (bStatus And bReady) = 0
        bStatus = mGPIB.SerialPoll
        Sleep 50
    Wend
    
End Sub



