VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iKeithleySwitch"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit


'----------------------------------------------------------------------------------
'------- (C) 1997-2001 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for Keithley 7000 serial switch boxes

Private Const s7002$ = "MODEL 7002"
Private Const s7001$ = "MODEL 7001"
Private Const s7011$ = "7011"
Private Const s7035$ = "7035"
Private Const s7012$ = "7012"
Private Const s7062$ = "7062"
Private Const s7053$ = "7053"

Private mInstrType As String
Private mCardType() As String
Private mList As String

Private mGPIB As CWGPIB

Public Function Initialize(ByRef CWGPIB As CWGPIB, _
                           ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
      
    On Error GoTo ErrHandler
    
    Dim s$, sData$()
    
    Set mGPIB = CWGPIB
        
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT1s
        .Configure
        .Clear
        
        'get instrument type
        .Write "*IDN?"
        s = NiceRead
        sData = Split(s, ",")
        mInstrType = sData(1)
        
        Select Case mInstrType
            Case s7001
            Case s7002
            Case Else
                s = "Instrument <" & s & "> is not supported."
                Err.Raise 9000, "Keithley Switching Box Class", s
        End Select
         
        'get card type
        .Write "*OPT?"
        s = NiceRead
        mCardType = Split(s, ",")
        
    End With
    
    Initialize = True
    Exit Function

ErrHandler:
    If RaiseError Then
        Err.Raise Err.Number, Err.Source, Err.Description
    End If
End Function

'instrument type
Public Property Get InstrumentType() As String
    InstrumentType = mInstrType
End Property

'card type
Public Property Get CardType(ByVal SlotNumber As Integer) As String
    Dim Index%

    Index = SlotNumber - 1     'array is 0 based
    If Index >= 0 And Index <= UBound(mCardType) Then
        CardType = mCardType(Index)
    Else
        CardType = "Invalid Slot Number: " & SlotNumber
    End If
End Property

'open all
Public Sub OpenAll()
    mGPIB.Write ":OPEN ALL"
End Sub

'open/close a channel
Public Function CloseChannel(ByVal slot As Integer, ByVal Row As Integer, _
                             Optional ByVal Col As Integer = -1) As Boolean
    Dim sCmd$
    
    sCmd = BuildConnectStr(slot, Row, Col)
    sCmd = ":CLOSE (@" & sCmd & ");CLOSE? (@" & sCmd & ")"
    
    mGPIB.Write sCmd
    CloseChannel = (NiceRead = 1)
End Function

Public Function OpenChannel(ByVal slot As Integer, ByVal Row As Integer, _
                             Optional ByVal Col As Integer = -1) As Boolean
    Dim sCmd$
    
    sCmd = BuildConnectStr(slot, Row, Col)
    sCmd = ":OPEN (@" & sCmd & ");OPEN? (@" & sCmd & ")"
    
    mGPIB.Write sCmd
    OpenChannel = (NiceRead = 1)
End Function

'list operation
Public Sub ListClear()
    mList = ""
End Sub

Public Sub ListAdd(ByVal slot As Integer, ByVal Row As Integer, Optional ByVal Col As Integer = -1)
    mList = mList & BuildConnectStr(slot, Row, Col) & ","
End Sub

Public Sub ListClose()
    Dim sCmd$
    
    If mList <> "" Then
        sCmd = Left(mList, Len(mList) - 1)
        sCmd = ":CLOSE (@" & sCmd & ")"
        mGPIB.Write sCmd
    End If
    
End Sub

Public Sub ListOpen()
    Dim sCmd$
    
    If mList <> "" Then
        sCmd = Left(mList, Len(mList) - 1)
        sCmd = ":OPEN (@" & sCmd & ")"
        mGPIB.Write sCmd
    End If
End Sub

Public Function ListStatus() As String
    Dim sCmd$
    
    If mList <> "" Then
        sCmd = Left(mList, Len(mList) - 1)
        sCmd = ":CLOSE? (@" & sCmd & ")"
        mGPIB.Write sCmd
        ListStatus = NiceRead
    End If
End Function

'build connect string
Private Function BuildConnectStr(ByVal slot As Integer, ByVal Row As Integer, _
                                 Optional ByVal Col As Integer = -1) As String
    
    'no card
    If Not IsNumeric(Me.CardType(slot)) Then
        Err.Raise 9001, mInstrType, "Card is not present in slot: " & slot
    
    'multiplexer
    ElseIf Col = -1 Then
        If Row < 0 Or Row > 40 Then
            Err.Raise 9002, mInstrType, "Invalid channel number: " & Row
        Else
            BuildConnectStr = slot & "!" & Row
        End If
    
    'matrix
    Else
        If Row < 0 Or Row > 4 Then
            Err.Raise 9002, mInstrType, "Invalid row number: " & Row
        ElseIf Col < 0 Or Col > 10 Then
            Err.Raise 9002, mInstrType, "Invalid col number: " & Col
        Else
            BuildConnectStr = slot & "!" & Row & "!" & Col
        End If
    
    End If

End Function

'nice read,
Private Function NiceRead() As String
    Dim s$
    s = mGPIB.Read
    NiceRead = Left(s, Len(s) - 1)
End Function
