VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iLuna"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit


'----------------------------------------------------------------------------------
'------- (C) 2003-2004 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a class for Luna OVA with limited functions
'Hardware interface is LAN with NI VISA

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const TimeoutStd& = 60000        '10 sec

Public Enum LunaOperationTypeEnum
    lunaMeas = 0
    lunaCalFull = 1
    lunaCalIntnl = 2
End Enum

Public Enum LunaMeasurementTypeEnum
    lunaMeasRefle = 0
    lunaMeasTrans = 1
End Enum

Public Enum LunaDataTypeEnum
    lunaIL = 0
    lunaGD = 1
    lunaCD = 2
    lunaPDL = 3
    lunaPMD = 4
    lunaLPD = 5         'linear phase delay
    lunaQPD = 6         'quadratic phase delay
    lunaMatrixAmp = 7
    lunaMatrixPhase = 8
    lunaTDAmp = 9
    lunaTDWav = 10
End Enum

Dim mModelName As String

Dim mVisa As cwVISA

Public Function Initialize(ByRef cwVISA As cwVISA, _
                           ByVal RsrcName As String, _
                           Optional ByVal bRaiseError As Boolean) As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mVisa = cwVISA
    With mVisa
        .RsrcName = RsrcName
        .Open True          'lock the machine exclusively
    End With
         
    'get instrument type, test communication
    mVisa.Timeout = 1000            'short timeout to see if I can lock the machine
    s = QueryData("*IDN?")
    mModelName = s
    mVisa.Timeout = TimeoutStd      'reset to std timeout
    
    'set
    MeasurementType(lunaMeas) = lunaMeasTrans
    Enabled_Laser = True
    Enabled_Filter = True
    
    Initialize = True
    
    Exit Function

ErrHandler:
    If bRaiseError Then MsgBox Err.Number & vbCrLf & Err.Description
End Function

'average
Public Property Let Enabled_Average(ByVal vData As Boolean)
    Dim sCmd$

    sCmd = IIf(vData, "1", "0")
    sCmd = "CONF:AVGE " + sCmd
    WritePort sCmd
End Property

Public Property Get Enabled_Average() As Boolean
    Enabled_Average = QueryData("CONF:AVGE?") = "1"
End Property

'average number
Public Property Let NumOfAverage(ByVal OperationType As LunaOperationTypeEnum, ByVal vData As Integer)
    Dim sCmd$

    If vData < 1 Then vData = 1
    sCmd = "CONF:AVGS " & vData & "," & OperationType
    WritePort sCmd
End Property

Public Property Get NumOfAverage(ByVal OperationType As LunaOperationTypeEnum) As Integer
    Dim sCmd$

    sCmd = "CONF:AVGS? " & OperationType
    NumOfAverage = QueryData_Numeric(sCmd)
End Property

'laser, system
Public Property Let Enabled_Laser(ByVal vData As Boolean)
    Dim sCmd$

    sCmd = IIf(vData, "1", "0")
    sCmd = "SYST:LASE " + sCmd
    WritePort sCmd
End Property

Public Property Get Enabled_Laser() As Boolean
    Enabled_Laser = QueryData("SYST:LASE?") = "1"
End Property

'DUT length
Public Property Let Enabled_AutoFindDUTLength(ByVal vData As Boolean)
    Dim sCmd$

    sCmd = IIf(vData, "1", "0")
    sCmd = "CONF:FDUT " + sCmd
    WritePort sCmd
End Property

Public Property Get Enabled_AutoFindDUTLength() As Boolean
    Enabled_AutoFindDUTLength = QueryData("CONF:FDUT?") = "1"
End Property

Public Property Let FindDUTLengthWithBroadband(ByVal vData As Boolean)
    Dim sCmd$

    sCmd = IIf(vData, "0", "1")
    sCmd = "CONF:DUTM " + sCmd
    WritePort sCmd
End Property

Public Property Get FindDUTLengthWithBroadband() As Boolean
    FindDUTLengthWithBroadband = QueryData("CONF:DUTM?") = "0"
End Property

Public Property Let DUTLength_m(ByVal vData As Single)
    WritePort "CONF:DUTL " & vData
End Property

Public Property Get DUTLength_m() As Single
    'force Luna to find DUT length automatically
    WritePort "CONF:DUTL"
    'query the result
    DUTLength_m = QueryData_Numeric("CONF:DUTL?")
End Property

'time domain segment length
Public Property Get TD_SegmentLengthList() As String
    TD_SegmentLengthList = QueryData("CONF:ARES?")
End Property

Public Property Let TD_SegmentLength_ns(ByVal vData As Single)
    WritePort "CONF:SRES " & vData
End Property

Public Property Get TD_SegmentLength_ns() As Single
    TD_SegmentLength_ns = QueryData_Numeric("CONF:SRES?")
End Property

'Filter
    'unit
Public Property Let RBW_Unit_pm(ByVal vData As Boolean)
    Dim sCmd$

    sCmd = IIf(vData, "0", "1")
    sCmd = "CONF:RSBU " + sCmd
    WritePort sCmd
End Property

Public Property Get RBW_Unit_pm() As Boolean
    RBW_Unit_pm = QueryData("CONF:RSBU?") = "0"
End Property

    'smoothing filter
Public Property Let Enabled_Filter(ByVal vData As Boolean)
    Dim sCmd$

    sCmd = IIf(vData, "1", "0")
    sCmd = "CONF:AAF " + sCmd
    WritePort sCmd
End Property

Public Property Get Enabled_Filter() As Boolean
    Enabled_Filter = QueryData("CONF:AAF?") = "1"
End Property

Public Property Let RBW_Filter(ByVal vData As Single)
    WritePort "CONF:FRBW " & vData
End Property

Public Property Get RBW_Filter() As Single
    RBW_Filter = QueryData("CONF:FRBW?")
End Property
    
    'time domain filter
Public Property Let Enabled_TimeDomainWindow(ByVal vData As Boolean)
    Dim Enabled$
    Dim sCmd$

    'apply
    Enabled = IIf(vData, "1", "0")
    sCmd = "CONF:TDW " + Enabled
    WritePort sCmd
    
    'retain setting
    sCmd = "CONF:TRET " & Enabled
    WritePort sCmd
End Property

Public Property Get Enabled_TimeDomainWindow() As Boolean
    Dim Enabled As Boolean
     
    Enabled = QueryData("CONF:TDW?") = "1"
    If Enabled Then Enabled = QueryData("CONF:TRET?") = "1"
    
    Enabled_TimeDomainWindow = Enabled
End Property

Public Sub LoadCalFile()
    WritePort "SYST:RLDC"
End Sub

Public Property Let TimeDomainWindow_Limits_Str(ByVal vData As String)
    WritePort "CONF:TLIM " + vData
End Property

Public Property Get TimeDomainWindow_Limits_Str() As String
    TimeDomainWindow_Limits_Str = QueryData("CONF:TLIM?")
End Property

Public Sub GetTimeDomainWindow_Limits(ByRef xLow As Single, ByRef xHigh As Single)
    Dim s$
    Dim sData$()
    
    s = QueryData("CONF:TLIM?")
    sData = Split(s, ",")
    xLow = sData(0)
    xHigh = sData(1)

End Sub

Public Sub SetTimeDomainWindow_Limits(ByVal PctFromCenter As Single)
    Dim X!(), Y!()
    Dim i&, ii&
    Dim xMax!, Max!
    Dim xLower!, xUpper!

    'Get TDW data
    GetTraceData lunaTDAmp, X, Y
    
    'find maximum
    ii = UBound(X)
    Max = Y(0)
    xMax = X(0)
    For i = 1 To ii
        If Y(i) > Max Then
            Max = Y(i)
            xMax = X(i)
        End If
    Next i
    
    'calculate the limits
    xLower = xMax * (1! - PctFromCenter)
    If xLower < X(0) Then xLower = X(0)
    xUpper = xMax * (1! + PctFromCenter)
    If xUpper > X(ii) Then xUpper = X(ii)
    
    'set limits
    WritePort "CONF:TLIM " & xLower & "," & xUpper
End Sub

Public Property Let TimeDomainWindow_Sigma_ns(ByVal vData As Single)
    WritePort "CONF:TSIG " & vData
End Property

Public Property Get TimeDomainWindow_Sigma_ns() As Single
    TimeDomainWindow_Sigma_ns = QueryData_Numeric("CONF:TSIG?")
End Property

Public Property Get RBW_TimeDomainWindow() As Single
    RBW_TimeDomainWindow = QueryData("CONF:TWRB?")
End Property

    'convolved resolution band width
Public Property Get RBW_Convolved() As Single
    RBW_Convolved = QueryData("CONF:CRBW?")
End Property

Public Property Get ErrorMsg() As String
    ErrorMsg = QueryData("SYST:ERR?")
End Property

Public Property Let MeasurementType(ByVal OperationType As LunaOperationTypeEnum, _
                                    ByVal vData As LunaMeasurementTypeEnum)
    Dim sCmd$

    sCmd = "CONF:TEST " & vData & "," & OperationType
    WritePort sCmd
End Property

Public Property Get MeasurementType(ByVal OperationType As LunaOperationTypeEnum) As LunaMeasurementTypeEnum
    Dim sCmd$

    sCmd = "CONF:TEST? " & OperationType
    MeasurementType = QueryData_Numeric(sCmd)
End Property

'-----------------------------------------------scan-------------------------------------------
'ready for scan
Public Property Get ReadyForScan() As Boolean
    Dim IsReady As Boolean

    'laser is on
    IsReady = QueryData("SYST:LASE?") = "1"

    'laser at operation temperature
    If IsReady Then IsReady = QueryData("SYST:WARM?") = "1"

    'optics are aligned
    If IsReady Then IsReady = QueryData("SYST:ALIG?") = "1"

    'luna in cal
    'If IsReady Then IsReady = QueryData("SYST:CAL?") = "1"

    'ready to scan
    If IsReady Then IsReady = QueryData("SYST:RDY?") = "1"

    'return
    ReadyForScan = IsReady
End Property

Public Function Scan() As Boolean
    Dim sCmd$
    Dim Delay!
    
    Const DelayPer_nm = 200
    
    'calculate needed delay
    Delay = NumOfAverage(lunaMeas) * ScanSpan_nm(lunaMeas) * DelayPer_nm
    mVisa.Timeout = Delay * 200  'over estimate
    
    'scan
    mVisa.Write "SCAN"
    'wait
    Sleep Delay
    'check
    WaitComplete

    'reset
    mVisa.Timeout = TimeoutStd
End Function

Public Property Let ScanStart_nm(ByVal vData As Single)
    Dim sCmd$

    sCmd = "CONF:STAR " + CStr(vData)
    WritePort sCmd
End Property

Public Property Get ScanStart_nm() As Single
    ScanStart_nm = QueryData_Numeric("CONF:STAR?")
End Property

Public Property Let ScanStop_nm(ByVal vData As Single)
    Dim sCmd$

    sCmd = "CONF:END " + CStr(vData)
    WritePort sCmd
End Property

Public Property Get ScanStop_nm() As Single
    ScanStop_nm = QueryData_Numeric("CONF:END?")
End Property

Public Property Let ScanCenter_nm(ByVal OperationType As LunaOperationTypeEnum, ByVal vData As Single)
    Dim sCmd$

    sCmd = "CONF:CWL " & vData & "," & OperationType
    WritePort sCmd
End Property

Public Property Get ScanCenter_nm(ByVal OperationType As LunaOperationTypeEnum) As Single
    Dim sCmd$

    sCmd = "CONF:CWL? " & OperationType
    ScanCenter_nm = QueryData_Numeric(sCmd)
End Property

'note that the Span is not a continus parameter. Luna only allow the following numbers
'1.25, 2.50, 5.01, 10.06, 20.25, 41.04
'the specific number is perhaps system dependent. It is better quary the number to confirm
Public Property Let ScanSpan_nm(ByVal OperationType As LunaOperationTypeEnum, ByVal vData As Single)
    Dim sCmd$

    sCmd = "CONF:RANG " & vData & "," & OperationType
    WritePort sCmd
End Property

Public Property Get ScanSpan_nm(ByVal OperationType As LunaOperationTypeEnum) As Single
    Dim sCmd$

    sCmd = "CONF:RANG? " & OperationType
    ScanSpan_nm = QueryData_Numeric(sCmd)
End Property

'data
Public Sub GetTraceData(ByVal DataType As LunaDataTypeEnum, ByRef Freq_GHz() As Single, ByRef Y() As Single)
    Dim Buff&
    Dim s$
    Dim sData$()
    Dim i&, ii&
    Dim wStart!, wStep!

    'each data point has 18 digit including Cr
    Const ElementSize& = 18

    'get data size
    s = "FETC:FSIZ?"
    i = QueryData_Numeric(s)
    Buff = CLng(i) * ElementSize
    
    'array size
    ii = i - 1
    ReDim Freq_GHz(ii)
    ReDim Y(ii)
    
    'get x           X unit is GHz for frequency and ns for time domain
    s = IIf(DataType < lunaTDAmp, "1", "3")
    WritePort "FETC:XAXI? " + s
    s = mVisa.Read(Buff)
    sData = Split(s, vbCr)
    For i = 0 To ii
        Freq_GHz(i) = sData(i)
    Next i
    
    'get y
    WritePort "FETC:MEAS? " & DataType
    s = mVisa.Read(Buff)
    sData = Split(s, vbCr)
    For i = 0 To ii
        Y(i) = sData(i)
    Next i
    
End Sub

Public Sub GetTraceData_YOnly(ByVal DataType As LunaDataTypeEnum, ByRef Y() As Single)
    Dim Buff&
    Dim s$
    Dim sData$()
    Dim i%, ii%
    Dim wStart!, wStep!

    'each data point has 18 digit including Cr
    Const ElementSize& = 18

    'get data size
    s = "FETC:FSIZ?"
    i = QueryData_Numeric(s)
    Buff = CLng(i) * ElementSize
    
    'array size
    ii = i - 1
    ReDim Freq_GHz(ii)
    ReDim Y(ii)
    
    'get y
    WritePort "FETC:MEAS? " & DataType
    s = mVisa.Read(Buff)
    sData = Split(s, vbCr)
    For i = 0 To ii
        Y(i) = sData(i)
    Next i
    
End Sub

'-----------------------------------------------------------------------
Private Function QueryData(ByVal sCmd As String) As String
    WritePort sCmd
    sCmd = ReadPort
    If sCmd <> "" Then QueryData = Left(sCmd, Len(sCmd) - 1)
End Function

Private Function QueryData_Numeric(ByVal sCmd As String) As Single
    Dim Value!

    sCmd = QueryData(sCmd)
    If IsNumeric(sCmd) Then
        Value = CSng(sCmd)
        QueryData_Numeric = Value
    End If
End Function

Private Function ReadPort() As String
    ReadPort = mVisa.Read
End Function

Private Sub WritePort(ByVal sCmd As String)
    mVisa.Write sCmd
    Sleep 100       'the instrument response if kind of slow
End Sub

Private Sub WaitComplete()
    Dim OPC$
    Dim MyStart!, Mytime!
    
    Const TimeOutInSec = 60!
    
    'On Error GoTo ErrHandler
    
    'query OPC status
    MyStart = Timer
    While OPC <> "1" And Mytime < TimeOutInSec
        OPC = QueryData("*OPC?")
        Mytime = (Timer - MyStart)
        'Debug.Print "throughput = " & Mytime
    Wend
    
ErrHandler:
    If Err.Number = 30336 Then Err.Number = 0
End Sub

Private Sub Class_Terminate()
    If Not mVisa Is Nothing Then mVisa.Close
End Sub
