VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iNPPowerMeter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2001-2005 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'----------------------------------------------------------------------------------
' This is a class for 4 models of optic power meter from Newport.    --------------
' It can automatically detect the model number and communicate with  --------------
' the instrument accordingly. So one version of code can be used for --------------
' different types of instrument so long as the instrument function   --------------
' needed is supported by the hardware. Many advanced functions are   --------------
' not available for the old 1830C model.                             --------------
'----------------------------------------------------------------------------------

Public Enum npPowerMeterUnitEnum
    npUnit_mW = 1
    npUnit_dB = 2
    npUnit_dBm = 3
    npunit_REL = 4
End Enum

Public Enum npPowerMeterFilterEnum
    npf_None = 0
    
    npf_Analog = 1
    npf_Digital = 2
    npf_APlusD = 3
    
    npf_1830_Slow = 1
    npf_1830_Medium = 2
    npf_1830_Fast = 3
    
End Enum

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const CVacuum# = 299792458
Private Const Nair# = 1.00026407

Private Const s2835C = "2835-C"
Private Const s2832C = "2832-C"
Private Const s1835C = "1835-C"
Private Const s1830C = "1830-C"

Private Const bitNDAchA% = &H1
Private Const bitNDAchB% = &H4
Private Const bitMAV% = &H10
Private Const mTimeOut% = 1000 'ms

Private mPort As Control
'Private mSerial As CWSerial
'Private mGPIB As cwGPIB
Private mIsRS232 As Boolean
Private mStrCh$
Private mModel$

Public Function Initialize(ByRef cwGPIB_or_cwSerial As Control, _
                           ByVal BoardNum_Or_ComPort As Integer, _
                           ByVal InstrAdrs_or_BuadRate As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
      
    On Error GoTo ErrHandler
    
    Dim s$, sData$()
    
    If TypeOf cwGPIB_or_cwSerial Is cwGPIB Then
        Set mPort = cwGPIB_or_cwSerial
    ElseIf TypeOf cwGPIB_or_cwSerial Is CWSerial Then
        Set mPort = cwGPIB_or_cwSerial
        mIsRS232 = True
    Else
        
    End If
        
    With mPort
        If mIsRS232 Then
            .ComPort = BoardNum_Or_ComPort
            .BaudRate = InstrAdrs_or_BuadRate
            .Parity = cwParityNone
            .DataBits = cwDataBitsEight
            .StopBits = cwStopBitsOne
            .Configure
            'disable echo
            .Write "ECHO0" & vbCrLf     'works for 1835, 2832, 2835 models
            .Write "E0" & vbCrLf        'works for 1830
        Else
            .BoardNumber = BoardNum_Or_ComPort
            .PrimaryAddress = InstrAdrs_or_BuadRate
            .TimeoutValue = cwGPIBTimeoutT1s
            .Configure
            .Clear
        End If
        
        'get instrument type
        .Write "*IDN?"
        On Error Resume Next        ' <- special handling for model 1830-C
        Sleep 100
        s = .Read
        If Err.Number = 0 Then
            sData = Split(s, ",")
            mModel = sData(1)
        End If
        
        If mModel = "" Then
            .Write "L?"
            Sleep 100
            s = ReadData
            If s = "0" Or s = "1" Then mModel = s1830C
        End If
        
        On Error GoTo ErrHandler
        Select Case mModel
            Case s2835C:  mStrCh = "_A"     'default to channel A
            Case s1835C:  mStrCh = ""
            Case s1830C:  mStrCh = ""
            Case s2832C:  mStrCh = "_A"
            Case Else:
                s = "Model " & mModel & " is not supported"
                Err.Raise -221, "Mewport Power Meter Class", s
        End Select
        
        'reset to default condition
        '.Write "*RST"
        '.Write "*CLS"
        Me.Wavelength_nm = 1550
    End With
    
    Initialize = True
    Exit Function

ErrHandler:
    If RaiseError Then
        Err.Raise Err.Number, Err.Source, Err.Description
    End If
End Function

'initialize meter setting by passing
'a semi-column delimited string of instrument dependent GPIB command
Public Sub InitialSetting(ByVal sIni As String)
    Dim cmdBuff$()
    Dim i%
    
    With mPort
        cmdBuff = Split(sIni, ";")
        For i = 0 To UBound(cmdBuff)
            .Write cmdBuff(i) & vbCrLf
        Next i
    End With
End Sub

'query instrument model
Public Property Get ModelName() As String
    ModelName = mModel
End Property

'display which channel
Public Property Let DisplayedChannel(ByVal vData As Integer)
    Dim sCmd$
    If mModel = s2835C Or mModel = s2832C Then
        sCmd = IIf(vData = 1, "DISPCH ""A"" ", "DISPCH ""B"" ")
        mPort.Write sCmd & vbCrLf
    End If
End Property

Public Property Get DisplayedChannel() As Integer
    Dim sCH$
    Const chA = "A"
    
    If mModel = s2835C Or mModel = s2832C Then
        mPort.Write "DISPCH?" & vbCrLf
        sCH = ReadData
    Else
        sCH = chA
    End If
    DisplayedChannel = IIf(sCH = chA, 1, 2)
End Property

Public Property Get DisplayedChannel_String() As String
    Dim ChNum%
    
    ChNum = Me.DisplayedChannel
    DisplayedChannel_String = IIf(ChNum = 1, "CHA", "CHB")
        
End Property

'set meter channel
Public Property Let MeterChannelNumber(ByVal vData As Integer)
    If mModel = s2835C Or mModel = s2832C Then
        mStrCh = IIf(vData = 1, "_A", "_B")
    End If
End Property

Public Property Get MeterChannelNumber() As Integer
    Dim i%
    Select Case mStrCh
        Case "":    i = 0
        Case "_A":  i = 1
        Case "_B":  i = 2
    End Select
    MeterChannelNumber = i
End Property

'get detector serial #
Public Property Get DetectorSN() As String
    If mModel <> s1830C Then
        mPort.Write "DETSN" + mStrCh + "?" & vbCrLf
        DetectorSN = ReadData
    End If
End Property

'filter
Public Property Let Filter(ByVal vData As npPowerMeterFilterEnum)
    Dim sCmd
    
    If mModel <> s1830C Then
        sCmd = "FILTER" & mStrCh & " " & vData & vbCrLf
    ElseIf vData <> npf_None Then
        sCmd = "F" & vData & vbCrLf
    End If
    
    If sCmd <> "" Then mPort.Write sCmd
    
End Property

'ACQ mode
Public Property Let Continuous(ByVal vData As Boolean)
    Dim sCmd
    
    If mModel <> s1830C Then
        sCmd = IIf(vData, """DCCONT""", """DCSNGL""")
        sCmd = "MODE" & mStrCh & " " & sCmd & vbCrLf
    Else
        sCmd = "G" & IIf(vData, "1", "0")
    End If
    
    mPort.Write sCmd
End Property

'sample precision
Public Property Let HighPrecision(ByVal vData As Boolean)
    Dim sCmd
    
    If mModel <> s1830C Then
        sCmd = IIf(vData, "20000", "4096")
        sCmd = "SPREC " & sCmd & vbCrLf
        mPort.Write sCmd
    End If

End Property

Public Property Get HighPrecision() As Boolean
    Dim sCmd$

    If mModel <> s1830C Then
        sCmd = "SPREC?" + vbCrLf
        mPort.Write sCmd
        sCmd = ReadData
        HighPrecision = (sCmd = "20000")
    End If
End Property

Public Property Let AverageTime_ms(ByVal vData As Single)
    Dim sCmd$
    
    If mModel <> s1830C Then
        'convert to frequency in Hz
        vData = 1000! / vData
        sCmd = "SFREQ " & vData & vbCrLf
        mPort.Write sCmd
    End If

End Property

Public Property Get AverageTime_ms() As Single
    Dim sCmd$
    Dim V!

    If mModel <> s1830C Then
        sCmd = "SFREQ?" + vbCrLf
        mPort.Write sCmd
        V = ReadData
        AverageTime_ms = 1000! / V
    End If
End Property

'unit
Public Property Let Unit(ByVal vData As npPowerMeterUnitEnum)
    Dim sCmd$

    If mModel = s1830C Then
        sCmd = "U" & vData & vbCrLf
    Else
        Select Case vData
            Case npUnit_mW:     sCmd = """W"""
            Case npUnit_dB:     sCmd = """dB"""
            Case npUnit_dBm:    sCmd = """dBm"""
            Case npunit_REL:    sCmd = """REL"""
        End Select
        sCmd = "Units" & mStrCh & " " & sCmd & vbCrLf
    End If
    
    mPort.Write sCmd
End Property

Public Property Get Unit() As npPowerMeterUnitEnum
    Dim sCmd$

    If mModel = s1830C Then
        sCmd = "U?" & vbCrLf
    Else
        sCmd = "Units" & mStrCh & "?" & vbCrLf
    End If
    mPort.Write sCmd
    sCmd = ReadData
    
    Select Case sCmd
        Case "W":       Unit = npUnit_mW
        Case "dB":      Unit = npUnit_dB
        Case "dBm":     Unit = npUnit_dBm
        Case "REL":     Unit = npunit_REL
    End Select
      
End Property

'range
Public Property Let Range(ByVal vData As Integer)
    Dim sCmd$
    
    If vData < 0 Then Exit Property
        
    If mModel = s1830C Then
        sCmd = "R" & vData & vbCrLf
    Else
        sCmd = "RANGE" & mStrCh & " " & vData & vbCrLf
    End If
    mPort.Write sCmd
End Property

Public Property Get Range() As Integer
    Dim sCmd$

    If mModel = s1830C Then
        sCmd = "R?" + vbCrLf
    Else
        sCmd = "RANGE" + mStrCh + "?" + vbCrLf
    End If
    mPort.Write sCmd
    Range = ReadData
End Property

'auto range
Public Property Let AutoRange(ByVal vData As Boolean)
    Dim sCmd$
    
    If mModel = s1830C Then
        sCmd = IIf(vData, "0", "8")
        sCmd = "R" & sCmd & vbCrLf
    Else
        sCmd = IIf(vData, "1", "0")
        sCmd = "AUTO" & mStrCh & " " & sCmd & vbCrLf
    End If
    
    mPort.Write sCmd
End Property

Public Property Get AutoRange() As Boolean
    Dim sCmd$
    
    If mModel = s1830C Then
        sCmd = "R?" & vbCrLf
        mPort.Write sCmd
        AutoRange = (ReadData = 0)
    Else
        sCmd = "AUTO" + mStrCh + "?" + vbCrLf
        mPort.Write sCmd
        AutoRange = (ReadData = 1)
    End If
End Property

'wave length
Public Property Let Frequency_GHz(ByVal vData As Double)
    Me.Wavelength_nm = CVacuum / Nair / vData
End Property

Public Property Get Frequency_GHz() As Double
    Frequency_GHz = CVacuum / Nair / Me.Wavelength_nm
End Property

Public Property Let Wavelength_nm(ByVal vData As Double)
    Dim sCmd$
    Dim Lambda%
    
    Lambda = CInt(vData)
    If mModel = s1830C Then
        sCmd = "W" & Lambda & vbCrLf
    Else
        sCmd = "LAMBDA" & mStrCh & " " & Lambda & vbCrLf
    End If
    mPort.Write sCmd
End Property

Public Property Get Wavelength_nm() As Double
    Dim sCmd$
  
    If mModel = s1830C Then
        sCmd = "W?" & vbCrLf
    Else
        sCmd = "LAMBDA" & mStrCh & "?" & vbCrLf
    End If
    
    mPort.Write sCmd
    Wavelength_nm = ReadData
End Property

'reference
Public Sub StoreReference()
    Dim sCmd$
    
    If mModel = s1830C Then
        'use current data for reference
        sCmd = "S"
    Else
        'use current data for reference
        sCmd = "STOREF" + mStrCh + vbCrLf
        mPort.Write sCmd
        'select reference source
        sCmd = "REFSEL" + mStrCh + " ""STOREF""" + vbCrLf
        mPort.Write sCmd
    End If
        
    mPort.Write sCmd
    
End Sub

Public Sub StoreReferenceCustom(ByVal ReferenceValue As Single)
    Dim sCmd$
    
    If mModel = s1830C Then
        'no such capability
    Else
        'set value
        sCmd = "USRREF" + mStrCh + " " + CStr(ReferenceValue) + vbCrLf
        mPort.Write sCmd
        'select reference source
        sCmd = "REFSEL" + mStrCh + " ""USRREF""" + vbCrLf
        mPort.Write sCmd
    End If
        
    mPort.Write sCmd
End Sub

'zero
Public Property Let ZeroMeter(ByVal vData As Boolean)
    Dim sCmd$
    
    If mModel = s1830C Then
        sCmd = "Z" + IIf(vData, "1", "0")
    Else
        If vData Then
            'use current reading as zero reference
            sCmd = "STOZERO" + mStrCh + vbCrLf
            mPort.Write sCmd
        End If
        sCmd = "ZERO" + mStrCh + IIf(vData, " 1", " 0")
    End If
        
    mPort.Write sCmd
    
End Property

Public Property Get ZeroMeter() As Boolean
    Dim sCmd$
    
    If mModel = s1830C Then
        sCmd = "Z?"
    Else
        sCmd = "ZERO" + mStrCh + "?" + vbCrLf
    End If
        
    mPort.Write sCmd
    ZeroMeter = (ReadData = "1")
End Property

'measure
Public Property Get Power_mW() As Single
    Power_mW = 1000! * Me.Measure
End Property

Public Function MeasPower(Optional ByVal Samples As Integer = 1) As Single
    Dim i%
    Dim Value!

    For i = 1 To Samples
        Value = Value + Me.Measure
    Next i
    MeasPower = Value / Samples
    
End Function

Public Function Measure() As Single
    Dim sCmd$
    Dim bitNDA%
    Dim StatusByte%
    Dim sReading$
    Dim aReading$()
    
     Select Case mStrCh
        Case "", "_A":  bitNDA = bitNDAchA
        Case "_B":      bitNDA = bitNDAchB
    End Select

TryAgain:

    If mModel = s1830C Then
        sCmd = "D?" + vbCrLf
    Else
        sCmd = "RWS" + mStrCh + "?" + vbCrLf
    End If
    
    With mPort
        'do query
        .Write sCmd
        'wait until data arrives
        StatusByte = 0
        Do Until (StatusByte And bitNDA)
            If mIsRS232 Then
                'it seems that the meter is slow enough to do power measurement
                StatusByte = bitNDA
            Else
                If mModel = s1830C Then
                    'there is no NDA bit for 1830
                    StatusByte = bitNDA
                    Sleep 30
                Else
                    Sleep 2
                    StatusByte = .SerialPoll
                End If
            End If
            'Debug.Print "C" & StatusByte
        Loop
        'read
        sReading = ReadData
        'parse
        If mModel = s1830C Then
            Measure = CSng(sReading)
        Else
            aReading = Split(sReading, ",")
            Select Case CInt(aReading(0))
                'OK
                Case 0
                    Measure = CSng(aReading(1))
                'Over Range
                Case 1
                    'Err.Raise "-222", mModel, mModel & " out of range"
                    GoTo TryAgain
                'saturated
                Case 2
                    'Err.Raise "-223", mModel, mModel & " Data saturated"
                    GoTo TryAgain
                'ring
                Case 4
                    'Err.Raise "-224", mModel, mModel & " Data ringing"
                    'Measure = -50
                    GoTo TryAgain
            End Select
        End If
    End With
End Function

Private Function ReadData() As String
    Dim s$
    Dim bStatus As Byte
    Dim Mytime!
    Dim MyStart!
    
    'Debug.Print "B" & Timer
    
    'wait until data is available
    If Not mIsRS232 Then
        MyStart = Timer
        While ((bStatus And bitMAV) <> bitMAV) And Mytime < mTimeOut
            bStatus = mPort.SerialPoll
            Mytime = (Timer - MyStart) * 1000
            'Debug.Print bStatus
        Wend
    End If
    
    'read
    s = mPort.Read
    s = Left(s, Len(s) - 1)
    
    'strip quto
    If Left(s, 1) = """" Then
        s = Mid(s, 2, Len(s) - 2)
    End If
    
    'return
    ReadData = s
    
    'Debug.Print "A" & Timer
End Function
