VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iNewportDiodeCtrl"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2005-2006 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'----------------------------------------------------------------------------------
' This class controls the Newport 3000-9000 Laser Diode Controllers
' ---------------------------------------------------------------------------------

Public Enum LaserControlModeEnum
    Laser_Ctrl_Const_Current_CW = 0
    Laser_Ctrl_Const_Current_HBW = 1
    Laser_Ctrl_Const_Current_LBW = 2
    Laser_Ctrl_Const_Feedback_Current = 3
    Laser_Ctrl_Const_Feedback_Power = 4
End Enum

Public Enum TECSensorTypeEnum
    TEC_Sensor_None = 0
    TEC_Sensor_ThermistorHighCurrent = 1
    TEC_Sensor_ThermistorLowCurrent = 2
    TEC_Sensor_LM335 = 3
    TEC_Sensor_AD590 = 4
    TEC_Sensor_RTD = 5
End Enum

Public Enum TECControlModeEnum
    TEC_Ctrl_Const_Current = 0
    TEC_Ctrl_Const_Resistance = 1
    TEC_Ctrl_Const_temperature = 2
End Enum

Private Const ErrSensorShort% = &H20
Private Const ErrSensorOpen% = &H40
Private Const ErrTECOpen% = &H80

Private Const ErrCurrentLimit% = &H1
Private Const ErrVoltageLimit% = &H2
Private Const ErrDetectorCurrentLimit% = &H4
Private Const ErrDetectorPowerLimit% = &H8
Private Const ErrOpenCircuit% = &H80
Private Const ErrShortCircuit% = &H100


Private mModel As String
Private mModelNumber As Integer
Private mGPIB As cwGPIB

Public Function Initialize(ByRef cwGPIB As cwGPIB, _
                           ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mGPIB = cwGPIB
                          
    With mGPIB
        'configure GPIB link
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT10s
        .Configure
                          
        'get model number
        mModel = QueryData("*IDN?")
        s = Mid(mModel, 9, 4)
        If IsNumeric(s) Then
            mModelNumber = CInt(s)
        Else
            MsgBox "Wrong instrument model " + mModel, vbInformation + vbOKOnly, "iNewportDiodeCtrl"
            Exit Function
        End If
        
    End With
    
    Initialize = True
    
ErrHandler:
    If RaiseError Then Err.Raise Err.Number, Err.Source, Err.Description
            
End Function

'--------------------------------------------------------------------------
'system level functions
'--------------------------------------------------------------------------
Public Property Get ModuleList() As String
    ModuleList = QueryData("EQUIP?")
End Property

Public Property Let DisplayEnabled(ByVal vData As Boolean)
    mGPIB.Write "LAS:DIS 1"
    mGPIB.Write "TEC:DIS 1"
    mGPIB.Write "MASTER"
End Property

'--------------------------------------------------------------------------
'diode driver functions
'--------------------------------------------------------------------------
Public Property Let LaserChannel(ByVal vData As Integer)
     mGPIB.Write "LAS:CHAN " & vData
End Property

Public Property Get LaserChannel() As Integer
    LaserChannel = QueryDataNumeric("LAS:CHAN?")
End Property

Public Property Let LaserCtrlMode(ByVal vData As LaserControlModeEnum)
    Dim sCmd$
    
    Select Case vData
        Case Laser_Ctrl_Const_Current_CW:           sCmd = "ICW"
        Case Laser_Ctrl_Const_Current_HBW:          sCmd = "IHBW"
        Case Laser_Ctrl_Const_Current_LBW:          sCmd = "ILBW"
        Case Laser_Ctrl_Const_Feedback_Current:     sCmd = "MDI"
        Case Laser_Ctrl_Const_Feedback_Power:       sCmd = "MDP"
    End Select
    
    sCmd = "LAS:MODE:" + sCmd
    mGPIB.Write sCmd
End Property

Public Property Get LaserCtrlMode() As LaserControlModeEnum
    Dim sCmd$
    Dim Mode%
    
    sCmd = "LAS:MODE?"
    sCmd = QueryData(sCmd)
    sCmd = UCase(sCmd)
    Select Case sCmd
        Case "ICW":     Mode = Laser_Ctrl_Const_Current_CW
        Case "IHBW":    Mode = Laser_Ctrl_Const_Current_HBW
        Case "ILBW":    Mode = Laser_Ctrl_Const_Current_LBW
        Case "MDI":     Mode = Laser_Ctrl_Const_Feedback_Current
        Case "MDP":     Mode = Laser_Ctrl_Const_Feedback_Power
    End Select
    
    LaserCtrlMode = Mode
End Property

Public Property Let LaserFeedbackSensitivity_uAmW(ByVal vData As Single)
    mGPIB.Write "LAS:CALMD " & vData
End Property

Public Property Get LaserFeedbackSensitivity_uAmW() As Single
    LaserFeedbackSensitivity_uAmW = QueryDataNumeric("LAS:CALMD?")
End Property

Public Property Let LaserEnabled(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "LAS:OUT " + IIf(vData, "1", "0")
    mGPIB.Write sCmd
End Property

Public Property Get LaserEnabled() As Boolean
    Dim sCmd$
    
    sCmd = "LAS:OUT?"
    sCmd = QueryData(sCmd)
    
    LaserEnabled = (sCmd = "1")
End Property

Public Property Get LaserErrorMsg() As String
    Dim sCmd$
    Dim Status&
    
    sCmd = "LAS:EVE?"
    Status = QueryDataNumeric(sCmd)
    
    Select Case True
        Case (Status And ErrOpenCircuit) = ErrOpenCircuit:                      sCmd = "Open Circuit"
        Case (Status And ErrShortCircuit) = ErrShortCircuit:                    sCmd = "Sensor Shorted"
        Case (Status And ErrCurrentLimit) = ErrCurrentLimit:                    sCmd = "Current Limit"
        Case (Status And ErrVoltageLimit) = ErrVoltageLimit:                    sCmd = "Sensor Shorted"
        Case (Status And ErrDetectorCurrentLimit) = ErrDetectorCurrentLimit:    sCmd = "Detector Current Limit"
        Case (Status And ErrDetectorPowerLimit) = ErrDetectorPowerLimit:        sCmd = "Detector Power Limit"
        Case Else:                                                              sCmd = "Error " & Status
    End Select
    
    LaserErrorMsg = sCmd
End Property


Public Property Let LaserCurrent_Setpoint_mA(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "LAS:LDI " + CStr(vData)
    mGPIB.Write sCmd
End Property

Public Property Get LaserCurrent_Setpoint_mA() As Single
    Dim sCmd$
    
    sCmd = "LAS:SET:LDI?"
    LaserCurrent_Setpoint_mA = QueryDataNumeric(sCmd)
End Property

Public Property Get LaserCurrent_Measured_mA() As Single
    Dim sCmd$
    
    sCmd = "LAS:LDI?"
    LaserCurrent_Measured_mA = QueryDataNumeric(sCmd)
End Property

Public Property Get LaserVoltage_Measured_V() As Single
    Dim sCmd$
    
    sCmd = "LAS:LDV?"
    LaserVoltage_Measured_V = QueryDataNumeric(sCmd)
End Property

Public Property Let MonitorCurrent_Setpoint_uA(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "LAS:MDI " + CStr(vData)
    mGPIB.Write sCmd
End Property

Public Property Get MonitorCurrent_Setpoint_uA() As Single
    Dim sCmd$
    
    sCmd = "LAS:SET:MDI?"
    MonitorCurrent_Setpoint_uA = QueryDataNumeric(sCmd)
End Property

Public Property Get MonitorCurrent_Measured_uA() As Single
    Dim sCmd$
    
    sCmd = "LAS:MDI?"
    MonitorCurrent_Measured_uA = QueryDataNumeric(sCmd)
End Property

Public Property Let MonitorPower_Setpoint_mW(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "LAS:MDP " + CStr(vData)
    mGPIB.Write sCmd
End Property

Public Property Get MonitorPower_Setpoint_mW() As Single
    Dim sCmd$
    
    sCmd = "LAS:SET:MDP?"
    MonitorPower_Setpoint_mW = QueryDataNumeric(sCmd)
End Property

Public Property Get MonitorPower_Measured_mW() As Single
    Dim sCmd$
    
    sCmd = "LAS:MDP?"
    MonitorPower_Measured_mW = QueryDataNumeric(sCmd)
End Property

'--------------------------------------------------------------------------
'TEC functions
'--------------------------------------------------------------------------
Public Property Let TECChannel(ByVal vData As Integer)
     mGPIB.Write "TEC:CHAN " & vData
End Property

Public Property Get TECChannel() As Integer
    TECChannel = QueryDataNumeric("TEC:CHAN?")
End Property


Public Property Let TECCtrlMode(ByVal vData As TECControlModeEnum)
    Dim sCmd$
    
    Select Case vData
        Case TEC_Ctrl_Const_Current:        sCmd = "ITE"
        Case TEC_Ctrl_Const_Resistance:     sCmd = "R"
        Case TEC_Ctrl_Const_temperature:    sCmd = "T"
    End Select
    
    sCmd = "TEC:MODE:" + sCmd
    mGPIB.Write sCmd
End Property

Public Property Get TECCtrlMode() As TECControlModeEnum
    Dim sCmd$
    Dim Mode%
    
    sCmd = "TEC:MODE?"
    sCmd = QueryData(sCmd)
    
    Select Case sCmd
        Case "ITE":     Mode = TEC_Ctrl_Const_Current
        Case "R":       Mode = TEC_Ctrl_Const_Resistance
        Case "T":       Mode = TEC_Ctrl_Const_temperature
    End Select
    
    TECCtrlMode = Mode
End Property

Public Property Let TECEnabled(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "TEC:OUT " + IIf(vData, "1", "0")
    mGPIB.Write sCmd
End Property

Public Property Get TECEnabled() As Boolean
    Dim sCmd$
    
    sCmd = "TEC:OUT?"
    sCmd = QueryData(sCmd)
    
    TECEnabled = (sCmd = "1")
End Property

Public Property Get TECErrorMsg() As String
    Dim sCmd$
    Dim Status&
    
    sCmd = "TEC:EVE?"
    Status = QueryDataNumeric(sCmd)
    
    Select Case True
        Case (Status And ErrSensorOpen) = ErrSensorOpen:    sCmd = "Sensor Open"
        Case (Status And ErrSensorShort) = ErrSensorShort:  sCmd = "Sensor Shorted"
        Case (Status And ErrTECOpen) = ErrTECOpen:          sCmd = "TEC Module Open"
        Case Else:                                          sCmd = "Error " & Status
    End Select
    
    TECErrorMsg = sCmd
End Property

Public Property Let TECSensorType(ByVal vData As TECSensorTypeEnum)
    Dim sCmd$
    
    sCmd = "TEC:SEN " & vData
    mGPIB.Write sCmd
End Property

Public Property Get TECSensorType() As TECSensorTypeEnum
    Dim sCmd$
    
    sCmd = "TEC:SEN?"
    TECSensorType = QueryDataNumeric(sCmd)
End Property

Public Property Let Temperature_Setpoint(ByVal vData As Single)
    Dim sCmd$
    
    sCmd = "TEC:T " + CStr(vData)
    mGPIB.Write sCmd
End Property

Public Property Get Temperature_Setpoint() As Single
    Dim sCmd$
    
    sCmd = "TEC:SET:T?"
   Temperature_Setpoint = QueryDataNumeric(sCmd)
End Property

Public Property Get Temperature_Measured() As Single
    Dim sCmd$
    
    sCmd = "TEC:T?"
    Temperature_Measured = QueryDataNumeric(sCmd)
End Property

'-------------------------------------------------------------------------
Private Function QueryDataNumeric(ByVal sCmd As String) As Single
    Dim s$
    
    s = QueryData(sCmd)
    If IsNumeric(s) Then
        QueryDataNumeric = CSng(s)
    Else
        QueryDataNumeric = -9999
    End If
End Function

Private Function QueryData(ByVal sCmd As String) As String
    Dim s$
    
    mGPIB.Write sCmd
    s = mGPIB.Read
    If Right(s, 2) = vbCrLf Then s = Left(s, Len(s) - 2)
    
    QueryData = s
End Function
































