VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iOOSpectrpmeter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Const OOUbound% = 2047
Private Const OOMaxPower% = 4095
Private Const OOMinTime% = 1

'''''''''''''''''''''''''''''''''''''''''/
'' Driver commands
'''''''''''''''''''''''''''''''''''''''''/
Private Const CMD_NONE = 0       ' no specific command
Private Const CMD_SETUP = 1      ' force hardware setup
Private Const CMD_STATUS = 2     ' send status messages
Private Const CMD_TEST = 9       ' return test data

'''''''''''''''''''''''''''''''''''''''''/
'' Driver error codes
'''''''''''''''''''''''''''''''''''''''''/
Private Const ER_NOERROR = 0             ' no error
Private Const ER_TIMEOUT = 2             ' scan timed out
Private Const ER_BUSY = 3                ' driver busy acquiring data
Private Const ER_SADSYNC = 4             ' SAD500 synchronization error
Private Const ER_MEMORY = 5              ' memory buffers not allocated
Private Const ER_UNKNOWN = 6             ' unknown application instance
Private Const ER_TIMER = 7               ' no timers available
Private Const ER_ERROR = 8               ' unknown error
Private Const ER_NOTHREAD = 9            ' unable to create Single Scan thread
Private Const ER_NIR_NOT_FOUND = 10      ' unable to find NIR attached to system
Private Const ER_USB2000_NOT_FOUND = 11  ' unable to find USB2000 attached to system
Private Const ER_ADC1000USB_NOT_FOUND = 12 ' unable to find ADC1000-USB attached to system
Private Const ER_HR2000_NOT_FOUND = 13   ' unable to find HR2000 attached to system

'''''''''''''''''''''''''''''''''''''''''/
'' Spectrometer Types
'''''''''''''''''''''''''''''''''''''''''/
Private Const S1000 = &H0             ' S1000/PC1000/PC1000 spectrometers
Private Const S2000BT = &H100         ' S2000BT spectrometer
Private Const S2000 = &H200           ' S2000/PC2000 spectrometers
Private Const S1024DW = &H300         ' S1024DW spectrometer (SAD500 only!)
Private Const NIR512 = &H400          ' NIR512 spectrometer
Private Const NIR256 = &H500
Private Const S4000 = &H600
Private Const QE65000 = &H700

'''''''''''''''''''''''''''''''''''''''''/
'' A/D adapter types
'''''''''''''''''''''''''''''''''''''''''/
Private Const ADC500 = &H0                            ' ADC500/PC1000
Private Const DAQ700 = &H1                            ' DAQ700
Private Const ADC1000 = &H2                           ' ADC1000/PC2000
Private Const SAD500 = &H3                            ' SAD500
Private Const USB2000 = &H4                           ' USB2000/HR2000
Private Const SERIALUSB = &H5                         ' SERIAL USB2000
Private Const ADC2000PCI = &H6                        ' ADC1000-PCI
Private Const ADC1000USB = &H7                        ' ADC1000-USB
Private Const SERIALADC = &H8                         ' all serial A/D converters
Private Const HR2000 = &H9                            ' HR2000
Private Const PC104 = &HA                             ' PC104
Private Const NIRADC = &HB                            ' NIRADC
Private Const HR4000 = &HC
Private Const HR2000PLUS = &HD
Private Const QEADC = &HE
Private Const USB2000PLUS = &HF
Private Const USB4000 = &H10
Private Const NO_ADC = &HFF

'''''''''''''''''''''''''''''''''''''''''/
'' SCANPARM structure for DoScan acquisition mode
'''''''''''''''''''''''''''''''''''''''''/
Private Type SCANPARM
  cmd As Integer                        ' command, use CMD_ constants
  fdc As Integer                        ' flash delay in msec
  dsf As Integer                        ' sampling frequency (kHz - S1000, msec - S2000)
  boxcar As Integer                     ' boxcar smoothing width
  average As Integer                    ' samples to average
  chan_ena(3) As Integer                ' spectrometer channel enabled array
  scan_dark As Integer                  ' scan dark
  correct_dark As Integer               ' correct for electrical dark signal
  extrig As Integer                     ' external trigger mode
  upper4chan As Integer                 ' flag true if want data from S2000 channels 4-7
  sdat(0 To OOUbound, 0 To 3) As Single     ' spectral data arrays
  UseUSecIntegrationTime As Long        ' special integration times for HR4000,HR2000+,QE65000
  USecIntegrationTime As Long
End Type

Private Type CoefDataType
    'SerialNumber(15) As Byte
    SerialNumber  As String * 16
    WLIntercept   As Single
    WLFirst       As Single
    WLSecond      As Single
    WLThird       As Single
    SLCoefficient As Single
    NL0           As Single
    NL1           As Single
    NL2           As Single
    NL3           As Single
    NL4           As Single
    NL5           As Single
    NL6           As Single
    NL7           As Single
    NLOrder       As Integer
End Type

Private Declare Function OOI_DoScan Lib "OOIDrv32.dll" Alias "#18" _
    (sp As SCANPARM) As Integer

Private Declare Sub OOI_Config Lib "OOIDrv32.dll" Alias "#19" _
    (ByVal spec As Integer, ByVal adc As Integer, _
     ByVal irq As Integer, ByVal baseadd As Integer)

Private Declare Function OOI_GetSpectrometerType Lib "OOIDrv32.dll" Alias "#29" () As Integer

Private Declare Function OOI_GetADCType Lib "OOIDrv32.dll" Alias "#30" () As Integer

Private Declare Function HR2000_GetCoefficients Lib "OOIDrv32.dll" Alias "#700" _
    (con As CoefDataType) As Long

'SerialNUmber shall be a 16-character fixed length string
'Private Declare Function HR2000_IdentifyHR2000 Lib "OOIDrv32.dll" Alias "#702" _
'    (ByVal devnum As Integer, ByVal SerialNumber As String) As Long

'Private Declare Sub HR2000_GetFirmwareVersion Lib "OOIDrv32.dll" Alias "#703" (ver As Byte)

'--------------------------------------------------------------------------------
Public Enum OOSpectrometerTypeEnum
    oo_SpecType_S1000 = S1000           ' S1000/PC1000/PC1000 spectrometers
    oo_SpecType_S2000BT = S2000BT       ' S2000BT spectrometer
    oo_SpecType_S2000 = S2000           ' S2000/PC2000/HR2000 spectrometers
    oo_SpecType_S4000 = S4000           ' S4000/PC4000/HR4000 spectrometers
    oo_SpecType_S1024DW = S1024DW       ' S1024DW spectrometer (SAD500 only!)
    oo_SpecType_NIR512 = NIR512         ' NIR512 spectrometer
    oo_SpecType_NIR256 = NIR256
    oo_SpecType_QE65000 = QE65000
End Enum

Public Enum OOADCTypeEnum
    oo_ADCType_HR2000 = HR2000
    oo_ADCType_HR4000 = HR4000
End Enum

'----------------------------------------------------------------------------------
Private Type OOSpecInfo
    Wavelength(OOUbound)    As Single
    Power(OOUbound)         As Single
    
    WLStart         As Single
    
    iMax            As Integer
    iMin            As Integer
    
    ThresholdPct    As Single
    ThresholdAbs    As Single
    NoiseLevel      As Single
    HalfMaximum     As Single
    
    Centroid        As Single
    
    FWHMLeft        As Single
    FWHMRight       As Single
    
    ThWidthLeft     As Single
    ThWidthRight    As Single
    
    IntTimeMax      As Integer
End Type

Private mSpecPara As SCANPARM
Private mCoef As CoefDataType
Private mSpecInfo As OOSpecInfo
'----------------------------------------------------------------------------------

Public Function Initialize() As Boolean
    Dim L&
    Dim i%
    Dim tmp!

    'build the 1st scan para
    With mSpecPara
        .cmd = CMD_NONE
        .fdc = 0                    ' flash delay, 0 for S2000
        .dsf = 5                    ' integration time in msec (S2000)/ A/D rate in kHz (S1000)
        .average = 0                ' scans to average
        .boxcar = 0                 ' boxcar smoothing width
        .chan_ena(0) = 1            ' enable wanted channel - default to 0/Master
        .chan_ena(1) = 0            ' disable all channels
        .chan_ena(2) = 0            ' disable all channels
        .chan_ena(3) = 0            ' disable all channels
        .scan_dark = 0              ' not scanning a dark spectrum
        .correct_dark = 0           ' do not correct for electrical dark
        .extrig = 0                 ' no external triggering
    End With
    
    ' do an initial DoScan() to complete the USB device initialization
    L = OOI_DoScan(mSpecPara)
    If L <> ER_NOERROR Then Exit Function
    
    ' read coef
    i = Me.ADCType
    Select Case i
        Case oo_ADCType_HR2000:     L = HR2000_GetCoefficients(mCoef)
        Case Else:                  L = 0
    End Select
    If (L <> 1) Then Exit Function

    'calculate the wavelength
    With mCoef
        L = UBound(mSpecInfo.Wavelength)
        For i = 0 To L
            tmp = .WLIntercept + .WLFirst * CSng(i) + .WLSecond * CSng(i ^ 2)
            mSpecInfo.Wavelength(i) = tmp + .WLThird * CSng(i ^ 3)
        Next i
    End With

    mSpecInfo.WLStart = mSpecInfo.Wavelength(0)

    Initialize = True
End Function

'instrument info
Public Function Config()
    Call OOI_Config(0, 0, 0, 0)
End Function

Public Property Get SpectrometerType() As OOSpectrometerTypeEnum
    SpectrometerType = OOI_GetSpectrometerType()
End Property

Public Property Get ADCType() As OOADCTypeEnum
    ADCType = OOI_GetADCType()
End Property

Public Property Get SerialNumber() As String
    Dim s$
    Dim p%
  
    'use info in mCeof
    s = mCoef.SerialNumber
    p = InStr(1, s, vbNullChar)
    If p > 0 Then s = left(s, p - 1)
    
    SerialNumber = s
End Property

Public Property Get InfoString() As String
    Dim s$
    
    s = Me.SerialNumber
    s = s & "   " & Format(mSpecInfo.Wavelength(0), "0.0")
    s = s & "-" & Format(mSpecInfo.Wavelength(Me.ArrayUbound), "0.0") & "nm"
    
    InfoString = s
End Property

Public Property Get ArrayUbound() As Integer
    ArrayUbound = UBound(mSpecInfo.Wavelength)
End Property

Public Property Get Wavelength_Min() As Single
    Wavelength_Min = mSpecInfo.Wavelength(0)
End Property

Public Property Get Wavelength_Max() As Single
    Wavelength_Max = mSpecInfo.Wavelength(Me.ArrayUbound)
End Property

'scan parameter
Public Property Let IntegrationTime_ms(ByVal vData As Integer)
    If vData < OOMinTime Then vData = OOMinTime
    mSpecPara.dsf = vData
End Property

Public Property Get IntegrationTime_ms() As Integer
    IntegrationTime_ms = mSpecPara.dsf
End Property

Public Property Let IntegrationTimeMax_ms(ByVal vData As Integer)
    mSpecInfo.IntTimeMax = vData
End Property

Public Property Get IntegrationTimeMax_ms() As Integer
    IntegrationTimeMax_ms = mSpecInfo.IntTimeMax
End Property

Public Property Let Wavelength_InterestStart(ByVal vData As Single)
    mSpecInfo.WLStart = vData
End Property

Public Property Get Wavelength_InterestStart() As Single
    Wavelength_InterestStart = mSpecInfo.WLStart
End Property

'scan
Public Function DoScan(ByRef Wavelength() As Single, Power() As Single) As Boolean
    Dim L&
    Dim LL&
    Dim Peak!
    
    'scan
TryScan:
    L = OOI_DoScan(mSpecPara)
    If L <> ER_NOERROR Then Exit Function
    
    'transfer data
    LL = UBound(mSpecPara.sdat)
    Peak = mSpecPara.sdat(0, 0)
    For L = 0 To LL
        mSpecInfo.Power(L) = mSpecPara.sdat(L, 0)
        'make sure peak is not saturated
        If Peak < mSpecInfo.Power(L) Then
            Peak = mSpecInfo.Power(L)
            If Peak > 0.98 * OOMaxPower And mSpecPara.dsf > OOMinTime Then
                mSpecPara.dsf = mSpecPara.dsf \ 2
                If mSpecPara.dsf < OOMinTime Then mSpecPara.dsf = OOMinTime
                GoTo TryScan
            End If
        End If
    Next L
    
        'make sure peak is not too small
        If Peak > 0 And Peak < 0.25 * OOMaxPower And mSpecPara.dsf < mSpecInfo.IntTimeMax Then
            'scale a new time
            mSpecPara.dsf = mSpecPara.dsf * 0.75 * OOMaxPower / Peak
            If mSpecPara.dsf > mSpecInfo.IntTimeMax Then mSpecPara.dsf = mSpecInfo.IntTimeMax
            GoTo TryScan
        End If
        
    'analyze
    AnalyzeSpec
        
    'return data
    Wavelength = mSpecInfo.Wavelength
    Power = mSpecInfo.Power
    
    DoScan = True
End Function

Public Property Get MaxIndex() As Integer
    MaxIndex = mSpecInfo.iMax
End Property

Public Property Get MinIndex() As Integer
    MinIndex = mSpecInfo.iMin
End Property

Public Property Get MaxPower() As Single
    MaxPower = mSpecInfo.Power(mSpecInfo.iMax)
End Property

Public Property Get HalfMaximum() As Single
    HalfMaximum = mSpecInfo.HalfMaximum
End Property

Public Property Get NoiseLevel()
    NoiseLevel = mSpecInfo.NoiseLevel
End Property

Public Property Let ThresholdPct(ByVal vData As Single)
    mSpecInfo.ThresholdPct = vData
End Property

Public Property Get ThresholdPct() As Single
    ThresholdPct = mSpecInfo.ThresholdPct
End Property

Public Property Get ThresholdAbs() As Single
    ThresholdAbs = mSpecInfo.ThresholdAbs
End Property

Public Property Get MaxWavelength() As Single
    MaxWavelength = mSpecInfo.Wavelength(mSpecInfo.iMax)
End Property

Public Property Get Centroid() As Single
    Centroid = mSpecInfo.Centroid
End Property

Public Property Get FWHM() As Single
    FWHM = mSpecInfo.FWHMRight - mSpecInfo.FWHMLeft
End Property

Public Property Get FWHMLeft() As Single
    FWHMLeft = mSpecInfo.FWHMLeft
End Property

Public Property Get FWHMRight() As Single
    FWHMRight = mSpecInfo.FWHMRight
End Property

Public Property Get ThresholdWidth() As Single
    ThresholdWidth = mSpecInfo.ThWidthRight - mSpecInfo.ThWidthLeft
End Property

Public Property Get ThresholdWidthLeft() As Single
    ThresholdWidthLeft = mSpecInfo.ThWidthLeft
End Property

Public Property Get ThresholdWidthRight() As Single
    ThresholdWidthRight = mSpecInfo.ThWidthRight
End Property

Private Sub AnalyzeSpec()
    Dim i%, ii%
    Dim iMin%, iMax%
    Dim Min!, Max!
    Dim Sum#, SumY#
    
    With mSpecInfo
        ii = UBound(.Power)
        
        'find min and max
        Min = .Power(i)
        iMin = 0
        Max = .Power(i)
        iMax = 0
        
        For i = 1 To ii
            If .Wavelength(i) > mSpecInfo.WLStart Then
                If .Power(i) > Max Then
                    Max = .Power(i)
                    iMax = i
                ElseIf .Power(i) < Min Then
                    Min = .Power(i)
                    iMin = i
                End If
            End If
        Next i
        
        .iMax = iMax
        .iMin = iMin
        
        'get noise level from two ends
        Sum = 0
        SumY = 0
        
        For i = 1 To 100
            Sum = Sum + .Power(i)
            SumY = SumY + 1
        Next i
        
        For i = (ii - 100) To ii - 1
            Sum = Sum + .Power(i)
            SumY = SumY + 1
        Next i
         
        .NoiseLevel = Sum / SumY
        
        'get half maximum
        .HalfMaximum = (.Power(.iMax) - .NoiseLevel) * 0.5
        .HalfMaximum = .NoiseLevel + .HalfMaximum
         
        'get FWHM points
        .FWHMLeft = GetYFromX(.Power, .Wavelength, .HalfMaximum, iMax, False)
        .FWHMRight = GetYFromX(.Power, .Wavelength, .HalfMaximum, iMax, True)
        
        'get threshold
        .ThresholdAbs = (.Power(.iMax) - .NoiseLevel) * .ThresholdPct
        .ThresholdAbs = .NoiseLevel + .ThresholdAbs
        
        'get width @ threshold
        .ThWidthLeft = GetYFromX(.Power, .Wavelength, .ThresholdAbs, iMax, False)
        .ThWidthRight = GetYFromX(.Power, .Wavelength, .ThresholdAbs, iMax, True)
             
         'calculate centroid
         Sum = .Wavelength(iMax) * (.Power(iMax) - .NoiseLevel)
         SumY = (.Power(iMax) - .NoiseLevel)
         
         'loop to lower end
         For i = iMax - 1 To 0 Step -1
             If .Power(i) < .ThresholdAbs Then Exit For
             Sum = Sum + .Wavelength(i) * (.Power(i) - .NoiseLevel)
             SumY = SumY + (.Power(i) - .NoiseLevel)
         Next i
         
         'loop to high end
         For i = iMax + 1 To ii
             If .Power(i) < .ThresholdAbs Then Exit For
             Sum = Sum + .Wavelength(i) * (.Power(i) - .NoiseLevel)
             SumY = SumY + (.Power(i) - .NoiseLevel)
         Next i
         
         'get centroid
         If SumY <> 0 Then .Centroid = Sum / SumY
         
    End With
End Sub

Private Function GetYFromX(ByRef X() As Single, ByRef Y() As Single, ByVal Xo As Single, _
                           ByVal StartIndex As Integer, ByVal Forward As Boolean) As Single
    Dim i%
    Dim iStart%, iStop%, iStep%
    Dim tmp!
    
    'search condition
    iStart = StartIndex
    If Forward Then
        iStop = UBound(X) - 1
        iStep = 1
    Else
        iStop = 0
        iStep = -1
    End If
    
    'find nearest neighbor
    For i = iStart To iStop Step iStep
        tmp = (X(i) - Xo) * (X(i + 1) - Xo)
        If tmp <= 0 Then Exit For
    Next i

    tmp = X(i) - X(i + 1)
    If tmp = 0 Then
        GetYFromX = 0.5 * (Y(i) + Y(i + 1))
    Else
        tmp = (Y(i) - Y(i + 1)) / tmp
        tmp = (Xo - X(i)) * tmp
        GetYFromX = Y(i) + tmp
    End If
    
End Function

Private Sub Class_Initialize()
    mSpecInfo.ThresholdPct = 0.1
    mSpecInfo.IntTimeMax = 1000
End Sub
