VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iOmegaCN9000"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2002 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for Omega CN9000 temperature controller
'it uses WAModbus.dll for Modbus communication interface through RS232


'DLL declare
Private Declare Function WA_openVBModbusConnection Lib "WAModbus.DLL" _
    (ByRef hComm As Long, ByVal PortStr As String, _
     ByVal BaudRate As Long, ByVal parity As Integer) As Integer

Private Declare Sub WA_closeModbusConnection Lib "WAModbus.DLL" (ByVal hComm As Long)

Private Declare Function WA_readModbusDevice Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByVal slaveAddress As Integer, _
     ByVal startAddress As Long, ByRef Data As Integer, _
     ByVal numPoints As Integer) As Integer
     
Private Declare Function WA_writeModbusDevice Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByVal slaveAddress As Integer, _
     ByVal startAddress As Long, ByRef Data As Integer, _
     ByVal numPoints As Integer) As Integer

Private Declare Function WA_readModbusDiagnostics Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByVal slaveAddress As Integer, _
     ByVal subFunction As Integer, ByRef Data As Integer) As Integer

Private Declare Function WA_getReadString Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByRef Data As Byte) As Integer
    
Private Declare Function WA_getWriteString Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByRef Data As Byte) As Integer
    
Private Declare Function WA_getReadResponse Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByRef Data As Byte) As Integer

Private Declare Function WA_getWriteResponse Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByRef Data As Byte) As Integer


'internal data
Private Const MyName As String = "Omega CN9000 Serial Thermal Controller"
Private Const mBaseAdrs As Long = 40001   'address offset for reg.
Private mhComm As Long          'port handler
Private mSlaveAdrs As Integer   'slave device address

Public Function Initialize(ByVal CommPortNum As Integer, ByVal BaudRate As Integer, _
                           ByVal SlaveAdrs As Integer, Optional RaiseError = True) As Boolean
    
    Dim result%
    Dim PortStr$
    Dim sErrorMsg$
    
    'check port num
    Select Case CommPortNum
        Case 1 To 32
            PortStr$ = "COM" & CommPortNum
        Case Else
            sErrorMsg = "Invalid comm port number: " & CommPortNum
            GoTo ErrHandler
    End Select
    
    'check buad rate
    Select Case BaudRate
        Case 1200, 2400, 4800, 9600, 19200
            'do nothing
        Case Else
            sErrorMsg = "Invalid baud rate: " & BaudRate
            GoTo ErrHandler
    End Select
    
    'check slave address
    Select Case SlaveAdrs
        Case 1 To 247
            mSlaveAdrs = SlaveAdrs
        Case Else
            sErrorMsg = "Invalid slave address for device: " & SlaveAdrs
            GoTo ErrHandler
    End Select
    
    'open port
    result = WA_openVBModbusConnection(mhComm, PortStr, BaudRate, 0&)
    
    'check error
    If result <> 0 Then
        sErrorMsg = "Can not open port for " & MyName
        GoTo ErrHandler
    End If
    
    'make setpoint low
    Me.SetPoint = 25
    Me.DisableOutput
    
    'set unit
    Me.DisplayDegC = True
    
    Initialize = True
    Exit Function
    
ErrHandler:
    If RaiseError Then MsgBox sErrorMsg, vbExclamation + vbOKOnly, MyName
End Function

Public Sub ClosePort()
    WA_closeModbusConnection mhComm
End Sub

'enable message send
Public Sub EnableMsgSent()
    Call Me.WriteModbus("0300", 5)
    Call Me.WriteModbus("1500", 5)
End Sub

'commit message sent
Public Sub CommitMsgSent()
    Call Me.WriteModbus("0300", 6)
    Call Me.WriteModbus("1600", 6)
End Sub

'DisplayUnit
Public Property Let DisplayDegC(ByVal vData As Boolean)
    Dim i%
    i = IIf(vData, 1, 2)
    Call Me.WriteModbus("0199", i)
End Property

'read actual temperature
Public Function ReadTemperature() As Single
    ReadTemperature = 0.1! * Me.ReadModbus("001C")
    'try again
    If ReadTemperature < 0 Then ReadTemperature = 0.1! * Me.ReadModbus("001C")
End Function

'auto tune
Public Sub EnableOutput()
    Call Me.WriteModbus("0189", 0)
End Sub

Public Sub DisableOutput()
    Call Me.WriteModbus("0189", 2)
End Sub

'start, stop, hold
Public Sub StartRamp()
    Call Me.WriteModbus("03D4", 1)
End Sub

Public Sub StopRamp()
    Call Me.WriteModbus("03D4", 0)
End Sub

Public Sub HoldRamp()
    Call Me.WriteModbus("03D4", 2)
End Sub

'set point - deg c
Public Property Let SetPoint(ByVal vData As Single)
    Call Me.WriteModbus("007F", vData * 10!)
End Property

Public Property Get SetPoint() As Single
    SetPoint = 0.1! * Me.ReadModbus("007F")
End Property

'Ramp rate - deg per min
Public Property Let RampRate(ByVal vData As Integer)
    'convert deg/min to deg/hour
    vData = vData * 60!
    If vData > 9995 Then vData = 9995
    Call Me.WriteModbus("02D0", vData)
End Property

Public Property Get RampRate() As Integer
    RampRate = Me.ReadModbus("02D0") / 60!
End Property

'soak time - minutes
Public Property Let SoakTime(ByVal vData As Single)
    If vData > 1440 Then vData = 1440
    Call Me.WriteModbus("02D2", vData * 10!)
End Property

Public Property Get SoakTime() As Single
    SoakTime = 0.1! * Me.ReadModbus("02D2")
End Property

'PID, ProportionalBand - DegC
Public Property Let ProportionalBand(ByVal vData As Single)
    Call Me.WriteModbus("0085", vData * 10!)
End Property

Public Property Get ProportionalBand() As Single
    ProportionalBand = 0.1! * Me.ReadModbus("0085")
End Property

'PID, Integral Time, minute
Public Property Let IntegralTime(ByVal vData As Single)
    
    Select Case vData
        Case Is <= 10: vData = vData * 10!
        Case Is <= 60: vData = vData + 90!
        Case Else:     vData = 60 + 90
    End Select

    Call Me.WriteModbus("018B", vData)
End Property

Public Property Get IntegralTime() As Single
    Dim t!
    t = Me.ReadModbus("018B")
    If t < 100 Then
        t = 0.1 * t
    Else
        t = t - 90!
    End If
    
    IntegralTime = t
End Property

'PID Derivate time, sec
Public Property Let DerivateTime(ByVal vData As Single)
    Call Me.WriteModbus("018C", vData)
End Property

Public Property Get DerivateTime() As Single
    DerivateTime = Me.ReadModbus("018C")
End Property

Public Property Let CycleTime(ByVal vData As Single)
   Select Case vData
        Case Is <= 10: vData = vData * 10!
        Case Is <= 60: vData = vData + 90!
        Case Else:     vData = 60 + 90
    End Select

    Call Me.WriteModbus("018D", vData)
End Property

Public Property Get CycleTime() As Single
    Dim t!
    t = Me.ReadModbus("018D")
    If t < 100 Then
        t = 0.1 * t
    Else
        t = t - 90!
    End If
    CycleTime = t
End Property

Public Property Let DAC(ByVal vData As Single)
    Call Me.WriteModbus("018A", vData * 2!)
End Property

Public Property Get DAC() As Single
    DAC = 0.5 * Me.ReadModbus("018A")
End Property

'generic read write
Public Function ReadModbus(ByVal AdrsInHex As String) As Integer
    Dim Adrs&
    Dim Data%
    Dim result%
    
    Adrs = GetAdrsLong(AdrsInHex)
    
    result = WA_readModbusDevice(mhComm, mSlaveAdrs, Adrs, Data, 1)
    If result = 0 Then
        ReadModbus = Data
    Else
        'MsgBox GetErrorStr(result, Data), vbCritical + vbOKOnly, MyName
        ReadModbus = -999
    End If
    
End Function

Public Sub WriteModbus(ByVal AdrsInHex As String, ByVal Data As Integer)
    Dim Adrs&
    Dim result%
    
    Adrs = GetAdrsLong(AdrsInHex)
    
    result = WA_writeModbusDevice(mhComm, mSlaveAdrs, Adrs, Data, 1)
    If result <> 0 Then
        'MsgBox GetErrorStr(result, Data), vbCritical + vbOKOnly, MyName
    End If
    
End Sub

Private Function GetAdrsLong(ByVal AdrsInHex As String) As Long
    GetAdrsLong = mBaseAdrs + CLng("&H" + AdrsInHex)
End Function

Private Function GetErrorStr(val%, dataVal%)
    Select Case (val)
        Case -6: GetErrorStr = "HOST_REJECTED_DATA_LENGTH"
        Case -5: GetErrorStr = "HOST_REJECTED_START_ADDRESS"
        Case -4: GetErrorStr = "HOST_RECEIVED_IO_ERR_SENDING"
        Case -3: GetErrorStr = "HOST_RECEIVED_MODBUS_ERROR"
        Case -2: GetErrorStr = "HOST_RECEIVED_TIMEOUT_ERROR"
        Case -1: GetErrorStr = "HOST_RECEIVED_CHECKSUM_ERROR"
    End Select
        
    Select Case dataVal
        Case -2: GetErrorStr = "HOST_RECEIVED_TIMEOUT_ERROR"
        Case -1: GetErrorStr = "HOST_RECEIVED_CHECKSUM_ERROR"
    End Select
End Function

Private Sub Class_Terminate()
    Me.ClosePort
End Sub
