VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iParker6K"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2005-2006 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a class for Parker/Compumotor 6K serial motion controller

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Public Enum CompMotorDriveTypeEnum
    CompMotor_Stepper = 0
    CompMotor_Servo = 1
End Enum

Private Const cstRetrialLimit% = 20
Private mModel As String
Private mNumOfAxis As Integer

Private mSerial As cwSerial

Public Function Initialize(ByRef cSerial As cwSerial, ByVal PortNum As Integer) As Boolean
    Dim success As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mSerial = cSerial
    
    With mSerial
        'communication info
        .ComPort = PortNum
        .DataBits = cwDataBitsEight
        .StopBits = cwStopBitsOne
        .Parity = cwParityNone
        'others
        .DataAsString = True
        .TimeoutValue = 1000
        .EOSChar = vbCr
         'try slow rate talk first
        .BaudRate = 9600
        .Configure
        success = sWrite("TREV")
        'ctrl is in slow mode, reset it to fast mode
'        If success Then
'            sWrite "BAUD115200"
'            Sleep 500
'        End If
'        'now try to do fast talk
'        .Reset
'        .BaudRate = 115200
'        .Configure
'        success = sWrite("TREV")
        'OK
        If success Then
            mModel = Me.sRead
        Else
            s = "Please make sure that the Parker motion controller powered on, "
            s = s + "and its RS232 cable is connected to computer com port "
            s = s & PortNum & "."
        End If
    End With
    
    Initialize = success
    
ErrHandler:
    If Err.Number <> 0 Then
        Err.Raise Err.Number, Err.Source, Err.Description, Err.HelpFile, Err.HelpContext
    End If
End Function



'----------------------------------------------------------------------------------------------------
'--------------------  DIO functions ----------------------------------------------------------------
'----------------------------------------------------------------------------------------------------
Public Property Let Dout(ByVal WhichBit As Integer, ByVal vData As Boolean)
    Dim sCmd$
    
    'If WhichBit < 1 Or WhichBit > 4 Then Exit Property
    sCmd = "OUT." & WhichBit & "-" & IIf(vData, 0, 1)
    sWrite sCmd
End Property

Public Property Get Dout(ByVal WhichBit As Integer) As Boolean
    Dim sCmd$
    Dim p%
    
    sCmd = "TOUT"
    If sWrite(sCmd) Then
        sCmd = sRead
        p = 5 + WhichBit
        sCmd = Mid(sCmd, p, 1)
        Dout = (sCmd = "0")
    End If
End Property

Public Function AssignDoutForMotion(ByVal WhichBit As Integer, ByVal WhichAxis As Integer) As Boolean
    Dim sCmd$
    
    sCmd = "OUTFNC" & WhichBit & "-" & WhichAxis & "B"
    sWrite sCmd
    sCmd = sRead
    AssignDoutForMotion = (sCmd = vbLf)
End Function

'Public Property Get DoutStatus() As Byte
'
'End Property

'----------------------------------------------------------------------------------------------------
'--------------------  Position functions -------------------------------------------------------
'----------------------------------------------------------------------------------------------------
Public Sub GetCurrentPosition(ByRef X As Single, ByRef Y As Single)
    Dim sCmd$
    Dim sData$()
    
    sCmd = "TPC"
    sWrite sCmd
    sCmd = sRead
    sData = Split(sCmd, ",")
    X = Mid(sData(0), 4)
    Y = sData(1)
End Sub

'----------------------------------------------------------------------------------------------------
'--------------------  Drive/Motion functions -------------------------------------------------------
'----------------------------------------------------------------------------------------------------
Public Property Let Axises(ByVal vData As Integer)
    mNumOfAxis = vData
End Property

Public Property Get Axises() As Integer
    Axises = mNumOfAxis
End Property

Public Function EnableDrive(ByVal WhichAxis As Integer, ByVal Enabled As Boolean) As Boolean
    Dim sCmd$
    
    'pad preceding axis with "X" - do not change symbol
    If WhichAxis > 0 Then
        sCmd = "DRIVE" + String(WhichAxis - 1, "X") + IIf(Enabled, "1", "0")
    Else
        sCmd = "DRIVE" + String(mNumOfAxis, IIf(Enabled, "1", "0"))
    End If
    
    EnableDrive = sWrite(sCmd)
End Function

Public Function DriveEnabled(ByVal WhichAxis) As Boolean
    Dim status%()
    
    'return is 0 based
    WhichAxis = WhichAxis - 1
    'do it
    ReadStatusBit 13, status
    If WhichAxis <= UBound(status) Then DriveEnabled = (status(WhichAxis) = 0)
End Function

Public Function DriveEnabledAll() As Boolean
    Dim status%()
    Dim i%
    'do it
    ReadStatusBit 13, status
    For i = 0 To UBound(status)
        If status(i) <> 0 Then Exit Function
    Next i
    DriveEnabledAll = True
End Function

Public Sub KillMotion()
    sWrite "!K"
End Sub

Public Function SetMotorType(ByVal WhichAxis As Integer, ByVal MotorType As CompMotorDriveTypeEnum) As Boolean
    Dim sCmd$
    'pad preceding axis with "X" - do not change symbol
    If WhichAxis > 0 Then
        sCmd = "AXSDEF" + String(WhichAxis - 1, "X") & MotorType
    Else
        sCmd = "AXSDEF" + String(mNumOfAxis, MotorType)
    End If
    SetMotorType = sWrite(sCmd)
End Function

Public Sub SetAcceleration(ByVal A1 As Single, ByVal A2 As Single)
    sWrite "A" & A1 & "," & A2
End Sub

Public Sub SetDistance(ByVal D1 As Single, ByVal D2 As Single)
    sWrite "D" & D1 & "," & D2
End Sub

Public Sub SetVelocity(ByVal V1 As Single, ByVal V2 As Single)
    sWrite "V" & V1 & "," & V2
End Sub

Public Property Let MoveAbsolute(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "MA" + IIf(vData, "11", "00")
    sWrite sCmd
End Property

Public Property Let MoveByDistance(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = "MC" + IIf(vData, "00", "11")
    sWrite sCmd
End Property

Public Sub MoveAndWait(ByVal D1 As Single, ByVal D2 As Single)
    Const bitMoving% = 1
    Dim status%()

    'move
    sWrite "D" & D1 & "," & D2
    Sleep 50
    sWrite "GO11"
    'wait, cehck for start bit
    ReDim status(2)
    status(0) = 1
    status(1) = 1
    While status(0) = 1 Or status(1) = 1
        Sleep 10
        DoEvents
        ReadStatusBit bitMoving, status
    Wend
End Sub

Public Sub Go()
    sWrite "GO11"
End Sub

Public Sub Home()
    Const bitMoving% = 1
    Dim status%()

    'home
    sWrite "HOM11"
    
    'wait, cehck for start bit
    ReDim status(2)
    status(0) = 1
    status(1) = 1
    While status(0) = 1 Or status(1) = 1
        Sleep 200
        DoEvents
        ReadStatusBit bitMoving, status
    Wend
End Sub

'----------------------------------------------------------------------------------------------------
'--------------------  Encoder/Feedback functions ---------------------------------------------------
'----------------------------------------------------------------------------------------------------



'----------------------------------------------------------------------------------------------------
'--------------------  Serial Communication functions -----------------------------------------------
'----------------------------------------------------------------------------------------------------
Public Function sWrite(ByVal sData As String, Optional RaiseError As Boolean = True) As Boolean
    Dim sRead$
    Dim counter%
    
    On Error GoTo ErrHandler
    
    With mSerial
        'write command
        .ClearInputBuffer
        sData = sData + vbCr
        .Write sData
        'check echo
        While sRead <> sData And counter < cstRetrialLimit
            'relax
            Sleep 10
            counter = counter + 1   'prevent dead cycle
            'check
            sRead = .Read
            'get rid of possible prompt ClearInputBuffer does done
            'as RS232 is slower than the PC
            'If Left(sRead, 1) = ">" Then sRead = Mid(sRead, 2)
        Wend
        'return
        sWrite = (counter < cstRetrialLimit)
    End With
    
    Exit Function

ErrHandler:
    'If RaiseError Then ErrorHandling
End Function

'generic read
Public Function sRead() As Variant
    Dim s$
    Dim L%
    
    'On Error GoTo ErrHandler
    
    s = mSerial.Read
    'truncate EOL character
    L = Len(s) - 1
    If L > 0 Then s = Left(s, L)
    'remove prompt and other characters, msg start with "*"
    L = InStr(1, s, "*")
    If L <> 0 Then
        sRead = Mid(s, L + 1)
    Else
        sRead = s
    End If
    
    Exit Function

ErrHandler:
    'ErrorHandling
End Function

Public Sub RunScript(ByVal sFile As String)
    Dim s$
    Dim f%
    
    If sFile = "" Then Exit Sub
    If Dir(sFile) = "" Then Exit Sub
    
    f = FreeFile
    Open sFile For Input As #f
    
    While Not EOF(f)
        Line Input #f, s
        s = Trim(s)
        If Left(s, 1) <> ";" And s <> "" Then
            Sleep 50
            sWrite s
        End If
    Wend
    
    Close #f
End Sub

'-------------------------------------
Private Function ReadStatusBit(ByVal WhichBit As Integer, ByRef status() As Integer) As Integer
    Dim s$
    Dim i%
    Const OffSet% = 5

    If sWrite("!TAS") Then
        s = mSerial.Read
        i = -1
        While s <> vbCr
            i = i + 1
            ReDim Preserve status(i)
            s = Replace(s, "_", "")
            status(i) = Mid(s, WhichBit + OffSet, 1)
            
            s = mSerial.Read
        Wend
    End If
    
    ReadStatusBit = i + 1
End Function

Private Sub Class_Terminate()
    EnableDrive -1, False
End Sub
