VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iSeikiD120"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2006 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for Suruga Seiki D120 stepper motor controller

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const GPIBReadTimeout = cwGPIBTimeoutT1s

Private Const DefMaxAxis% = 6
Private Const DefMinMxis% = 1

Private Const bitSuspend = &H80
Private Const bitLimitStop = &H10

Private mGPIB As cwGPIB

Public Function Initialize(ByRef cwGPIB As cwGPIB, _
                           ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$
    
    On Error GoTo ErrHandler
    
    Set mGPIB = cwGPIB
                    
    With mGPIB
        'configure GPIB link
        .DataAsString = True
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = GPIBReadTimeout
        'special for this device
        .EOSChar = vbCr
        .AppendEOSOnWrite = False
        .EOSEndsRead = True
        .EOTMode = True
        .EOIWithEOS = True
        .Unaddressing = True
        .Configure
        'enable remote ctrl
        sWrite "*MODE3"
        
       
    End With
                          
    Initialize = True
    
ErrHandler:
    If RaiseError Then Err.Raise Err.Number, Err.Source, Err.Description
End Function

Public Property Get CurrentPosition(ByVal WhichAxis As Integer) As Long
    Dim sCmd$
    
    sCmd = "A" & WhichAxis & "?S"
    CurrentPosition = QueryDataNumeric(sCmd)
End Property

Public Property Let CurrentPosition(ByVal WhichAxis As Integer, ByVal vData As Long)
    Dim sCmd$
    
    sCmd = "A" & WhichAxis & "#S" & vData
    sWrite sCmd
End Property

Public Property Let DisplayAxis(ByVal WhichAxis As Integer)
    Dim sCmd$
    
    sCmd = "A" & WhichAxis
    sWrite sCmd
End Property

Public Sub Halt()
    sWrite "H"
End Sub

Public Sub Home(ByVal WhichAxis As Integer)
    Dim sCmd$
    
    sCmd = "A" & WhichAxis & "#OG"
    sWrite sCmd
    WaitComplete WhichAxis
    
End Sub

Public Sub MoveAbsolute(ByVal WhichAxis As Integer, ByVal NewPosition As Long)
    Dim sCmd$
    
    'amount
    sCmd = "A" & WhichAxis & "#A" & NewPosition & "G"
    
    'move
    sWrite sCmd
    
    'wait
    WaitComplete WhichAxis
        
End Sub

Public Sub MoveRelative(ByVal WhichAxis As Integer, ByVal Amount As Long)
    Dim sCmd$
    
    'check direction
    sCmd = IIf(Amount > 0, "UG", "DG")
    
    'amount
    sCmd = "A" & WhichAxis & "#P" & Amount & sCmd
    
    'move
    sWrite sCmd
    
    'wait
    WaitComplete WhichAxis
        
End Sub

Public Sub SetVAL(ByVal Axis As Integer, ByVal V As Long, ByVal A As Long, ByVal L As Long)
    Dim sCmd$
    
    sCmd = "A" & Axis & "#F" & V & "R" & A & "L" & L
    sWrite sCmd
End Sub

'----------------------------------------------------------------------------------
Private Sub WaitBit(ByVal BitMask As Long, Optional ByVal TimeOutInSec As Single = 30)
    Dim bStatus&
    Dim MyStart!, Mytime!
    
    On Error GoTo ErrHandler
    
    'do serial pool to see if the command has completed
    MyStart = Timer
    While ((bStatus And BitMask) <> BitMask) And Mytime < TimeOutInSec
        Sleep 10
        DoEvents
        bStatus = mGPIB.SerialPoll
        Mytime = (Timer - MyStart)
    Wend
    
ErrHandler:
    If Err.Number = 30336 Then Err.Number = 0
End Sub

Private Sub WaitComplete(ByVal WhichAxis As Integer, Optional ByVal TimeOutInSec As Single = 30)
    WaitBit WhichAxis, TimeOutInSec
End Sub


Private Function QueryDataNumeric(ByVal sCmd As String) As Single
    Dim s$
    
    s = QueryData(sCmd)
    If IsNumeric(s) Then
        QueryDataNumeric = CSng(s)
    Else
        QueryDataNumeric = -9999
    End If
End Function

Private Function QueryData(ByVal sCmd As String) As String
    Dim s$
    
    sWrite sCmd
    s = mGPIB.Read
    s = Left(s, Len(s) - 1)
    QueryData = s
    
    'do an additional read will somehow clear the buff
    On Error Resume Next
        With mGPIB
            .TimeoutValue = cwGPIBTimeoutT1ms   'put shortest time, just get over with it
            .Configure
            s = .Read
            .TimeoutValue = GPIBReadTimeout
            .Configure
        End With
    On Error GoTo 0

End Function

Private Sub sWrite(ByVal sCmd As String)
    'addpend Cr Lf to the end of command to notify instrument EOI
    sCmd = sCmd + vbCrLf
    mGPIB.Write sCmd
    'wait a bit
    Sleep 100
End Sub

