VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iTunicsPlus"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2002-2003 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'This is a calss for NetTest Tunics Plus Tunable Light Source

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const cstIDN = "TUNICS"

'status bit
Private Const bitOPC& = &H1     'operation compelte
Private Const bitERRC& = &H2    'error in received command
Private Const bitERRV& = &H4    'error in received parameter
Private Const bitLIM& = &H8    'current limit
Private Const bitMAV& = &H10    'message available

Private mGPIB As cwGPIB
Private mEnabled As Boolean

Public Function Initialize(ByRef cwGPIB As cwGPIB, _
                           ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
    Dim s$, sData$()
    
    Set mGPIB = cwGPIB
                          
    With mGPIB
        'configure GPIB link
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT10s
        .Configure
                          
        'get model num
        s = QueryData("*IDN?")
        sData = Split(s, ",")
        s = sData(1)
        If s <> cstIDN Then
            s = "Instrument <" & s & "> is not supported."
            Err.Raise 9000, "NetTest Tunable Light Source", s
        End If
        
        'do common settings
        With Me
            .OutputEnabled = False
            .PowerUnit_mW = True
            .PowerMode = True
            .LineWidthCtrl = False
            .ActiveCavityCtrl = True
        End With
    End With
                          
    Initialize = True
End Function

Public Property Let OutputEnabled(ByVal vData As Boolean)
    Dim sCmd$
    
    mEnabled = vData
    
    sCmd = IIf(mEnabled, "ENABLE", "DISABLE")
    WriteData sCmd
End Property

Public Property Get OutputEnabled() As Boolean
    OutputEnabled = mEnabled
End Property

'wavelength
Public Property Let Wavelength_nm(ByVal vData As Double)
    Dim sCmd$
    sCmd = "L=" & vData
    WriteData sCmd
End Property

Public Property Get Wavelength_nm() As Double
    Wavelength_nm = QueryDataNumeric("L?")
End Property

Public Property Let Frequency_GHz(ByVal vData As Double)
    Dim sCmd$
    sCmd = "F=" & vData
    WriteData sCmd
End Property

Public Property Get Frequency_GHz() As Double
    Frequency_GHz = QueryDataNumeric("F?")
End Property

'power
Public Property Let PowerUnit_mW(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "MW", "DBM")
    WriteData sCmd
End Property

Public Property Let ConstantPower(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "APCON", "APCOFF")
    WriteData sCmd
End Property

Public Property Let PowerMode(ByVal vData As Boolean)
    Me.ConstantPower = True
End Property

Public Property Let Power(ByVal vData As Single)
    Dim sCmd$
    sCmd = "P=" & vData
    WriteData sCmd
End Property

Public Property Get Power() As Single
    Power = QueryDataNumeric("P?")
End Property

Public Property Let Power_mW(ByVal vData As Single)
    Me.PowerUnit_mW = True
    Me.Power = vData
End Property

Public Property Get Power_mW() As Single
    Me.PowerUnit_mW = True
    Power_mW = Me.Power
End Property

Public Property Let Power_dBm(ByVal vData As Single)
    Me.PowerUnit_mW = False
    Me.Power = vData
End Property

Public Property Get Power_dBm() As Single
    Me.PowerUnit_mW = False
    Power_dBm = Me.Power
End Property

Public Property Let LineWidthCtrl(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "CTRLON", "CTRLOFF")
    WriteData sCmd
End Property

Public Property Let ActiveCavityCtrl(ByVal vData As Boolean)
    Dim sCmd$
    
    sCmd = IIf(vData, "ACTCTRLON", "ACTCTRLOFF")
    WriteData sCmd
End Property

Public Property Let Current_mA(ByVal vData As Single)
    Dim sCmd$
    sCmd = "I=" & vData
    WriteData sCmd
End Property

Public Property Get Current_mA() As Single
    Current_mA = QueryDataNumeric("I?")
End Property

'------------------------------------------------------------------
Private Function QueryDataNumeric(ByVal sCmd As String) As Single
    Dim s$
    Dim p%
    
    s = QueryData(sCmd)
    p = InStr(s, "=")
    s = Mid(s, p + 1)
    If IsNumeric(s) Then
        QueryDataNumeric = CSng(s)
    Else
        QueryDataNumeric = -1
    End If
End Function

Private Function QueryData(ByVal sCmd As String) As String
    Dim s$
    
    mGPIB.Write sCmd
    WaitData
    s = mGPIB.Read
    s = Left(s, Len(s) - 1)
    
    QueryData = s
End Function

Private Sub WaitBit(ByVal BitMask As Long, Optional ByVal TimeOutInSec As Single = 60)
    Dim bStatus&
    Dim MyStart!, Mytime!
    
    On Error GoTo ErrHandler
    
    'do serial pool to see if the command has completed
    MyStart = Timer
    While ((bStatus And BitMask) <> BitMask) And Mytime < TimeOutInSec
        Sleep 10   'do not lock up system
        bStatus = mGPIB.SerialPoll
        Mytime = (Timer - MyStart)
    Wend
    
ErrHandler:
    If Err.Number = 30336 Then Err.Number = 0
End Sub

Private Sub WaitComplete(Optional ByVal TimeOutInSec As Single = 60)
    WaitBit bitOPC, TimeOutInSec
End Sub

Private Sub WaitData(Optional ByVal TimeOutInSec As Single = 60)
    WaitBit bitMAV + bitOPC, TimeOutInSec
End Sub

Private Sub WriteData(ByVal sCmd As String)
    mGPIB.Write sCmd
    WaitComplete
End Sub

Private Sub Class_Terminate()
    mGPIB.Reset
End Sub
