VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iWalicsOSA"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2001-2002 W2Laboratories, www.w2lab.com                   ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for NetTest Walics OSA

'wave length limit
Private Const L_Band_LowLimit! = 186.5
Private Const L_Band_HighLimit! = 191.45
Private Const C_Band_LowLimit! = 191.282
Private Const C_Band_HighLimit! = 196.359
Private Const constMinThreshold = -80
Private Const constMaxSMSR = 80

Public Enum iWallicsOSAModeEnum
    osaMode_Graph = 0
    osaMode_WDM = 1
End Enum

Public Enum iWallicsOSASweepEnum
    osaSweep_Single = 0
    osaSweep_Repeat = 1
    osaSweep_Stop = 2
End Enum

Public Enum iWalicsOSAResEnum
    osaRes_Full = 0
    osaRes_01nm = 1
    osaRes_02nm = 2
    osaRes_05nm = 3
    osaRes_1nm = 4
End Enum

Public Enum iWalicsOSAUnitEnum
    osaUnit_nm = 0
    osaUnit_THz = 1
End Enum

Public Enum iWalicsOSASenEnum
    osaSen_Normal = 0
    osaSen_High1 = 1
    osaSen_High2 = 2
End Enum

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

Private Const sWalics$ = "Walics"

Private Const bitMAV& = &H10    'message available
Private Const bitOPC& = &H80    'operation compelte
Private Const bitIDL& = &H1     'sweep finished
Private Const bitErr& = &H10    'exe error

Private mGPIB As cwGPIB

Private mSerialNum As Long
Private mModelName As String
Private mSMSRDataIndex As iWalicsSMSRDataArrayIndex
Private mPeakDataIndex As iWalicsPeakDataArrayIndex

Public Function Initialize(ByRef cwGPIB As cwGPIB, _
                           ByVal BoardNum As Integer, _
                           ByVal InstrAdrs As Integer, _
                           Optional ByVal RaiseError As Boolean) As Boolean
      
    On Error GoTo ErrHandler
    
    Dim s$, sData$()
    
    Set mGPIB = cwGPIB
        
    With mGPIB
        .BoardNumber = BoardNum
        .PrimaryAddress = InstrAdrs
        .TimeoutValue = cwGPIBTimeoutT10s
        .Configure
        '.Clear         <--- this takes a long time for OSA to reset
        
        'get instrument type
        .Write "*IDN?"
        Sleep 500
        'Me.WaitComplete
        s = NiceRead
        sData = Split(s, ",")
        mModelName = sData(1)
        
        Select Case mModelName
            Case sWalics
            Case Else
                s = "Instrument <" & s & "> is not supported."
                Err.Raise 9000, "Wallics OSA", s
        End Select
       
        'get serial number
        If IsNumeric(sData(2)) Then
            mSerialNum = CLng(sData(2))
        Else
            Err.Raise 9001
        End If
                   
    End With
    
    Initialize = True
    Exit Function

ErrHandler:
    If RaiseError Then
        Err.Raise Err.Number, Err.Source, Err.Description
    End If
End Function

Friend Property Get SMSRDataIndex() As iWalicsSMSRDataArrayIndex
    SMSRDataIndex = mSMSRDataIndex
End Property

Friend Property Get PeakDataIndex() As iWalicsPeakDataArrayIndex
    PeakDataIndex = mPeakDataIndex
End Property

'get instrument info
Public Property Let SerialNumber(ByVal vData As Long)
    mSerialNum = vData
End Property

Public Property Get SerialNumber() As Long
    SerialNumber = mSerialNum
End Property

Public Property Get ModelName() As String
    ModelName = mModelName
End Property

Public Property Get NetPath() As String
    NetPath = "\\" & mModelName & "_" & mSerialNum & "\Files\"
End Property

'display mode
Public Property Let DisplayMode(ByVal vData As iWallicsOSAModeEnum)
    Dim sCmd$
    
    Select Case vData
        Case osaMode_Graph:     sCmd = "GRAPH"
        Case osaMode_WDM:       sCmd = "WDM"
    End Select
    
    mGPIB.Write sCmd
End Property

Public Property Get DisplayMode() As iWallicsOSAModeEnum
    Dim sCmd$

    sCmd = "MODE?"
    mGPIB.Write sCmd
    sCmd = NiceRead
    Select Case sCmd
        Case "GRAPH":   DisplayMode = osaMode_Graph
        Case "WDM":     DisplayMode = osaMode_WDM
    End Select
    
End Property

'resolution
Public Property Let Resolution(ByVal vData As iWalicsOSAResEnum)
    Dim sCmd$
    
    Select Case vData
        Case osaRes_Full:   sCmd = "FULL"
        Case osaRes_01nm:   sCmd = "0.1"
        Case osaRes_02nm:   sCmd = "0.2"
        Case osaRes_05nm:   sCmd = "0.5"
        Case osaRes_1nm:    sCmd = "1"
    End Select
    
    sCmd = "RES " & sCmd
    mGPIB.Write sCmd
End Property

Public Property Get Resolution() As iWalicsOSAResEnum
    Dim sCmd$
    
    sCmd = "RES?"
    mGPIB.Write sCmd
    sCmd = NiceRead
    Select Case sCmd
        Case "FULL":    Resolution = osaRes_Full
        Case "0.1":     Resolution = osaRes_01nm
        Case "0.2":     Resolution = osaRes_02nm
        Case "0.5":     Resolution = osaRes_05nm
        Case "1":       Resolution = osaRes_1nm
    End Select
    
End Property

'X Unit
Public Property Let XUnit(ByVal vData As iWalicsOSAUnitEnum)
    Dim sCmd$
    
    Select Case vData
        Case osaUnit_nm:    sCmd = "NM"
        Case osaUnit_THz:   sCmd = "THZ"
    End Select
    
    mGPIB.Write sCmd
End Property

Public Property Get XUnit() As iWalicsOSAUnitEnum
    Dim sCmd$
    
    sCmd = "NM?"
    mGPIB.Write sCmd
    sCmd = NiceRead
    If sCmd = "1" Then
        XUnit = osaUnit_nm
    Else
        XUnit = osaUnit_THz
    End If
End Property

'Y property
Public Property Let Threshold(ByVal vData As Single)
    On Error Resume Next
    Dim sCmd$
    If vData < constMinThreshold Then vData = constMinThreshold
    sCmd = "THRESHOLD " & vData
    mGPIB.Write sCmd
End Property

Public Property Get Threshold() As Single
    Threshold = QueryValue("THRESHOLD?")
End Property

Public Property Let AutoThreshold(ByVal vData As Boolean)
    Dim sCmd$
    sCmd = IIf(vData, "ON", "OFF")
    sCmd = "TH_AUTO " & sCmd
    mGPIB.Write sCmd
End Property

Public Property Get AutoThreshold() As Boolean
    Dim s$
    s = QueryValue("TH_AUTO?")
    AutoThreshold = (s = "ON")
End Property

Public Property Let AutoThDeltaPow(ByVal vData As Single)
    Dim sCmd$
    sCmd = "TH_DP " & vData
    mGPIB.Write sCmd
End Property

Public Property Get AutoThDeltaPow() As Single
    AutoThDeltaPow = QueryValue("TH_DP?")
End Property

Public Property Let AutoThDeltaWL(ByVal vData As Single)
    Dim sCmd$
    sCmd = "TH_DL " & vData
    mGPIB.Write sCmd
End Property

Public Property Get AutoThDeltaWL() As Single
    AutoThDeltaWL = QueryValue("TH_DL?")
End Property

Public Property Let YLogScale(ByVal vData As Boolean)
    Dim sCmd$
    sCmd = IIf(vData, "LOG", "LIN")
    mGPIB.Write sCmd
End Property

Public Property Get YLogScale() As Boolean
    Dim i%
    i = QueryValue("LOG?")
    YLogScale = (i = 1)
End Property

Public Property Let YScale(ByVal vData As Single)
    Dim sCmd$
    sCmd = "DBDIV " & vData
    mGPIB.Write sCmd
End Property

Public Property Get YScale() As Single
    YScale = QueryValue("DBDIV?")
End Property

'scan range
Public Sub Set2FullRange()
    mGPIB.Write "FULL"
End Sub

Public Property Let Center(ByVal vData As Single)
    Dim sCmd$
    sCmd = "CENTER " & vData
    mGPIB.Write sCmd
End Property

Public Property Get Center() As Single
    Center = QueryValue("CENTER?")
End Property

Public Property Let Span(ByVal vData As Single)
    Dim sCmd$
    sCmd = "SPAN " & vData
    mGPIB.Write sCmd
End Property

Public Property Get Span() As Single
    Span = QueryValue("SPAN?")
End Property

Public Property Let ScanStart(ByVal vData As Single)
    Dim sCmd$
    sCmd = "START " & vData
    mGPIB.Write sCmd
End Property

Public Property Get ScanStart() As Single
    ScanStart = QueryValue("START?")
End Property

Public Property Let ScanStop(ByVal vData As Single)
    Dim sCmd$
    sCmd = "END " & vData
    mGPIB.Write sCmd
End Property

Public Property Get ScanStop() As Single
    ScanStop = QueryValue("END?")
End Property

Public Sub Peak2Center()
    Dim sCmd$
    sCmd = "PEAK_CENTER"
    mGPIB.Write sCmd
End Sub

'do scan
Public Property Let SweepMode(ByVal vData As iWallicsOSASweepEnum)
    Dim sCmd$
    
    Select Case vData
        Case osaSweep_Single:   sCmd = "SINGLE"
        Case osaSweep_Repeat:   sCmd = "REPEAT"
        Case osaSweep_Stop:     sCmd = "STOP"
    End Select
    
    mGPIB.Write sCmd
End Property

Public Property Get SweepMode() As iWallicsOSASweepEnum
    Dim i As Integer
    
    i = QueryValue("REPEAT?")
    If i = 1 Then
        SweepMode = osaSweep_Repeat
    Else
        SweepMode = osaSweep_Stop
    End If
End Property

Public Property Let Sensitivity(ByVal vData As iWalicsOSASenEnum)
    Dim sCmd$
    
    Select Case vData
        Case osaSen_Normal:     sCmd = "NORMAL"
        Case osaSen_High1:      sCmd = "HIGH1"
        Case osaSen_High2:      sCmd = "HIGH2"
    End Select
    
    sCmd = "SENSITIVITY " & sCmd
    mGPIB.Write sCmd
    
End Property

Public Property Get Sensitivity() As iWalicsOSASenEnum
    Dim sCmd$
    
    sCmd = "SENSITIVITY?"
    mGPIB.Write sCmd
    sCmd = NiceRead
    
    Select Case sCmd
        Case "NORMAL":  Sensitivity = osaSen_Normal
        Case "HIGH1":   Sensitivity = osaSen_High1
        Case "HIGH2":   Sensitivity = osaSen_High2
    End Select
End Property

Public Function TakeOneSweep(Optional ByVal Center As Single, _
                             Optional ByVal Span As Single) As Boolean
    'get ready
    If Center <> 0 Then Me.Center = Center
    If Span <> 0 Then Me.Span = Span
    
    'clear status bit
    mGPIB.Write "*SRE 0"
    
    'do sweep
    Me.SweepMode = osaSweep_Single
    
    'wait
    Me.WaitSweep
    
    TakeOneSweep = True
End Function

Public Function SearchAndZoom(ByVal sBand As String, ByVal Span As Single) As Single
    Dim fCenter!, fSpan!
    Dim sPeak$()
    
    'get range
    Select Case sBand
        Case "C"
            fCenter = 0.5 * (C_Band_HighLimit + C_Band_LowLimit)
            fSpan = (C_Band_HighLimit - C_Band_LowLimit) * 1.2
        Case "L"
            fCenter = 0.5 * (L_Band_HighLimit + L_Band_LowLimit)
            fSpan = (L_Band_HighLimit - L_Band_LowLimit) * 1.2
    End Select
    
    'set first search range
    Me.TakeOneSweep fCenter, fSpan
    
    'check peak
    Me.GetPeakData sPeak
    
    'do full scan if peak is not found
    If sPeak(mPeakDataIndex.PeakPow) < -10 Then
        Me.Set2FullRange
        Me.TakeOneSweep
        Me.GetPeakData sPeak
    End If
    
    'find 1st order center, zoom out, scan again
    fCenter = sPeak(mPeakDataIndex.PeakWL)
    Me.TakeOneSweep fCenter, Span * 5
    
    'lock
    fCenter = Me.GetPeakWavelength
    Me.Center = fCenter
    Me.Span = Span
    Me.Peak2Center
    
    SearchAndZoom = Me.Center
End Function

Public Function FindCenter(Optional ByVal ScanStart As Single, _
                           Optional ByVal ScanStop As Single) As Single
    Dim oldSen%

    'make scan fast
    oldSen = Me.Sensitivity
    Me.Sensitivity = osaSen_Normal
    
    'set range
    If ScanStart = 0 Or ScanStop = 0 Then
        Me.Set2FullRange
    Else
        Me.ScanStart = ScanStart
        Me.ScanStop = ScanStop
    End If
    
    'do a single scan
    Me.TakeOneSweep
    
    'return
    FindCenter = GetPeakWavelength
    
    'reset
    Me.Sensitivity = oldSen
End Function

'measurement functions
Public Function GetTraceData(ByVal wlStart As Single, ByVal wlStop As Single, _
                             ByRef Points As Long, _
                             ByRef wlData() As Single, ByRef powData() As Single) As Boolean
    Dim sCmd$
    Dim sData$()
    Dim Buff&
    
    Const lLimit = 2
    Const uLimit = 100000
    
    'set limit
    If Points < lLimit Then
        Points = lLimit
    ElseIf Points > uLimit Then
        Points = uLimit
    End If
    'send query
    sCmd = "GRAPH_DATA? " & wlStart & " " & wlStop & " " & Points
    mGPIB.Write sCmd
    'calculate needed input read buffer
    Buff = 20 + 7 * Points      '20 byte for echo, and each data points is 7 byte long
    'wait data
    Me.WaitData
    'read data
    sCmd = mGPIB.Read(Buff)
    Buff = InStr(sCmd, "=")
    If Buff <> 0 Then sCmd = Mid(sCmd, Buff + 1)
    sData = Split(sCmd, ",")
    'return data
    Points = 1 + UBound(sData)
    ReDim wlData(Points - 1)
    ReDim powData(Points - 1)
    For Buff = 0 To Points - 1
        wlData(Buff) = wlStart + Buff * (wlStop - wlStart) / (Points - 1)
        powData(Buff) = CSng(sData(Buff))
    Next Buff
End Function
                             
Public Sub GetPeakData(ByRef PeakDataArray() As String)
    Dim sCmd$
    
    sCmd = "PEAK?"
    mGPIB.Write sCmd
    Me.WaitData
    
    sCmd = NiceRead
    PeakDataArray = Split(sCmd, ",")
    If PeakDataArray(mPeakDataIndex.ValleyPow) < constMinThreshold Then _
       PeakDataArray(mPeakDataIndex.ValleyPow) = constMinThreshold
    
End Sub

Public Sub GetSMSRData(ByRef SMSRDataArray() As String)
    Dim sCmd$
    Dim sData$()
    Dim counter%
    Const NMax% = 5
    
Retry:

    'set method
    sCmd = "FCT_ANALYSIS_METHOD 2PMAX"
    mGPIB.Write sCmd
    
    'query data
    sCmd = "SMSR_MO?"
    mGPIB.Write sCmd
    Me.WaitData
    sCmd = NiceRead
    
    'return
    SMSRDataArray = Split(sCmd, ",")
    
    'cehck
    If SMSRDataArray(mSMSRDataIndex.SMSR) > constMaxSMSR And counter < NMax Then
        Call Me.GetPeakData(sData)
        Me.Threshold = sData(mPeakDataIndex.ValleyPow) + 5
        counter = counter + 1
        GoTo Retry
    End If
    
End Sub

Public Function GetPeakPower() As Single
    Dim sData$()
    
    Call Me.GetPeakData(sData)
    
    GetPeakPower = sData(mPeakDataIndex.PeakPow)
End Function

Public Function GetPeakWavelength() As Single
    Dim sData$()
    
    Call Me.GetPeakData(sData)
    
    GetPeakWavelength = sData(mPeakDataIndex.PeakWL)
End Function

Public Function GetSMSR() As Single
    Dim sData$()
    
    Call Me.GetSMSRData(sData)
    GetSMSR = sData(mSMSRDataIndex.SMSR)
End Function

Public Function GetModeSpacing() As Single
    Dim sData$()
    
    Call Me.GetSMSRData(sData)
    GetModeSpacing = sData(mSMSRDataIndex.ModeSpacing)
End Function

'file
Public Function SaveDisplay2Local(ByVal FileName As String) As Boolean
    Dim sCmd$
    Dim response%
    
    'remove file extension if it is there
    FileName = UCase(FileName)
    If Right(FileName, 4) = ".BMP" Then FileName = Left(FileName, Len(FileName) - 4)
    
    'cear error
    Me.CheckError
    
    'save
    sCmd = "SAVE_BMP " & FileName
    mGPIB.Write sCmd
    Me.WaitComplete
    
    'check overwriting problem
    If Me.CheckError Then
        sCmd = "File " & FileName & " already exist. Do you want to overwrite?"
        response = MsgBox(sCmd, vbQuestion + vbYesNo + vbDefaultButton2)
        If response = vbYes Then
            sCmd = "SAVE0_BMP " & FileName
            mGPIB.Write sCmd
            Me.WaitComplete
            SaveDisplay2Local = True
        End If
        
    'no error, return success
    Else
        SaveDisplay2Local = True
    End If
   
End Function

Public Function SaveDisplay2Network(ByVal FullFileName As String, _
                                    Optional OverWrite As Boolean = True) As Boolean
    Dim sTmp$
    
    On Error GoTo ErrHandler
    
    'delete old file if needed
    If OverWrite Then
        If Dir(FullFileName) <> "" Then Kill FullFileName
    End If
    
    'save file to local first
    sTmp = "tmp_" & Format(Date, "YYMMDD") & "_" & Format(Time, "HHMMSS")
    Me.SaveDisplay2Local sTmp
    
    'move file to network
    sTmp = Me.NetPath & sTmp & ".bmp"
    If Dir(sTmp) <> "" Then
        Name sTmp As FullFileName
    End If
        
    SaveDisplay2Network = True
    
    Exit Function
    
ErrHandler:
    Select Case Err.Number
        Case 52:    sTmp = "Walics OSA is not connected to the network."
        Case 58:    sTmp = "Target file: " & FullFileName & " already exist"
        Case Else:  sTmp = Err.Description
    End Select
    MsgBox sTmp
End Function

'error check
Public Function CheckError() As Boolean
    Dim Error&
    
    Error = QueryValue("*ESR?")
    CheckError = ((Error And bitErr) = bitErr)
    
End Function

'wait functions
Public Sub WaitSweep(Optional ByVal TimeOutInSec As Single = 60)
    WaitBit bitIDL + bitOPC, TimeOutInSec
End Sub

Public Sub WaitComplete(Optional ByVal TimeOutInSec As Single = 60)
    WaitBit bitOPC, TimeOutInSec
End Sub

Public Sub WaitData(Optional ByVal TimeOutInSec As Single = 60)
    WaitBit bitMAV + bitOPC, TimeOutInSec
End Sub

Private Sub WaitBit(ByVal BitMask As Long, Optional ByVal TimeOutInSec As Single = 60)
    Dim bStatus&
    Dim MyStart!, Mytime!
    
    On Error GoTo ErrHandler
    
    'do serial pool to see if the command has completed
    MyStart = Timer
    While ((bStatus And BitMask) <> BitMask) And Mytime < TimeOutInSec
        DoEvents    'do not lock up system
        bStatus = mGPIB.SerialPoll
        Mytime = (Timer - MyStart)
    Wend
    
ErrHandler:
    If Err.Number = 30336 Then Err.Number = 0
End Sub

'nice read,
Private Function QueryValue(ByVal sCmd As String) As Single
    mGPIB.Write sCmd
    Me.WaitComplete
    sCmd = NiceRead
    If IsNumeric(sCmd) Then QueryValue = CSng(sCmd)
End Function

Private Function NiceRead() As String
    Dim i%
    Dim s$
    
    'get rid off EOL character
    s = mGPIB.Read
    s = Left(s, Len(s) - 1)
    
    'strip the header
    i = InStr(1, s, "=")
    If i > 0 Then s = Mid(s, i + 1)
    
    NiceRead = s
End Function


Private Sub Class_Initialize()
    mSMSRDataIndex.MainModeWL = 0
    mSMSRDataIndex.MainModePow = 1
    mSMSRDataIndex.SideModeWL = 2
    mSMSRDataIndex.SideModePow = 3
    mSMSRDataIndex.ModeSpacing = 4
    mSMSRDataIndex.SMSR = 5
    
    mPeakDataIndex.PeakWL = 0
    mPeakDataIndex.PeakPow = 1
    mPeakDataIndex.ValleyWL = 2
    mPeakDataIndex.ValleyPow = 3

End Sub
