VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "iWatlow96"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'----------------------------------------------------------------------------------
'------- (C) 2002 W2Laboratories, www.w2lab.com                        ------------
'------- This software is provided as is                               ------------
'------- While all the programmed functionalities are tested,          ------------
'------- please verify them under your application conditions          ------------
'----------------------------------------------------------------------------------
'----------------------------------------------------------------------------------

'this is a class for Watlow 96 temperature controller
'it uses WAModbus.dll for Modbus communication interface through RS232

'DLL declare
Private Declare Function WA_openVBModbusConnection Lib "WAModbus.DLL" _
    (ByRef hComm As Long, ByVal PortStr As String, _
     ByVal BaudRate As Long, ByVal parity As Integer) As Integer

Private Declare Sub WA_closeModbusConnection Lib "WAModbus.DLL" (ByVal hComm As Long)

Private Declare Function WA_readModbusDevice Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByVal slaveAddress As Integer, _
     ByVal startAddress As Long, ByRef data As Integer, _
     ByVal numPoints As Integer) As Integer
     
Private Declare Function WA_writeModbusDevice Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByVal slaveAddress As Integer, _
     ByVal startAddress As Long, ByRef data As Integer, _
     ByVal numPoints As Integer) As Integer

Private Declare Function WA_readModbusDiagnostics Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByVal slaveAddress As Integer, _
     ByVal subFunction As Integer, ByRef data As Integer) As Integer

Private Declare Function WA_getReadString Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByRef data As Byte) As Integer
    
Private Declare Function WA_getWriteString Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByRef data As Byte) As Integer
    
Private Declare Function WA_getReadResponse Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByRef data As Byte) As Integer

Private Declare Function WA_getWriteResponse Lib "WAModbus.DLL" _
    (ByVal hComm As Long, ByRef data As Byte) As Integer


'internal data
Private Const MyName As String = "Watlow Serial 96 Thermal Controller"
Private Const mBaseAdrs As Long = 40001   'address offset for reg.
Private Const mOffTemp As Single = -32768

Private mhComm As Long          'port handler
Private mSlaveAdrs As Integer   'slave device address

'data type for step
Public Enum wt96StepEnum
    wtRamp = 0
    wtSoak = 1
    'wtJump = 2
    wtEnd = 3
End Enum


Public Function Initialize(ByVal CommPortNum As Integer, ByVal BaudRate As Integer, _
                           ByVal SlaveAdrs As Integer) As Boolean
    
    Dim result%
    Dim PortStr$
    Dim sErrorMsg$
        
    'check port num
    Select Case CommPortNum
        Case 1 To 32
            PortStr$ = "COM" & CommPortNum
        Case Else
            sErrorMsg = "Invalid comm port number: " & CommPortNum
            GoTo ErrHandler
    End Select
    
    'check buad rate
    Select Case BaudRate
        Case 1200, 2400, 4800, 9600, 19200
            'do nothing
        Case Else
            sErrorMsg = "Invalid baud rate: " & BaudRate
            GoTo ErrHandler
    End Select
    
    'check slave address
    Select Case SlaveAdrs
        Case 1 To 247
            mSlaveAdrs = SlaveAdrs
        Case Else
            sErrorMsg = "Invalid slave address for device: " & SlaveAdrs
            GoTo ErrHandler
    End Select
    
    'open port
    result = WA_openVBModbusConnection(mhComm, PortStr, BaudRate, 0&)
    
    'check error
    If result <> 0 Then
        sErrorMsg = "Can not open port for " & MyName
        GoTo ErrHandler
    End If
    
    'check if data can be written
    If Not WriteModbus(901, 1) Then
        sErrorMsg = "Can not open port for " & MyName
        GoTo ErrHandler
    End If
    
    'set unit
    Me.DisplayDegC = True
    
    'make setpoint low
    Me.DisableOutput
    
    'ramp is time based
    Call Me.WriteModbus(1208, 0)
    
    'use PID unit type
    Call Me.WriteModbus(900, 2)

    Initialize = True
    Exit Function
    
ErrHandler:
    MsgBox sErrorMsg, vbExclamation + vbOKOnly, MyName
End Function

Public Sub ClosePort()
    WA_closeModbusConnection mhComm
End Sub


'DisplayUnit
Public Property Let DisplayDegC(ByVal vData As Boolean)
    Dim i%
    i = IIf(vData, 1, 0)
    Call Me.WriteModbus(901, i)
End Property

'read actual temperature
Public Function ReadTemperature() As Single
    ReadTemperature = Me.ReadModbus(100)
End Function

'status
Public Property Get InProgress() As Boolean
    Dim i%
     
    i = Me.ReadModbus(5013)     'read profile state
    InProgress = (i <> 0)        '2 for running
    
End Property

'disable
Public Sub DisableOutput()
    'Call Me.WriteModbus(4000, 1)    'start file 1, the one get programmed
    Me.SetPoint = mOffTemp
End Sub

'start, stop, hold
Public Sub StartRamp()
    Call Me.WriteModbus(4000, 1)    'start file 1, the one get programmed
    Call Me.WriteModbus(4001, 1)    'stare step 1, ramp step
    Call Me.WriteModbus(4002, 5)    'start profile now
End Sub

Public Sub StopRamp()
    'hold ramp first
    Me.HoldRamp
    'go and start the end step
    Call Me.WriteModbus(4000, 1)    'start file 1, the one get programmed
    Call Me.WriteModbus(4001, 8)    'stare step 8, end step step
    Call Me.WriteModbus(4002, 5)    'start profile now
End Sub

Public Sub HoldRamp()
    Call Me.WriteModbus(1210, 1)
End Sub

Public Sub ResumeRamp()
    Call Me.WriteModbus(1209, 1)
End Sub

'set point - deg c
Public Property Let SetPoint(ByVal vData As Single)
    Call Me.WriteModbus(300, vData)
End Property

Public Property Get SetPoint() As Single
    SetPoint = Me.ReadModbus(300)
End Property

'thermal profile
Public Function BuildProfile(ByVal iStepNum As Integer, ByVal eStepType As wt96StepEnum, _
                             Optional ByVal EndSetPoint As Single = -1, _
                             Optional ByVal Time_min As Single = -1) As Boolean
               
    Const nPeriod% = 13
    Dim i%
    Dim h%
    Dim m%
    Dim s%
    
    'check step
    i = iStepNum - 1
    If i < 0 Or i > 7 Then Exit Function
    
    'add offset
    i = i * nPeriod
                        
    'set type
    Call Me.WriteModbus(5020 + i, eStepType)
    If eStepType = wtEnd Then Call Me.WriteModbus(5032 + i, 1)  'end end, not end hold
    
    'set set point
    If EndSetPoint <> -1 Then Call Me.WriteModbus(5021 + i, EndSetPoint)
    
    'time
    If Time_min <> -1 Then
        'minute in integer
        m = Fix(Time_min)
        'hour
        h = m \ 60
        'sec
        s = (Time_min - m) * 60
        'min besides hour
        m = m - h * 60
        Call Me.WriteModbus(5022 + i, h)
        Call Me.WriteModbus(5023 + i, m)
        Call Me.WriteModbus(5024 + i, s)
    End If
                  
End Function

'PID, ProportionalBand - DegC
Public Property Let ProportionalBand(ByVal vData As Single)
    Call Me.WriteModbus(500, vData)
End Property

Public Property Get ProportionalBand() As Single
    ProportionalBand = Me.ReadModbus(500)
End Property

'PID, Integral Time, minute
Public Property Let IntegralTime(ByVal vData As Single)
    Call Me.WriteModbus(501, vData * 100!)
End Property

Public Property Get IntegralTime() As Single
    IntegralTime = Me.ReadModbus(501) / 100!
End Property

'PID Derivate time, min
Public Property Let DerivateTime(ByVal vData As Single)
    Call Me.WriteModbus(503, vData * 100!)
End Property

Public Property Get DerivateTime() As Single
    DerivateTime = Me.ReadModbus(503) / 100!
End Property

Public Property Let CycleTime(ByVal vData As Single)
    Call Me.WriteModbus(506, vData)
End Property

Public Property Get CycleTime() As Single
    CycleTime = Me.ReadModbus(506)
End Property

Public Property Let DAC(ByVal vData As Single)
    Call Me.WriteModbus(505, vData)
End Property

Public Property Get DAC() As Single
    DAC = Me.ReadModbus(505)
End Property

'generic read write
Public Function ReadModbus(ByVal AdrsReg As Integer) As Integer
    Dim Adrs&
    Dim data%
    Dim result%
    
    Adrs = GetAdrsLong(AdrsReg)
    
    result = WA_readModbusDevice(mhComm, mSlaveAdrs, Adrs, data, 1)
    If result = 0 Then
        ReadModbus = data
    Else
        'MsgBox GetErrorStr(result, Data), vbCritical + vbOKOnly, MyName
        ReadModbus = -999
    End If
    
End Function

Public Function WriteModbus(ByVal AdrsReg As Integer, ByVal data As Integer) As Boolean
    Dim Adrs&
    Dim result%
    
    Adrs = GetAdrsLong(AdrsReg)
    
    result = WA_writeModbusDevice(mhComm, mSlaveAdrs, Adrs, data, 1)
'    If result <> 0 Then
'        'MsgBox GetErrorStr(result, Data), vbCritical + vbOKOnly, MyName
'    End If
    WriteModbus = (result = 0)
End Function

Private Function GetAdrsLong(ByVal Adrs As Integer) As Long
'    GetAdrsLong = mBaseAdrs + CLng("&H" + AdrsInHex)
    GetAdrsLong = mBaseAdrs + Adrs
End Function

Private Function GetErrorStr(val%, dataVal%)
    Select Case (val)
        Case -6: GetErrorStr = "HOST_REJECTED_DATA_LENGTH"
        Case -5: GetErrorStr = "HOST_REJECTED_START_ADDRESS"
        Case -4: GetErrorStr = "HOST_RECEIVED_IO_ERR_SENDING"
        Case -3: GetErrorStr = "HOST_RECEIVED_MODBUS_ERROR"
        Case -2: GetErrorStr = "HOST_RECEIVED_TIMEOUT_ERROR"
        Case -1: GetErrorStr = "HOST_RECEIVED_CHECKSUM_ERROR"
    End Select
        
    Select Case dataVal
        Case -2: GetErrorStr = "HOST_RECEIVED_TIMEOUT_ERROR"
        Case -1: GetErrorStr = "HOST_RECEIVED_CHECKSUM_ERROR"
    End Select
End Function

Private Sub Class_Terminate()
    Me.ClosePort
End Sub


