VERSION 5.00
Begin VB.UserControl svcBertan 
   BackColor       =   &H00FFFFC0&
   ClientHeight    =   480
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   480
   Picture         =   "svcBertan.ctx":0000
   ScaleHeight     =   480
   ScaleWidth      =   480
   ToolboxBitmap   =   "svcBertan.ctx":030A
End
Attribute VB_Name = "svcBertan"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Public Enum svcMeasurementType
    mDCV = 0
    mDCI = 1
End Enum

Dim m_Error As Boolean

Dim m_eStr As String
Const m_def_estr = ""

Dim m_VoltageLimit As Single
Const m_def_VoltageLimit = "100000"

'Default Property Values:
Const m_def_DeviceName = "Bertan 225/226 High Voltage Power Supply"
Const m_def_BoardAddress = 0
Const m_def_DeviceAddress = 1

'Property Variables:
Dim m_DeviceName As String
Dim m_BoardAddress As Integer
Dim m_DeviceAddress As Integer

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=7,0,0,0
Public Property Get BoardAddress() As Integer
    BoardAddress = m_BoardAddress
End Property

Public Property Let BoardAddress(ByVal New_BoardAddress As Integer)
    m_BoardAddress = New_BoardAddress
    PropertyChanged "BoardAddress"
End Property

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=7,0,0,1
Public Property Get DeviceAddress() As Integer
    DeviceAddress = m_DeviceAddress
End Property

Public Property Let DeviceAddress(ByVal New_DeviceAddress As Integer)
    m_DeviceAddress = New_DeviceAddress
    PropertyChanged "DeviceAddress"
End Property

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=13,0,0,""
Public Property Get DeviceName() As String
    DeviceName = m_DeviceName
End Property

Public Property Let DeviceName(ByVal New_DeviceName As String)
    m_DeviceName = New_DeviceName
    PropertyChanged "DeviceName"
End Property

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=5
Public Sub Initialize()
    DevClear m_BoardAddress, m_DeviceAddress
    WriteCommand "OE2"
    WriteCommand "OC1"
End Sub

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=5
Public Sub IEEEOut(ByVal cmd As String)
     WriteCommand cmd
End Sub

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=14
Public Function Measure(ByVal MeasureWhat As svcMeasurementType) As Variant
    Dim cmd$, mea$
   
    Select Case MeasureWhat
        Case mDCV:        cmd = "T1"
        Case mDCI:        cmd = "T2"
    End Select
    
    WriteCommand cmd
    mea = GetValue
    
    If Left(mea, 1) = "N" Then
        Select Case Mid(mea, 3, 1)
            Case "V"
                Measure = CSng(Mid(mea, 4, 6)) * 1000#
            Case "I"
                If Right(mea, 1) = "M" Then
                    Measure = CSng(Mid(mea, 4, 6)) * 0.001
                Else
                    Measure = CSng(Mid(mea, 4, 6)) * 0.000001
                End If
        End Select
        Measure = Format(Measure, "Scientific")
    Else
        Measure = 0
    End If
    
End Function

Public Property Get ValueIn() As Variant
    ValueIn = GetValue
End Property

Public Sub Enable()
    m_eStr = "G"
End Sub

Public Sub Disable()
    m_eStr = ""
    WriteCommand "P0.0000KG"
End Sub

Public Sub Halt()
    WriteCommand "Z"
End Sub

Public Sub ResumeAfterHalt()
    WriteCommand "R"
End Sub

Public Sub SetVoltage(ByVal voltage As Single)
    Dim vStr$
    
    If voltage > m_VoltageLimit Then Err.Raise 9002, "Bertan", "Voltage higher than the set limit."
    voltage = voltage / 1000#
    
    Select Case voltage
        Case Is < 1:    vStr = Format(voltage, "0.00000")
        Case Is < 10:   vStr = Format(voltage, "0.0000")
        Case Is < 100:  vStr = Format(voltage, "00.000")
        Case Else:      Err.Raise 9002, "Bertan", "Set voltage to high."
    End Select
    
    WriteCommand "P" & vStr & "K" & m_eStr
    
End Sub

Public Sub SetVoltageLimit(ByVal voltage As Single)
    Dim vStr$
    
    m_VoltageLimit = voltage
    voltage = voltage / 1000#
    
    Select Case voltage
        Case Is < 1:    vStr = Format(voltage, "0.00000")
        Case Is < 10:   vStr = Format(voltage, "0.0000")
        Case Is < 100:  vStr = Format(voltage, "00.000")
        Case Else:      Err.Raise 9002, "Bertan", "Voltage limit out of the allowed limit."
    End Select
    
    WriteCommand "L" & vStr & "K" & m_eStr

End Sub

Public Sub SetCurrentLimit(ByVal current As Single)
    Dim Str$
     
    Select Case current
        'Case Is < 0.001:    Str = Format(current * 1000000#, "000.00") & "UG"
        Case Is < 0.01:     Str = Format(current * 1000#, "0.0000") & "MG"
        Case Is < 0.1:      Str = Format(current * 1000#, "00.000") & "MG"
        Case Else:          Err.Raise 9002, "Bertan", "Current limit out of the allowed limit."
    End Select
    
    WriteCommand "L" & Str

End Sub

Private Sub UserControl_Initialize()
     m_VoltageLimit = m_def_VoltageLimit
     m_eStr = m_def_estr
End Sub

'Initialize Properties for User Control
Private Sub UserControl_InitProperties()
    m_BoardAddress = m_def_BoardAddress
    m_DeviceAddress = m_def_DeviceAddress
    m_DeviceName = m_def_DeviceName
   
End Sub

'Load property values from storage
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)

    m_BoardAddress = PropBag.ReadProperty("BoardAddress", m_def_BoardAddress)
    m_DeviceAddress = PropBag.ReadProperty("DeviceAddress", m_def_DeviceAddress)
    m_DeviceName = PropBag.ReadProperty("DeviceName", m_def_DeviceName)
    
End Sub

Private Sub UserControl_Resize()
    Width = 480
    Height = 480
End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)

    Call PropBag.WriteProperty("BoardAddress", m_BoardAddress, m_def_BoardAddress)
    Call PropBag.WriteProperty("DeviceAddress", m_DeviceAddress, m_def_DeviceAddress)
    Call PropBag.WriteProperty("DeviceName", m_DeviceName, m_def_DeviceName)
   
End Sub

Private Sub WriteCommand(ByVal cmd As String)
    
    Send m_BoardAddress, m_DeviceAddress, cmd, DABend
    Do Until (ibsta And CMPL) Or (ibsta And EERR)
        DoEvents
    Loop
    CheckError
    If m_Error Then Err.Raise 9001, "GPIB Error", ErrDesp
End Sub

Private Function GetValue() As Variant
    Dim p%
    Dim Data As String * 100
    
    GetValue = ""
    
    If Not m_Error Then
        Call Receive(m_BoardAddress, m_DeviceAddress, Data, STOPend)
        CheckError
        If Not m_Error Then
            p = InStr(Data, Chr(10))
            If p > 0 Then GetValue = Left(Data, p - 1)
            p = InStr(Data, Chr(13))
            If p > 0 Then GetValue = Left(Data, p - 1)
        End If
    End If
    
    If m_Error Then Err.Raise 9001, "GPIB Error", ErrDesp
    
End Function

'Error handling
Private Sub CheckError()
    m_Error = GPIBError(m_DeviceName, m_BoardAddress, m_DeviceAddress)
End Sub
