VERSION 5.00
Begin VB.UserControl svcK487 
   ClientHeight    =   825
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   750
   InvisibleAtRuntime=   -1  'True
   Picture         =   "svcK487.ctx":0000
   ScaleHeight     =   825
   ScaleWidth      =   750
   ToolboxBitmap   =   "svcK487.ctx":030A
End
Attribute VB_Name = "svcK487"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Public Enum svcMeasurementType
    mDCV = 0
    mDCI = 1
    mResistance = 5
End Enum

Public Enum K487CurrentLimit
    K487Imax_25uA = 0
    K487Imax_2500uA = 1
End Enum

Public Enum K487CurrentRange
    K487R_Auto = 0
    K487R_2nA = 1
    K487R_20nA = 2
    K487R_200nA = 3
    K487R_2uA = 4
    K487R_20uA = 5
    K487R_200uA = 6
    K487R_2mA = 7
End Enum

Private m_Error As Boolean
Private m_487 As String

'Default Property Values:
Const m_def_DeviceName = "Keithley 486/487 Picoammeter"
Const m_def_BoardAddress = 0
Const m_def_DeviceAddress = 20

Private m_BoardAddress%
Private m_DeviceAddress%
Private m_DeviceName$


'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=7,0,0,0
Public Property Get BoardAddress() As Integer
    BoardAddress = m_BoardAddress
End Property

Public Property Let BoardAddress(ByVal New_BoardAddress As Integer)
    m_BoardAddress = New_BoardAddress
    PropertyChanged "BoardAddress"
End Property

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=7,0,0,1
Public Property Get DeviceAddress() As Integer
    DeviceAddress = m_DeviceAddress
End Property

Public Property Let DeviceAddress(ByVal New_DeviceAddress As Integer)
    m_DeviceAddress = New_DeviceAddress
    PropertyChanged "DeviceAddress"
End Property

'WARNING! DO NOT REMOVE OR MODIFY THE FOLLOWING COMMENTED LINES!
'MemberInfo=13,0,0,
Public Property Get DeviceName() As String
    DeviceName = m_DeviceName
End Property

Public Property Let DeviceName(ByVal New_DeviceName As String)
    m_DeviceName = New_DeviceName
    PropertyChanged "DeviceName"
End Property

Public Property Get ValueIn() As Variant
    ValueIn = GetValue
End Property


Public Sub Initialize()
        
    DevClear m_BoardAddress, m_DeviceAddress
    WriteCommand "L0X"
    'disbale zeor check
    WriteCommand "C0X"
    'trig on talk
    WriteCommand "T1X"
    'check instr type
    WriteCommand "U0X"
    m_487 = GetValue
    m_487 = Left(m_487, 3)
   
End Sub

Public Sub Reset()
    WriteCommand "LOX"
End Sub

Public Function SelfTest() As Integer
    Dim errSta$
    
    SelfTest = 1
    
    WriteCommand "J0X"
    WriteCommand "U1X"
    If Not m_Error Then
        errSta$ = GetValue
        If Mid(errSta, 7, 1) = "0" Then
            SelfTest = 0
        End If
    End If
    
End Function

Public Sub ZeroCheck()
    WriteCommand "C2X"
    DelayFor 2000
    WriteCommand "C0X"
End Sub

Public Sub IEEEOut(ByVal cmd As String)
     WriteCommand cmd
End Sub

Public Sub EnableVOut()
    Check487
    WriteCommand "V0,0,X"
    WriteCommand "O1X"
End Sub

Public Sub DsiableVout()
    Check487
    WriteCommand "V0,0,X"
    WriteCommand "O0X"
End Sub

Public Sub HaltVout()
    Check487
    WriteCommand "O0X"
End Sub

Public Sub ResumeAfterHalt()
    Check487
    WriteCommand "O1X"
End Sub

Public Function SetVoltage(ByVal Voltage As Single) As Integer
    Dim vStr$
    
    Check487
    'set voltage
    vStr = "V" & Voltage & ","
    Select Case Abs(Voltage)
        Case Is < 50:   vStr = vStr & "0,X"
        Case Is < 505:  vStr = vStr & "1,X"
        Case Else:      Err.Raise 9022, DeviceName, "Set voltage higher than the meter can output."
    End Select
    WriteCommand vStr
    
    'check current limit
    WriteCommand "U9X"
    vStr = GetValue
    SetVoltage = CInt(Mid(vStr, 4, 1))
    
End Function

Public Sub SetCurrentLimit(ByVal Current As K487CurrentLimit)
    Check487
    WriteCommand "V,," & Current & "X"
End Sub

Public Sub SetCurrentRange(ByVal Range As K487CurrentRange)
    
    WriteCommand "R" & Range & "X"
End Sub

Public Function Measure(ByVal MeasureWhat As svcMeasurementType) As Variant
    Dim data$
   
    If MeasureWhat <> mDCI Then Check487
   
    Select Case MeasureWhat
        Case mDCI
            data = GetValue
            data = Mid(data, 5)
        Case mDCV
            WriteCommand "U8X"
            data = GetValue
            data = Mid(data, 4, 11)
        Case mResistance
            WriteCommand "F1U7X"
            data = GetValue
            data = Mid(data, 4, 12)
            WriteCommand "F0X"
    End Select
        
    Measure = data
    
End Function

'Initialize Properties for User Control
Private Sub UserControl_InitProperties()
    m_BoardAddress = m_def_BoardAddress
    m_DeviceAddress = m_def_DeviceAddress
    m_DeviceName = m_def_DeviceName
End Sub

'Load property values from storage
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)

    m_BoardAddress = PropBag.ReadProperty("BoardAddress", m_def_BoardAddress)
    m_DeviceAddress = PropBag.ReadProperty("DeviceAddress", m_def_DeviceAddress)
    m_DeviceName = PropBag.ReadProperty("DeviceName", m_def_DeviceName)
End Sub

Private Sub UserControl_Resize()
    Width = 480
    Height = 480
End Sub

Private Sub UserControl_Terminate()
    ReDim m_CardType(0)
End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)

    Call PropBag.WriteProperty("BoardAddress", m_BoardAddress, m_def_BoardAddress)
    Call PropBag.WriteProperty("DeviceAddress", m_DeviceAddress, m_def_DeviceAddress)
    Call PropBag.WriteProperty("DeviceName", m_DeviceName, m_def_DeviceName)
End Sub

Private Sub WriteCommand(ByVal cmd As String)
    
    Send m_BoardAddress, m_DeviceAddress, cmd, DABend
    Do Until (ibsta And CMPL) Or (ibsta And EERR)
        DoEvents
    Loop
    CheckError
    If m_Error Then Err.Raise 9001, "GPIB Error", ErrDesp
End Sub

Private Function GetValue() As Variant
    Dim p%
    Dim data As String * 100
    
    GetValue = ""
    
    If Not m_Error Then
        Call Receive(m_BoardAddress, m_DeviceAddress, data, STOPend)
        CheckError
        If Not m_Error Then
            p = InStr(data, Chr(13))
            If p > 0 Then GetValue = Left(data, p - 1)
        End If
    End If
    
    If m_Error Then Err.Raise 9001, "GPIB Error", ErrDesp
    
End Function

Private Sub CheckError()
    m_Error = GPIBError(m_DeviceName, m_BoardAddress, m_DeviceAddress)
End Sub

Private Sub Check487()
    If m_487 <> "487" Then _
        Err.Raise 9011, DeviceName, "Selected option is only available for Model 487."
End Sub

Private Sub DelayFor(ByVal ms As Long)
    Dim Tf#
    
    Tf = Timer + ms / 1000
    If Tf > 86400# Then Tf = Tf - 86400#     'mid night correction
    Do Until Timer > Tf
        DoEvents
    Loop
    
End Sub
